@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: NMfit.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: NMfit.h 322 2007-07-24 00:17:11Z wojdyr $
 
 #ifndef FITYK__NMFIT__H__
 #define FITYK__NMFIT__H__
@@ -25,7 +25,7 @@ struct Vertex
 class NMfit : public Fit
 {
 public:
-    NMfit();
+    NMfit(Ftk* F);
     ~NMfit();
     fp init(); // called before autoiter()
     void autoiter();
