@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) 2005 Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: ast.cpp 294 2007-05-16 03:18:25Z wojdyr $
+// $Id: ast.cpp 381 2008-01-23 15:22:19Z wojdyr $
 
 //  Based on ast_calc example from Boost::Spirit by Daniel Nuffer
 
@@ -63,12 +63,15 @@ string OpTree::str(const vector<string> *vars)
         case OP_EXP:  return "exp(" + c1->str(vars) + ")";
         case OP_ERFC: return "erfc(" + c1->str(vars) + ")";
         case OP_ERF:  return "erf(" + c1->str(vars) + ")";
+        case OP_SINH:  return "sinh(" + c1->str(vars) + ")";
+        case OP_COSH:  return "cosh(" + c1->str(vars) + ")";
+        case OP_TANH: return "tanh("+ c1->str(vars) + ")";
         case OP_SIN:  return "sin(" + c1->str(vars) + ")";
         case OP_COS:  return "cos(" + c1->str(vars) + ")";
-        case OP_ATAN: return "atan("+ c1->str(vars) + ")";
         case OP_TAN:  return "tan(" + c1->str(vars) + ")";
         case OP_ASIN: return "asin("+ c1->str(vars) + ")";
         case OP_ACOS: return "acos("+ c1->str(vars) + ")";
+        case OP_ATAN: return "atan("+ c1->str(vars) + ")";
         case OP_LGAMMA: return "lgamma("+ c1->str(vars) + ")";
         case OP_DIGAMMA: return "digamma("+ c1->str(vars) + ")";
         case OP_LOG10:return "log10("+c1->str(vars) + ")";
@@ -106,12 +109,15 @@ string OpTree::ascii_tree(int width, int start, const vector<string> *vars)
             case OP_EXP:  node = "EXP";  break;
             case OP_ERFC: node = "ERFC"; break;
             case OP_ERF:  node = "ERF";  break;
+            case OP_SINH: node = "SINH"; break;
+            case OP_COSH: node = "COSH"; break;
+            case OP_TANH: node = "TANH"; break;
             case OP_SIN:  node = "SIN";  break;
             case OP_COS:  node = "COS";  break;
-            case OP_ATAN: node = "ATAN"; break;
             case OP_TAN:  node = "TAN";  break;
             case OP_ASIN: node = "ASIN"; break;
             case OP_ACOS: node = "ACOS"; break;
+            case OP_ATAN: node = "ATAN"; break;
             case OP_LOG10:node = "LOG";  break;
             case OP_LN:   node = "LN";   break;
             case OP_SQRT: node = "SQRT"; break;
@@ -385,6 +391,39 @@ OpTree* do_ln(OpTree *a)
         return new OpTree(OP_LN, simplify_terms(a));
 }
 
+OpTree* do_sinh(OpTree *a)
+{
+    if (a->op == 0) {
+        double val = sinh(a->val);
+        delete a;
+        return new OpTree(val);
+    }
+    else
+        return new OpTree(OP_SINH, simplify_terms(a));
+}
+
+OpTree* do_cosh(OpTree *a)
+{
+    if (a->op == 0) {
+        double val = cosh(a->val);
+        delete a;
+        return new OpTree(val);
+    }
+    else
+        return new OpTree(OP_COSH, simplify_terms(a));
+}
+
+OpTree* do_tanh(OpTree *a)
+{
+    if (a->op == 0) {
+        double val = tanh(a->val);
+        delete a;
+        return new OpTree(val);
+    }
+    else
+        return new OpTree(OP_TANH, simplify_terms(a));
+}
+
 OpTree* do_sin(OpTree *a)
 {
     if (a->op == 0) {
@@ -823,7 +862,7 @@ fp get_constant_value(string const &s)
             string in_expr(expr, in_pos+4);
             int n;
             if (parse(in_expr.c_str(), 
-                      *ch_p(' ') >> '@' >> uint_p[assign_a(n)] >> *ch_p(' ')
+                      !space_p >> '@' >> uint_p[assign_a(n)] >> !space_p
                      ).full) {
                 data = AL->get_data(n);
                 expr.resize(in_pos);
@@ -909,6 +948,18 @@ vector<OpTree*> calculate_deriv(const_tm_iter_t const &i,
                 der = do_oneover(larg);
                 do_op = do_ln;
             }
+            else if (s == "sinh") {
+                der = do_cosh(larg);
+                do_op = do_sinh;
+            }
+            else if (s == "cosh") {
+                der = do_sinh(larg);
+                do_op = do_cosh;
+            }
+            else if (s == "tanh") {
+                der = do_oneover(do_sqr(do_cosh(larg)));
+                do_op = do_tanh;
+            }
             else if (s == "sin") {
                 der = do_cos(larg);
                 do_op = do_sin;
@@ -1065,7 +1116,7 @@ vector<OpTree*> calculate_deriv(const_tm_iter_t const &i,
 string get_derivatives_str(string const &formula)
 {
     string s;
-    tree_parse_info<> info = ast_parse(formula.c_str(), FuncG, space_p);
+    tree_parse_info<> info = ast_parse(formula.c_str(), FuncG>>end_p, space_p);
     if (!info.full)
         throw ExecuteError("Can't parse formula: " + formula);
     const_tm_iter_t const &root = info.trees.begin();
