@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: ast.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: ast.h 349 2007-08-30 03:04:36Z wojdyr $
 
 #ifndef FITYK__AST__H__
 #define FITYK__AST__H__
@@ -23,12 +23,15 @@ enum
     OP_EXP,       
     OP_ERFC,
     OP_ERF,
+    OP_SINH,       
+    OP_COSH,       
+    OP_TANH,       
     OP_SIN,       
     OP_COS,       
-    OP_ATAN,      
     OP_TAN,       
     OP_ASIN,      
     OP_ACOS,      
+    OP_ATAN,      
     OP_LOG10,     
     OP_LN,        
     OP_SQRT,      
