@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) 2005 Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: calc.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: calc.cpp 349 2007-08-30 03:04:36Z wojdyr $
 
 #include "calc.h"
 #include "ast.h"
@@ -131,6 +131,15 @@ void AnyFormula::exec_vm_op_action(vector<int>::const_iterator &i,
             case OP_LN:
                 *stackPtr = log(*stackPtr); 
                 break;
+            case OP_SINH:
+                *stackPtr = sinh(*stackPtr);
+                break;
+            case OP_COSH:
+                *stackPtr = cosh(*stackPtr);
+                break;
+            case OP_TANH:
+                *stackPtr = tanh(*stackPtr); 
+                break;
             case OP_SIN:
                 *stackPtr = sin(*stackPtr);
                 break;
