@@ -1,13 +1,13 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: gnuplot.cpp 322 2007-07-24 00:17:11Z wojdyr $
+// $Id: gnuplot.cpp 376 2008-01-08 13:18:16Z wojdyr $
 
 // CLI-only file
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <fstream>
+//#include <fstream>
 #include <string>
 #include <sys/types.h>
 #include <signal.h>
@@ -97,7 +97,8 @@ bool GnuPlot::gnuplot_pipe_ok()
 int GnuPlot::plot() 
 {
     // plot only active data with sum
-    DataWithSum const* ds = ftk->get_ds(ftk->get_active_ds_position());
+    int ds_number = ftk->view.get_datasets()[0];
+    DataWithSum const* ds = ftk->get_ds(ds_number);
     Data const* data = ds->get_data();
     Sum const* sum = ds->get_sum();
     if (!gnuplot_pipe_ok())
