@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: cmd2.cpp 297 2007-05-25 02:33:19Z wojdyr $
+// $Id: cmd2.cpp 419 2008-04-12 14:51:54Z wojdyr $
 
 /// big grammars in Spirit take a lot of time and memory to compile
 /// so they must be splitted into separate compilation units
@@ -13,6 +13,7 @@
 #include <boost/spirit/utility/chset.hpp>
 #include <boost/spirit/utility/chset_operators.hpp>
 #include <math.h>
+#include <xylib/xylib.h> //get_version()
 
 #include "cmd2.h"
 #include "optional_suffix.h"
@@ -53,25 +54,46 @@ bool outdated_plot = false;
 bool no_info_output = false;
 
 
-vector<DataWithSum*> get_datasets_from_indata()
+vector<int> get_ds_indices_from_indata()
 {
-    vector<DataWithSum*> result;
+    vector<int> result;
+    // no datasets specified
     if (vds.empty()) {
         if (AL->get_ds_count() == 1)
-            result.push_back(AL->get_ds(0));
+            result.push_back(0);
         else
             throw ExecuteError("Dataset must be specified (eg. 'in @0').");
     }
+    // @*
     else if (vds.size() == 1 && vds[0] == all_datasets)
         for (int i = 0; i < AL->get_ds_count(); ++i)
-            result.push_back(AL->get_ds(i));
+            result.push_back(i);
+    // general case
     else
         for (vector<int>::const_iterator i = vds.begin(); i != vds.end(); ++i)
-            result.push_back(AL->get_ds(*i));
+            if (*i == all_datasets) {
+                for (int j = 0; j < AL->get_ds_count(); ++j) {
+                    if (!contains_element(result, j))
+                        result.push_back(j);
+                }
+                return result;
+            }
+            else
+                result.push_back(*i);
+    return result;
+}
+
+vector<DataWithSum*> get_datasets_from_indata()
+{
+    vector<int> indices = get_ds_indices_from_indata();
+    vector<DataWithSum*> result(indices.size());
+    for (size_t i = 0; i < indices.size(); ++i)
+        result[i] = AL->get_ds(indices[i]);
     return result;
 }
 
 IntRangeGrammar  IntRangeG;
+CompactStrGrammar  CompactStrG;
 
 } //namespace cmdgram
 
@@ -81,22 +103,118 @@ namespace {
 
 void do_import_dataset(char const*, char const*)
 {
-    AL->import_dataset(tmp_int, t, t2, vn);
+    if (t == ".") {
+        if (tmp_int == new_dataset)
+            throw ExecuteError("New dataset can't be reverted");
+        if (!vt.empty())
+            throw ExecuteError("Options can't be given when reverting data");
+        AL->get_data(tmp_int)->revert();
+    }
+    else
+        AL->import_dataset(tmp_int, t, vt);
     outdated_plot=true;  
 }
+void do_revert_data(char const*, char const*)
+{
+    AL->get_data(tmp_int)->revert();
+    outdated_plot=true;  
+}
+
+// returns true if x columns in all datasets have the same x values
+bool equal_x_colums(bool only_active)
+{
+    vector<fp> xx;
+    vector<Point> const& points = AL->get_data(0)->points();
+    for (size_t i = 0; i < points.size(); ++i) {
+        if (!only_active || points[i].is_active)
+            xx.push_back(points[i].x);
+    }
+
+    for (int j = 1; j < AL->get_ds_count(); ++j) {
+        vector<Point> const& points2 = AL->get_data(j)->points();
+        size_t n = 0;
+        for (size_t i = 0; i < points2.size(); ++i) {
+            if (!only_active || points2[i].is_active) {
+                double x = points2[i].x;
+                if (n >= xx.size() || fabs(xx[n] - x) > 1e-6 * fabs(xx[n] + x))
+                    return false;
+                ++n;
+            }
+        }
+    }
+    return true;
+}
+
+
+// change F(...) to @n.F(...), and the same with Z
+string add_ds_to_sum(string const& s, int nr)
+{
+    string s2;
+    s2.reserve(s.size());
+    for (string::const_iterator i = s.begin(); i != s.end(); ++i) {
+        if ((*i == 'F' || *i == 'Z')
+             && (i == s.begin() || (*(i-1) != '.' && !isalnum(*(i-1)) 
+                                    && *(i-1) != '_' && *(i-1) != '$' 
+                                    && *(i-1) != '%')) 
+             && (i+1 == s.end() || (!isalnum(*(i+1)) && *(i+1) != '_'))) {
+            s2 += "@" + S(nr) + ".";
+        }
+        s2 += *i;
+    }
+    return s2;
+}
 
 void do_export_dataset(char const*, char const*)
 {
-    vector<string> const& ff_names = AL->get_sum(tmp_int)->get_ff_names();
-    AL->get_data(tmp_int)->export_to_file(t, vt, ff_names); 
+    vector<string>& cols = vt;
+    int idx = tmp_int; 
+
+    ostringstream os;
+    os << "# exported by fityk " VERSION << endl;
+
+    bool only_active = !contains_element(cols, "a");
+    int ds_count = AL->get_ds_count();
+    vector<vector<fp> > r;
+
+    // first dataset
+    Data *data = AL->get_data(idx == all_datasets ? 0 : idx);
+    string title = data->get_title();
+    for (vector<string>::const_iterator i = cols.begin(); i != cols.end(); ++i){
+        os << (i == cols.begin() ? "#" : "\t") << title << ":" << *i;
+        r.push_back(get_all_point_expressions(add_ds_to_sum(*i, idx), 
+                                              data, only_active));
+    }
+
+    // the rest of datasets (if any)
+    if (idx == all_datasets && ds_count > 1) { 
+        // special exception - remove redundant x columns
+        if (cols[0] == "x" && equal_x_colums(only_active))
+            cols.erase(cols.begin());
+
+        for (int i = 1; i < ds_count; ++i) {
+            data = AL->get_data(i);
+            string title = data->get_title();
+            for (vector<string>::const_iterator j = cols.begin(); 
+                                                    j != cols.end(); ++j) {
+                os << "\t" << title << ":" << *j;
+                r.push_back(get_all_point_expressions(add_ds_to_sum(*j, idx), 
+                                                      data, only_active));
+            }
+        }
+    }
+    os << endl; // after title
+
+    // output 2D table r as TSV
+    size_t nc = r.size();
+    for (size_t i = 0; i != r[0].size(); ++i) {
+        for (size_t j = 0; j != nc; ++j) {
+            os << r[j][i] << (j < nc-1 ? '\t' : '\n'); 
+        }
+    }
+
+    prepared_info += "\n" + os.str();
 }
 
-void do_append_data(char const*, char const*)
-{
-    int n = AL->append_ds();
-    AL->activate_ds(n);
-    outdated_plot=true;  
-}
 
 void do_load_data_sum(char const*, char const*)
 {
@@ -106,30 +224,12 @@ void do_load_data_sum(char const*, char const*)
     if (tmp_int == new_dataset) 
         tmp_int = AL->append_ds();
     AL->get_data(tmp_int)->load_data_sum(dd, t);
-    AL->activate_ds(tmp_int);
     outdated_plot=true;  
 }
 
 void do_plot(char const*, char const*)
 {
-    if (vds.size() == 1 && vds[0] >= 0)
-        AL->activate_ds(vds[0]);
-    bool need_ds = false;
-    for (vector<string>::const_iterator i = vr.begin(); i != vr.end(); ++i)
-        if (i->empty())
-            need_ds = true;
-    if (need_ds) {
-        vector<DataWithSum*> dsds = get_datasets_from_indata();
-        //move active ds to the front
-        DataWithSum* ads = AL->get_ds(AL->get_active_ds_position());
-        vector<DataWithSum*>::iterator pos = find(dsds.begin(),dsds.end(), ads);
-        if (pos != dsds.end() && pos != dsds.begin()) {
-            *pos = dsds[0];
-            dsds[0] = ads;
-        }
-        AL->view.set_datasets(dsds);
-    }
-    AL->view.parse_and_set(vr);
+    AL->view.parse_and_set(vr, get_ds_indices_from_indata());
     AL->get_ui()->draw_plot(1, true);
     outdated_plot = false;
 }
@@ -139,8 +239,15 @@ void do_output_info(char const*, char const*)
     prepared_info = strip_string(prepared_info);
     if (no_info_output)
         return;
-    if (output_redir.empty())
+    if (output_redir.empty()) {
+        int max_screen_info_length = 2048;
+        int more = prepared_info.length() - max_screen_info_length;
+        if (more > 0) {
+            prepared_info.resize(max_screen_info_length);
+            prepared_info += "\n[... " + S(more) + " characters more...]";
+        }
         AL->rmsg(prepared_info);
+    }
     else {
         ofstream os(output_redir.c_str(), 
                     ios::out | (info_append ? ios::app : ios::trunc));
@@ -150,6 +257,50 @@ void do_output_info(char const*, char const*)
     }
 }
 
+void do_print_info_in_data(char const* a, char const* b)
+{
+    string s = string(a,b);
+    vector<DataWithSum*> v = get_datasets_from_indata();
+
+    if (startswith(s, "fit"))
+        prepared_info += "\n" + AL->get_fit()->get_info(v);
+    else if (startswith(s, "errors"))
+        prepared_info += "\n" + AL->get_fit()->getErrorInfo(v, with_plus);
+
+    else if (startswith(s, "peaks")) {
+        vector<fp> errors;
+        if (with_plus) 
+            errors = AL->get_fit()->get_symmetric_errors(v);
+        for (vector<DataWithSum*>::const_iterator i = v.begin(); i != v.end(); ++i) {
+            prepared_info += "\n# " + (*i)->get_data()->get_title() + "\n"
+                          + (*i)->get_sum()->get_peak_parameters(errors) + "\n";
+        }
+    }
+    else if (startswith(s, "formula")) {
+        bool gnuplot = AL->get_settings()->get_e("formula-export-style") == 1;
+        for (vector<DataWithSum*>::const_iterator i = v.begin(); i != v.end(); ++i) {
+            prepared_info += "\n# " + (*i)->get_data()->get_title() + "\n"
+                   + (*i)->get_sum()->get_formula(!with_plus, gnuplot);
+        }
+    }
+    else if (startswith(s, "title")) {
+        for (vector<DataWithSum*>::const_iterator i = v.begin(); i != v.end(); ++i) {
+            prepared_info += "\n" + (*i)->get_data()->get_title();
+        }
+    }
+    else if (startswith(s, "filename")) {
+        for (vector<DataWithSum*>::const_iterator i = v.begin(); i != v.end(); ++i) {
+            prepared_info += "\n" + (*i)->get_data()->get_filename();
+        }
+    }
+    else if (startswith(s, "data")) {
+        for (vector<DataWithSum*>::const_iterator i = v.begin(); i != v.end(); ++i) {
+            prepared_info += "\n" + (*i)->get_data()->get_info();
+        }
+    }
+}
+
+
 void do_print_info(char const* a, char const* b)
 {
     string s = string(a,b);
@@ -172,7 +323,8 @@ void do_print_info(char const* a, char const* b)
                 "\nCompilation date: " __DATE__ 
                 "\nBoost.Spirit version: " + S(SPIRIT_VERSION / 0x1000) 
                                 + "." + S(SPIRIT_VERSION % 0x1000 / 0x0100)
-                                + "." + S(SPIRIT_VERSION % 0x0100);
+                                + "." + S(SPIRIT_VERSION % 0x0100)
+                + "\nxylib version:" + xylib::get_version();
  
         }
         else
@@ -192,15 +344,6 @@ void do_print_info(char const* a, char const* b)
                         m += (*i)->xname + " ";
         }
     }
-    else if (s[0] == '$') {
-        const Variable* v = AL->find_variable(string(s, 1));
-        m = v->get_info(AL->get_parameters(), with_plus);
-        if (with_plus) {
-            vector<string> refs = AL->get_variable_references(string(s, 1));
-            if (!refs.empty())
-                m += "\nreferenced by: " + join_vector(refs, ", ");
-        }
-    }
     else if (s == "functions") {
         if (functions.empty())
             m = "No functions found.";
@@ -234,15 +377,6 @@ void do_print_info(char const* a, char const* b)
             for (int i = 0; i < AL->get_ds_count(); ++i)
                 m += "\n@" + S(i) + ": " + AL->get_data(i)->get_title();
     }
-    else if (s[0] == '@') {
-        Data const* data = AL->get_data(tmp_int);
-        if (s.find(".title") != string::npos)
-            m = data->get_title();
-        else if (s.find(".filename") != string::npos)
-            m = data->get_filename();
-        else
-            m = data->get_info();
-    }
     else if (s == "view") {
         m = AL->view.str();
     }
@@ -250,35 +384,13 @@ void do_print_info(char const* a, char const* b)
         m = AL->get_settings()->print_usage();
     else if (startswith(s, "fit-history"))
         m = AL->get_fit_container()->param_history_info();
-    else if (startswith(s, "fit"))
-        m = AL->get_fit()->get_info(get_datasets_from_indata());
-    else if (startswith(s, "errors"))
-        m = AL->get_fit()->getErrorInfo(get_datasets_from_indata(), with_plus);
-    else if (startswith(s, "peaks")) {
-        vector<fp> errors;
-        vector<DataWithSum*> v = get_datasets_from_indata();
-        if (with_plus) 
-            errors = AL->get_fit()->get_symmetric_errors(v);
-        for (vector<DataWithSum*>::const_iterator i=v.begin(); i!=v.end(); ++i){
-            m += "# " + (*i)->get_data()->get_title() + "\n";
-            m += (*i)->get_sum()->get_peak_parameters(errors) + "\n";
-        }
-    }
-    else if (startswith(s, "formula")) {
-        vector<DataWithSum*> v = get_datasets_from_indata();
-        for (vector<DataWithSum*>::const_iterator i=v.begin(); i!=v.end(); ++i){
-            m += "# " + (*i)->get_data()->get_title() + "\n";
-            bool gnuplot = AL->get_settings()->get_e("formula-export-style")==1;
-            m += (*i)->get_sum()->get_formula(!with_plus, gnuplot) + "\n";
-        }
-    }
     else if (s == "commands")
         m = AL->get_ui()->get_commands().get_info(with_plus);
     else if (startswith(s, "guess")) {
         vector<DataWithSum*> v = get_datasets_from_indata();
         for (vector<DataWithSum*>::const_iterator i = v.begin(); 
                                                            i != v.end(); ++i)
-            m += get_guess_info(*i, vr);
+            m += Guess(AL, *i).get_guess_info(vr);
     }
     prepared_info += "\n" + m;
 }
@@ -365,6 +477,21 @@ void do_print_debug_info(char const*, char const*)
 void do_print_data_expr(char const*, char const*)
 {
     string s;
+
+    // special case, only one variable, e.g. "info $_1"
+    if (t2[0] == '$' && parse(t2.c_str(), VariableLhsG).full) {
+        string varname = t2.substr(1);
+        const Variable* v = AL->find_variable(varname);
+        s = v->get_info(AL->get_parameters(), with_plus);
+        if (with_plus) {
+            vector<string> refs = AL->get_variable_references(varname);
+            if (!refs.empty())
+                s += "\nreferenced by: " + join_vector(refs, ", ");
+        }
+        prepared_info += "\n" + s;
+        return;
+    }
+
     if (vds.empty() && !is_data_dependent_expression(t2)) //no data
         s = S(get_transform_expression_value(t2, 0));
     else {
@@ -402,8 +529,21 @@ void do_print_func_type(char const* a, char const* b)
 void do_guess(char const*, char const*)
 {
     vector<DataWithSum*> v = get_datasets_from_indata();
-    for (vector<DataWithSum*>::const_iterator i = v.begin(); i != v.end(); ++i)
-        guess_and_add(*i, t, t2, vr, vt);
+    string const& name = t;
+    string const& function = t2;
+
+    // "%name = guess Linear in @0, @1" makes no sense
+    if (!name.empty() && v.size() > 1)
+        // SyntaxError actually
+        throw ExecuteError("many functions can't be assigned to one name."); 
+
+    for (vector<DataWithSum*>::const_iterator i = v.begin(); i != v.end(); ++i) {
+        DataWithSum *ds = *i;
+        vector<string> vars = vt;
+        Guess(AL, ds).guess(name, function, vr, vars);
+        string real_name = AL->assign_func(name, function, vars);
+        ds->get_sum()->add_function_to(real_name, 'F');
+    }
     outdated_plot=true;  
 }
 
@@ -436,10 +576,11 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
 
     in_data
         = eps_p [clear_a(vds)]
-        >> !("in" >> (lexeme_d['@' >> uint_p [push_back_a(vds)]
-                               ]
+        >> !("in" >> (lexeme_d['@' >> (uint_p [push_back_a(vds)]
+                                      |ch_p('*')[push_back_a(vds, all_datasets)]
+                                      )
+                              ]
                        % ','
-                     | str_p("@*") [push_back_a(vds, all_datasets)]
                      )
             )
         ;
@@ -450,12 +591,6 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
         | eps_p [assign_a(ds_pref, minus_one)]
         ;
 
-    compact_str
-        = lexeme_d['\'' >> (+~ch_p('\''))[assign_a(t)] 
-                   >> '\'']
-        | lexeme_d[+chset<>(anychar_p - chset<>(" \t\n\r;,#"))] [assign_a(t)]
-        ;
-
     type_name
         = lexeme_d[(upper_p >> +alnum_p)] 
         ;
@@ -464,34 +599,32 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
         = lexeme_d[alpha_p >> *(alnum_p | '_')]
         ;
 
-    existing_dataset_nr
-        = lexeme_d['@' >> uint_p [assign_a(tmp_int)]
+    dataset_nr
+        = lexeme_d['@' >> ( uint_p [assign_a(tmp_int)]
+                          | ch_p('*') [assign_a(tmp_int, all_datasets)]
+                          )
                   ]
         ;
 
-    dataset_nr
-        = str_p("@+") [assign_a(tmp_int, new_dataset)]
-        | existing_dataset_nr
+    dataset_lhs
+        = lexeme_d['@' >> ( uint_p [assign_a(tmp_int)]
+                          | ch_p('+') [assign_a(tmp_int, new_dataset)]
+                          )
+                  ]
         ;
 
     dataset_handling
-        = (dataset_nr >> ch_p('<') >> compact_str //load from file
-           >> lexeme_d[!(alpha_p >> *alnum_p)] [assign_a(t2)] [clear_a(vn)]
-           >> !(uint_p [push_back_a(vn)] 
+          //load from file
+        = (dataset_lhs >> '<' >> CompactStrG [clear_a(vt)]
+           >> !(lexeme_d[+(alnum_p | '-' | '_')] [push_back_a(vt)] 
                 % ',')
           ) [&do_import_dataset]
-        | dataset_nr >> ch_p('=') [clear_a(vn)] [assign_a(t, empty)] //sum/dup
+          //sum / duplicate
+        | dataset_lhs >> ch_p('=') [clear_a(vn)] [assign_a(t, empty)] 
           >> !(lexeme_d[lower_p >> +(alnum_p | '-' | '_')] [assign_a(t)])
           >> (lexeme_d['@' >> uint_p [push_back_a(vn)]  
-                                           ] % '+') [&do_load_data_sum]
-        | (existing_dataset_nr [clear_a(vt)]
-           >> !('(' >> ((DataExpressionG
-                        | ("*F(" >> DataExpressionG >> ")")
-                        ) [push_back_a(vt)] 
-                        % ',')
-                >> ')')
-           >> '>' >> compact_str) [&do_export_dataset]
-        | str_p("@+")[&do_append_data] 
+                      | "0"
+                                   ] % '+') [&do_load_data_sum]
         ;
 
     plot_range  //first clear vr if needed 
@@ -505,21 +638,25 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
     
     info_arg
         = (str_p("commands") >> IntRangeG) [&do_list_commands] 
-        |  ( str_p("version") 
-           | str_p("variables") 
-           | VariableLhsG 
-           | str_p("types") 
-           | str_p("functions") 
-           | str_p("datasets") 
-           | str_p("commands") 
-           | str_p("view") 
-           | str_p("set")) [&do_print_info] 
-        | ((str_p("fit-history") 
-            | "fit"
-            | "errors"
-            | "peaks"
-            | "formula"
-                    ) >> in_data) [&do_print_info] 
+        | ( str_p("version") 
+          | "variables" 
+          | "types" 
+          | "functions" 
+          | "datasets" 
+          | "commands" 
+          | "view" 
+          | "set"
+          | "fit-history"
+          ) [&do_print_info] 
+        | (( str_p("fit")
+           | "errors"
+           | "peaks"
+           | "formula"
+           | "filename" 
+           | "title" 
+           | "data"
+           ) 
+           >> in_data) [&do_print_info_in_data] 
         | (str_p("guess") [clear_a(vr)]
            >> plot_range >> in_data) [&do_print_info]
         | type_name[&do_print_func_type]
@@ -533,9 +670,13 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
         | (ds_prefix >> str_p("dF") >> '(' 
            >> no_actions_d[DataExpressionG][assign_a(t2)] 
            >> ')') [&do_print_sum_derivatives_info]
-        | (existing_dataset_nr 
-                >> (str_p(".filename") | ".title" | eps_p)) [&do_print_info]
-        | "debug " >> compact_str [&do_print_debug_info] //no output_redir
+          // export data
+        | dataset_nr [clear_a(vt)]
+          >> str_p("(") 
+          >> (DataExpressionG [push_back_a(vt)] 
+              % ',')
+          >> str_p(")") [&do_export_dataset]
+        | "debug " >> CompactStrG [&do_print_debug_info] //no output_redir
         // this will eat also ',', because voigt(x,y) needs it
         // unfortunatelly "i der x^2, sin(x)" is made impossible
         | "der " >> (+chset<>(anychar_p - chset<>(";#"))) [&do_print_deriv]
@@ -567,14 +708,15 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
            >> !( ( str_p(">>") [assign_a(info_append, true_)]
                  | str_p(">") [assign_a(info_append, false_)]
                  )
-                >> compact_str [assign_a(output_redir, t)]
+                >> CompactStrG [assign_a(output_redir, t)]
                )
           ) [&do_output_info]
         | (optional_suffix_p("p","lot") [clear_a(vr)] 
            >> plot_range >> plot_range >> in_data) [&do_plot]
         | guess [&do_guess]
-        | (ds_prefix >> "title" >> '=' >> compact_str)[&set_data_title]
         | dataset_handling
+        | optional_suffix_p("s","et") 
+          >> (ds_prefix >> "title" >> '=' >> CompactStrG)[&set_data_title]
         ;
 }
 
