@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: cmd3.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: cmd3.h 400 2008-03-03 14:51:40Z wojdyr $
 
 #ifndef FITYK__CMD3__H__
 #define FITYK__CMD3__H__
@@ -26,7 +26,7 @@ struct Cmd3Grammar : public grammar<Cmd3Grammar>
   {
     definition(Cmd3Grammar const& self);
 
-    rule<ScannerT> in_data, optional_plus, compact_str, 
+    rule<ScannerT> in_data, optional_plus, 
                    set_arg, commands_arg, fit_arg, statement;  
 
     rule<ScannerT> const& start() const { return statement; }
