@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr 
 // Licence: GNU General Public License version 2
-// $Id: common.h 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: common.h 317 2007-07-15 00:39:46Z wojdyr $
 
 /*
  *  various headers and definitions. Included by all files.
@@ -18,13 +18,15 @@
 
 #include <string>
 #include <sstream>
-#include <fstream>
+#include <iomanip>
 #include <vector>
 #include <map>
 #include <math.h>
 #include <assert.h>
-#include <stdexcept>
 
+#include "fityk.h" //ExecuteError
+using fityk::ExecuteError;
+using fityk::ExitRequestedException;
 
 //--------------------------  N U M E R I C  --------------------------------
 
@@ -79,14 +81,12 @@ inline bool is_finite(fp a)
 /// Round real to integer.
 inline int iround(fp d) { return static_cast<int>(floor(d+0.5)); }
 
-// swap two variables
-template<typename T>
-inline void Swap(T& fp1 ,T& fp2) 
-{
-    T swaptmp = fp1;
-    fp1 = fp2;
-    fp2 = swaptmp;
-}
+#ifndef __GNUC__
+        //TODO implement erf and erfc
+	inline float erfc(float f) { return 0.5f; }
+	inline float erf(float f) { return 0.5f; }
+	inline float trunc(float f) { return (float) (int) f; }
+#endif	// !__GNUC__
 
 
 //---------------------------  S T R I N G  --------------------------------
@@ -103,6 +103,11 @@ inline std::string S(char *k) { return std::string(k); }
 inline std::string S(char const k) { return std::string(1, k); }
 inline std::string S(std::string const &k) { return k; }
 inline std::string S() { return std::string(); }
+// double -> string, with given precision
+inline std::string S(double k, int /*prec*/) { 
+    return static_cast<std::ostringstream&>(std::ostringstream() 
+                                /* << std::setprecision(prec) */<< k).str();
+}
 
 
 /// True if the string contains only a real number
@@ -254,6 +259,7 @@ void purge_all_elements(std::vector<T*> &vec)
     vec.clear();
 }
 
+/// check if vector (first arg) contains given element (second arg)
 template<typename T, typename T2>
 bool contains_element(std::vector<T> const& vec, T2 const& t)
 {
@@ -290,6 +296,22 @@ std::vector<T2> get_map_values(std::map<T1,T2> const& m)
     return result;
 }
 
+//----------------  filename utils  -------------------------------------
+#if defined(_WIN32) || defined(WIN32) || defined(__NT__) || defined(__WIN32__) || defined(__OS2__)
+#define PATH_COMPONENT_SEP '\\'
+#elif defined(__MAC__) || defined(__APPLE__) || defined(macintosh)
+//Mac OS <= 9
+#define PATH_COMPONENT_SEP ':'  
+#else
+#define PATH_COMPONENT_SEP '/'
+#endif
+
+inline std::string get_directory(std::string const& filename)
+{
+    std::string::size_type i = filename.rfind(PATH_COMPONENT_SEP);
+    return i==std::string::npos ? std::string() : std::string(filename, 0, i+1);
+}
+
 
 //-------------------- M I S C E L A N O U S ------------------------------
 
@@ -306,28 +328,5 @@ std::string time_now ();
 
 enum OutputStyle  { os_normal, os_warn, os_quot, os_input };
 
-/// exception thrown at run-time (when executing parsed command)
-class ExecuteError : public std::runtime_error 
-{
-public:
-    ExecuteError(const std::string& msg) : runtime_error(msg) {}
-};
-
-/// exception thrown to finish the program
-struct ExitRequestedException : std::exception {};
-
-
-
-class Sum;
-class Data;
-class V_f;
-class V_z;
-class V_g;
-class LMfit;
-class GAfit;
-class NMfit;
-class Crystal;
-class GnuPlot;
-
 #endif
 
