@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: datatrans.cpp 288 2007-04-22 02:14:02Z wojdyr $
+// $Id: datatrans.cpp 401 2008-03-05 13:50:23Z wojdyr $
 
 // the idea of VM is based on one of boost::spirit samples - vmachine_calc
 
@@ -270,8 +270,10 @@ namespace datatrans {
     if (op == OP_##x) return #x;
 string dt_op(int op)
 {
-    OP_(NEG)   OP_(EXP)   OP_(SIN)   OP_(COS)  OP_(ATAN)  OP_(ABS)  OP_(ROUND) 
-    OP_(TAN) OP_(ASIN) OP_(ACOS)
+    OP_(NEG)   OP_(EXP)   
+    OP_(SIN)   OP_(COS)  OP_(TAN)  OP_(SINH) OP_(COSH)  OP_(TANH)  
+    OP_(ABS)  OP_(ROUND) 
+    OP_(ATAN) OP_(ASIN) OP_(ACOS)
     OP_(LOG10) OP_(LN)  OP_(SQRT)  OP_(POW)  
     OP_(GAMMA) OP_(LGAMMA) OP_(VOIGT)
     OP_(ADD)   OP_(SUB)   OP_(MUL)   OP_(DIV)  OP_(MOD)
@@ -442,6 +444,15 @@ bool execute_code(int n, int &M, vector<fp>& stack,
             case OP_TAN:
                 STACK_OP *stackPtr = tan(*stackPtr); 
                 break;
+            case OP_SINH:
+                STACK_OP *stackPtr = sinh(*stackPtr);
+                break;
+            case OP_COSH:
+                STACK_OP *stackPtr = cosh(*stackPtr);
+                break;
+            case OP_TANH:
+                STACK_OP *stackPtr = tanh(*stackPtr); 
+                break;
             case OP_ATAN:
                 STACK_OP *stackPtr = atan(*stackPtr); 
                 break;
@@ -826,390 +837,6 @@ bool execute_code(int n, int &M, vector<fp>& stack,
     return return_value;
 }
 
-#if 0
-// not tested !!!!!!
-// this function could be used for optimization of data transformations
-// but optimization is not neccessary (?)
-void multipoint_execute_code(int M, vector<fp>& stack, 
-                  vector<Point> const& old_points, vector<Point>& new_points,
-                  vector<int> const& code)  
-{
-    DT_DEBUG("executing code (multipoint),  M=" + S(M))
-    assert(M == size(new_points));
-    int offset = -1; //stack offset, will be ++'ed first
-    vector<vector<int>::const_iterator> skips(M, code.begin());
-
-#undef STACK_OP
-#define STACK_OP \
-    for (vector<fp>::iterator stackPtr = stack.begin() + M * offset; \
-                                stackPtr!=stack.end(); stackPtr=stack.end()) \
-        for (int n = 0; n < M; ++n, ++stackPtr) \
-          //  if (i >= skips[n])
-
-#undef STACK_OFFSET_CHANGE
-#define STACK_OFFSET_CHANGE(ch) \
-    offset += (ch);
-
-    for (vector<int>::const_iterator i=code.begin(); i != code.end(); i++) {
-        if (size(stack) < (offset+2) * M)
-            stack.resize((offset+2) * M);
-// the exact copy ...
-//---8<--- START OF DT COMMON BLOCK --------------------------------------
-        DT_DEBUG("op " + dt_op(*i))
-        switch (*i) {
-            //unary-operators
-            case OP_NEG:
-                STACK_OP *stackPtr = - *stackPtr;
-                break;
-            case OP_SQRT:
-                STACK_OP *stackPtr = sqrt(*stackPtr);
-                break;
-            case OP_GAMMA:
-                STACK_OP *stackPtr = gammafn(*stackPtr);
-                break;
-            case OP_LGAMMA:
-                STACK_OP *stackPtr = lgammafn(*stackPtr);
-                break;
-            case OP_EXP:
-                STACK_OP *stackPtr = exp(*stackPtr);
-                break;
-            case OP_ERFC:
-                STACK_OP *stackPtr = erfc(*stackPtr);
-                break;
-            case OP_ERF:
-                STACK_OP *stackPtr = erf(*stackPtr);
-                break;
-            case OP_LOG10:
-                STACK_OP *stackPtr = log10(*stackPtr); 
-                break;
-            case OP_LN:
-                STACK_OP *stackPtr = log(*stackPtr); 
-                break;
-            case OP_SIN:
-                STACK_OP *stackPtr = sin(*stackPtr);
-                break;
-            case OP_COS:
-                STACK_OP *stackPtr = cos(*stackPtr);
-                break;
-            case OP_TAN:
-                STACK_OP *stackPtr = tan(*stackPtr); 
-                break;
-            case OP_ATAN:
-                STACK_OP *stackPtr = atan(*stackPtr); 
-                break;
-            case OP_ASIN:
-                STACK_OP *stackPtr = asin(*stackPtr); 
-                break;
-            case OP_ACOS:
-                STACK_OP *stackPtr = acos(*stackPtr); 
-                break;
-            case OP_ABS:
-                STACK_OP *stackPtr = fabs(*stackPtr);
-                break;
-            case OP_ROUND:
-                STACK_OP *stackPtr = floor(*stackPtr + 0.5);
-                break;
-
-            case OP_x_IDX:
-                STACK_OP *stackPtr = find_idx_in_sorted(old_points, *stackPtr);
-                break;
-
-#ifndef STANDALONE_DATATRANS
-            case OP_FUNC:
-                i++;
-                STACK_OP 
-                *stackPtr = AL->get_function(*i)->calculate_value(*stackPtr);
-                break;
-            case OP_SUM_F:
-                i++;
-                STACK_OP *stackPtr = AL->get_sum(*i)->value(*stackPtr);
-                break;
-            case OP_SUM_Z:
-                i++;
-                STACK_OP *stackPtr = AL->get_sum(*i)->zero_shift(*stackPtr);
-                break;
-            case OP_NUMAREA:
-                i += 2;
-                STACK_OFFSET_CHANGE(-2)
-                if (*(i-1) == OP_FUNC) {
-                    STACK_OP
-                    *stackPtr = AL->get_function(*i)->numarea(*stackPtr, 
-                                        *(stackPtr+1), iround(*(stackPtr+2)));
-                }
-                else if (*(i-1) == OP_SUM_F) {
-                    STACK_OP
-                    *stackPtr = AL->get_sum(*i)->numarea(*stackPtr, 
-                                        *(stackPtr+1), iround(*(stackPtr+2)));
-                }
-                else // OP_SUM_Z
-                    throw ExecuteError("numarea(Z,...) is not implemented."
-                                       "Does anyone need it?"); 
-                break;
-
-            case OP_FINDX:
-                i += 2;
-                STACK_OFFSET_CHANGE(-2)
-                if (*(i-1) == OP_FUNC) {
-                    STACK_OP
-                    *stackPtr = AL->get_function(*i)->find_x_with_value(
-                                      *stackPtr, *(stackPtr+1), *(stackPtr+2));
-                }
-                else if (*(i-1) == OP_SUM_F) {
-                    throw ExecuteError("findx(F,...) is not implemented. "
-                                       "Does anyone need it?"); 
-                }
-                else // OP_SUM_Z
-                    throw ExecuteError("findx(Z,...) is not implemented. "
-                                       "Does anyone need it?"); 
-                break;
-
-            case OP_FIND_EXTR:
-                i += 2;
-                STACK_OFFSET_CHANGE(-1)
-                if (*(i-1) == OP_FUNC) {
-                    STACK_OP
-                    *stackPtr = AL->get_function(*i)->find_extremum(*stackPtr,
-                                                                *(stackPtr+1));
-                }
-                else if (*(i-1) == OP_SUM_F) {
-                    throw ExecuteError("extremum(F,...) is not implemented. "
-                                       "Does anyone need it?"); 
-                }
-                else // OP_SUM_Z
-                    throw ExecuteError("extremum(Z,...) is not implemented. "
-                                       "Does anyone need it?"); 
-                break;
-#endif //not STANDALONE_DATATRANS
-
-            case OP_PARAMETERIZED:
-                i++;
-                STACK_OP *stackPtr = parameterized[*i]->calculate(*stackPtr);
-                break;
-
-            //binary-operators
-            case OP_MIN2:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = min(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_MAX2:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = max(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_RANDU:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = rand_uniform(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_RANDNORM:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr += rand_gauss() * *(stackPtr+1);
-                break;
-            case OP_ADD:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr += *(stackPtr+1);
-                break;
-            case OP_SUB:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr -= *(stackPtr+1);
-                break;
-            case OP_MUL:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr *= *(stackPtr+1);
-                break;
-            case OP_DIV:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr /= *(stackPtr+1);
-                break;
-            case OP_MOD:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP
-                *stackPtr -= floor(*stackPtr / *(stackPtr+1)) * *(stackPtr+1);
-                break;
-            case OP_POW:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = pow(*stackPtr, *(stackPtr+1));
-                break;
-
-            // comparisions
-            case OP_LT:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = is_lt(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_GT:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = is_gt(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_LE:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = is_le(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_GE:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = is_ge(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_EQ:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = is_eq(*stackPtr, *(stackPtr+1));
-                break;
-            case OP_NEQ:
-                STACK_OFFSET_CHANGE(-1)
-                STACK_OP *stackPtr = is_neq(*stackPtr, *(stackPtr+1));
-                break;
-
-                // next comparision hack, see rbool rule for more...
-            case OP_NCMP_HACK:
-                STACK_OFFSET_CHANGE(+1)
-                // put number that is accidentally in unused part of the stack
-                STACK_OP *stackPtr = *(stackPtr+1); 
-                break;
-            
-            // putting-number-to-stack-operators
-            // stack overflow not checked
-            case OP_NUMBER:
-                STACK_OFFSET_CHANGE(+1)
-                i++;
-                STACK_OP *stackPtr = numbers[*i];
-                break;
-            case OP_VAR_n:
-                STACK_OFFSET_CHANGE(+1)
-                STACK_OP *stackPtr = static_cast<fp>(n);
-                break;
-            case OP_VAR_M:
-                STACK_OFFSET_CHANGE(+1)
-                STACK_OP *stackPtr = static_cast<fp>(new_points.size());
-                break;
-            case OP_VAR_x:
-                STACK_OP
-                *stackPtr = get_var_with_idx(*stackPtr, old_points, &Point::x);
-                break;
-            case OP_VAR_y:
-                STACK_OP
-                *stackPtr = get_var_with_idx(*stackPtr, old_points, &Point::y);
-                break;
-            case OP_VAR_s:
-                STACK_OP
-                *stackPtr = get_var_with_idx(*stackPtr, old_points, 
-                                             &Point::sigma);
-                break;
-            case OP_VAR_a:
-                STACK_OP
-                *stackPtr = bool(iround(get_var_with_idx(*stackPtr, old_points, 
-                                                         &Point::is_active)));
-                break;
-            case OP_VAR_X:
-                STACK_OP
-                *stackPtr = get_var_with_idx(*stackPtr, new_points, &Point::x);
-                break;
-            case OP_VAR_Y:
-                STACK_OP
-                *stackPtr = get_var_with_idx(*stackPtr, new_points, &Point::y);
-                break;
-            case OP_VAR_S:
-                STACK_OP
-                *stackPtr = get_var_with_idx(*stackPtr, new_points, 
-                                             &Point::sigma);
-                break;
-            case OP_VAR_A:
-                STACK_OP
-                *stackPtr = bool(iround(get_var_with_idx(*stackPtr, new_points, 
-                                                         &Point::is_active)));
-                break;
-
-            //assignment-operators
-            case OP_ASSIGN_X:
-                STACK_OP new_points[n].x = *stackPtr;
-                STACK_OFFSET_CHANGE(-1)
-                break;
-            case OP_ASSIGN_Y:
-                STACK_OP new_points[n].y = *stackPtr;
-                STACK_OFFSET_CHANGE(-1)
-                break;
-            case OP_ASSIGN_S:
-                STACK_OP new_points[n].sigma = *stackPtr;
-                STACK_OFFSET_CHANGE(-1)
-                break;
-            case OP_ASSIGN_A:
-                STACK_OP new_points[n].is_active = is_neq(*stackPtr, 0.);
-                STACK_OFFSET_CHANGE(-1)
-                break;
-
-            // logical; can skip part of VM code !
-            case OP_NOT:
-                STACK_OP *stackPtr = is_eq(*stackPtr, 0.);
-                break;
-//---8<--- END OF DT COMMON BLOCK --------------------------------------
-            case OP_AND:
-                STACK_OP 
-                if (is_zero(*stackPtr))    
-                    skips[n] = skip_code(i, OP_AND, OP_AFTER_AND) + 1;
-                STACK_OFFSET_CHANGE(-1)
-                break;
-
-            case OP_OR:
-                STACK_OP 
-                if (!is_zero(*stackPtr))   
-                    skips[n] = skip_code(i, OP_OR, OP_AFTER_OR) + 1;
-                STACK_OFFSET_CHANGE(-1)
-                break;
-
-            case OP_TERNARY:
-                STACK_OP
-                if (is_zero(*stackPtr)) 
-                    skips[n] = skip_code(i, OP_TERNARY, OP_TERNARY_MID) + 1;
-                STACK_OFFSET_CHANGE(-1)
-                break;
-            case OP_TERNARY_MID:
-                //if we are here, condition was true. Skip.
-                STACK_OP
-                skips[n] = skip_code(i, OP_TERNARY_MID, OP_AFTER_TERNARY) + 1;
-                break;
-
-            case OP_AFTER_AND: //do nothing
-            case OP_AFTER_OR:
-            case OP_AFTER_TERNARY:
-                break;
-
-            case OP_DELETE_COND:
-                for (int n = M-1; n >= 0; --n)
-                    if (!is_zero(stack[n + M * offset]))
-                        new_points.erase(new_points.begin() + n);
-                offset--;
-                break;
-
-            //transformation condition 
-            case OP_RANGE: 
-              {
-                //x[i...j]= or delete[i...j]
-                STACK_OP 
-                {
-                int right = iround(*stackPtr); //Last In First Out
-                if (right <= 0)
-                    right += M;
-                int left = iround(*(stackPtr-1));
-                stackPtr-=2;
-                if (left < 0)
-                    left += M;
-                assert (*(i+1) != OP_DELETE);
-                //if n not in [i...j] then skip to prevent OP_ASSIGN_.
-                bool n_between = (left <= n && n < right);
-                if (!n_between) 
-                    skips[n] = find_end(i) + 1; 
-                }
-                break;
-              }
-            case OP_BEGIN:
-                if (*(i+1) == OP_DO_ONCE)
-                    skip_to_end(i);
-                break;
-            case OP_END: //nothing -- it is used only for skipping assignment
-                break;
-            default:
-                DT_DEBUG("Unknown operator in VM code: " + S(*i))
-                assert(0);
-        }
-    }
-}
-#endif
-
 
 /// change  AGSUM X ... X END_AGGREGATE  to  NUMBER INDEX 
 void replace_aggregates(int M, vector<Point> const& old_points,
@@ -1325,34 +952,32 @@ void execute_vm_code(const vector<Point> &old_points, vector<Point> &new_points)
 
 } //namespace
 
-vector<Point> transform_data(string const& str, vector<Point> const& old_points)
+bool compile_data_transformation(string const& str)
 {
     clear_parse_vecs();
-    // First compile string...
-    parse_info<> result = parse(str.c_str(), DataTransformG, space_p);
-    if (!result.full) {
+    parse_info<> result = parse(str.c_str(), DataTransformG >> end_p, space_p);
+    return (bool) result.full;
+}
+
+bool compile_data_expression(string const& str)
+{
+    clear_parse_vecs();
+    parse_info<> result = parse(str.c_str(), DataExpressionG >> end_p, space_p);
+    return (bool) result.full; 
+}
+
+vector<Point> transform_data(string const& str, vector<Point> const& old_points)
+{
+    bool r = compile_data_transformation(str);
+    if (!r) 
         throw ExecuteError("Syntax error in data transformation formula.");
-    }
+    
     // and then execute compiled code.
     vector<Point> new_points = old_points; //initial values of new_points
     execute_vm_code(old_points, new_points);
     return new_points;
 }
 
-bool validate_transformation(string const& str)
-{
-    clear_parse_vecs();
-    parse_info<> result = parse(str.c_str(), DataTransformG, space_p);
-    return (bool) result.full;
-}
-
-bool validate_data_expression(string const& str)
-{
-    clear_parse_vecs();
-    parse_info<> result = parse(str.c_str(), DataExpressionG, space_p);
-    return (bool) result.full; 
-}
-
 bool is_data_dependent_code(vector<int> const& code)
 {
     for (vector<int>::const_iterator i = code.begin(); i != code.end(); ++i) 
@@ -1364,18 +989,17 @@ bool is_data_dependent_code(vector<int> const& code)
 
 bool is_data_dependent_expression(string const& s)
 {
-    if (!validate_data_expression(s)) //it fills `code'
+    if (!compile_data_expression(s)) //it fills `code'
         return false;
     return is_data_dependent_code(code);
 }
 
 fp get_transform_expression_value(string const &s, Data const* data)
 {
-    clear_parse_vecs();
-    // First compile string... puts result into code etc.
-    parse_info<> result = parse(s.c_str(), DataExpressionG, space_p);
-    if (!result.full) 
+    bool r = compile_data_expression(s);
+    if (!r) 
         throw ExecuteError("Syntax error in expression: " + s);
+
     if (!data && is_data_dependent_code(code))
         throw ExecuteError("Dataset is not specified and the expression "
                            "depends on it: " + s);
@@ -1401,11 +1025,10 @@ fp get_transform_expr_value(vector<int>& code_, vector<Point> const& points)
 
 bool get_dt_code(string const& s, vector<int>& code_, vector<fp>& numbers_)
 {
-    clear_parse_vecs();
-    // First compile string... puts result into code etc.
-    parse_info<> result = parse(s.c_str(), DataExpressionG, space_p);
-    if (!result.full) 
+    bool r = compile_data_expression(s);
+    if (!r) 
         return false;
+
     for (vector<int>::iterator i = code.begin(); i != code.end(); ++i) 
         if (*i == OP_PARAMETERIZED 
                 || *i == OP_AGMIN || *i == OP_AGMAX || *i == OP_AGSUM 
@@ -1434,11 +1057,11 @@ vector<fp> get_all_point_expressions(string const &s, Data const* data,
 {
     vector<fp> values;
     vector<Point> const& points = data->points();
-    clear_parse_vecs();
-    // First compile string... puts result into code etc.
-    parse_info<> result = parse(s.c_str(), DataExpressionG, space_p);
-    if (!result.full) 
+
+    bool r = compile_data_expression(s);
+    if (!r) 
         throw ExecuteError("Syntax error in expression: " + s);
+
     int M = (int) points.size();
     vector<Point> new_points = points;
     vector<fp> stack(stack_size);
