@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: datatrans2.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: datatrans2.cpp 326 2007-08-01 05:54:03Z wojdyr $
 
 /// big grammars in Spirit take a lot of time and memory to compile
 /// so they must be splitted into separate compilation units
@@ -35,13 +35,13 @@ public:
 
     void do_prepare() {
         for (int i = 0; i < size(params) - 1; i += 2)
-            bb.push_back(B_point(params[i], params[i+1]));
+            bb.push_back(PointQ(params[i], params[i+1]));
     }
 
     fp calculate(fp x) { return get_linear_interpolation(bb, x); }
 
 private:
-    std::vector<B_point> bb;
+    std::vector<PointQ> bb;
 };
 
 
@@ -54,14 +54,14 @@ public:
 
     void do_prepare() {
         for (int i = 0; i < size(params) - 1; i += 2)
-            bb.push_back(B_point(params[i], params[i+1]));
+            bb.push_back(PointQ(params[i], params[i+1]));
         prepare_spline_interpolation(bb);
     }
 
     fp calculate(fp x) { return get_spline_interpolation(bb, x); }
 
 private:
-    std::vector<B_point> bb;
+    std::vector<PointQ> bb;
 };
 
 //----------------------   functors   -------------------------
