@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: datatrans4.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: datatrans4.cpp 353 2007-11-05 15:14:45Z wojdyr $
 
 /// big grammars in Spirit take a lot of time and memory to compile
 /// so they must be splitted into separate compilation units
@@ -35,6 +35,9 @@ DataExprFunGrammar::definition<ScannerT>::definition(
         |   (as_lower_d["exp("] >> DataExpressionG >> ')') [push_op(OP_EXP)] 
         |   (as_lower_d["log10("] >> DataExpressionG >> ')')[push_op(OP_LOG10)]
         |   (as_lower_d["ln("] >> DataExpressionG >> ')') [push_op(OP_LN)] 
+        |   (as_lower_d["sinh("] >> DataExpressionG >> ')') [push_op(OP_SINH)] 
+        |   (as_lower_d["cosh("] >> DataExpressionG >> ')') [push_op(OP_COSH)] 
+        |   (as_lower_d["tanh("] >> DataExpressionG >> ')') [push_op(OP_TANH)] 
         |   (as_lower_d["sin("] >> DataExpressionG >> ')') [push_op(OP_SIN)] 
         |   (as_lower_d["cos("] >> DataExpressionG >> ')') [push_op(OP_COS)] 
         |   (as_lower_d["tan("] >> DataExpressionG >> ')') [push_op(OP_TAN)] 
