@@ -1,12 +1,13 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: fit.cpp 322 2007-07-24 00:17:11Z wojdyr $
+// $Id: fit.cpp 420 2008-04-13 16:44:13Z wojdyr $
 
-#include "common.h"
-#include "fit.h"
 #include <algorithm>
 #include <sstream>
 #include <math.h>
+#include <string.h>
+
+#include "fit.h"
 #include "logic.h"
 #include "sum.h"
 #include "data.h"
@@ -248,7 +249,8 @@ void Fit::compute_derivatives_for(DataWithSum const* ds,
     }   
 }
 
-string Fit::print_matrix (const vector<fp>& vec, int m, int n, char *mname)
+string Fit::print_matrix (const vector<fp>& vec, int m, int n, 
+                          const char *mname)
     //m rows, n columns
 { 
     if (F->get_verbosity() <= 0)  //optimization (?)
@@ -263,7 +265,7 @@ string Fit::print_matrix (const vector<fp>& vec, int m, int n, char *mname)
             h << vec[i] << (i < n - 1 ? ", " : " }") ;
     }
     else { //matrix 
-        std::string blanks (strlen (mname) + 1, ' ');
+        std::string blanks (strlen(mname) + 1, ' ');
         for (int j = 0; j < m; j++){
             if (j > 0)
                 h << blanks << "  ";
@@ -328,6 +330,16 @@ void Fit::fit(int max_iter, vector<DataWithSum*> const& dsds)
     user_interrupt = false;
     init(); //method specific init
     max_iterations = max_iter;
+
+    // print stats
+    int nu = count(par_usage.begin(), par_usage.end(), true);
+    int np = 0;
+    for (vector<DataWithSum*>::const_iterator i = dsds.begin(); 
+                                                    i != dsds.end(); ++i) 
+        np += (*i)->get_data()->get_n(); 
+    F->msg ("Fit " + S(nu) + " (of " + S(na) + ") parameters to " + S(np) 
+            + " points ...");
+
     autoiter();
 }
 
@@ -436,20 +448,20 @@ void Fit::Jordan(vector<fp>& A, vector<fp>& b, int n)
             continue; // x[i]=b[i], b[i]==0
         }
         if (maxnr != i) {                            // interchanging rows
-            for (int j=i; j<n; j++)
+            for (int j = i; j < n; j++)
                 swap (A[n*maxnr+j], A[n*i+j]);
             swap (b[i], b[maxnr]);
         }
-        register fp foo = 1.0 / A[i*n+i];
+        fp c = 1.0 / A[i*n+i];
         for (int j = i; j < n; j++)
-            A[i*n+j] *= foo;
-        b[i] *= foo;
+            A[i*n+j] *= c;
+        b[i] *= c;
         for (int k = 0; k < n; k++)
             if (k != i) {
-                foo = A[k * n + i];
+                fp d = A[k * n + i];
                 for (int j = i; j < n; j++)
-                    A[k * n + j] -= A[i * n + j] * foo;
-                b[k] -= b[i] * foo;
+                    A[k * n + j] -= A[i * n + j] * d;
+                b[k] -= b[i] * d;
             }
     }
 }
