@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: func.cpp 322 2007-07-24 00:17:11Z wojdyr $
+// $Id: func.cpp 424 2008-04-15 11:08:42Z wojdyr $
 
 #include "common.h"
 #include "func.h"
@@ -50,11 +50,13 @@ Function::Function (Ftk const* F_,
 /// can be used also for eg. Foo(3+$bleh, area/fwhm/sqrt(pi/ln(2)))
 vector<string> Function::get_varnames_from_formula(string const& formula)
 {
+    vector<string> names;
     int lb = formula.find('(');
     int rb = find_matching_bracket(formula, lb);
     string all_names(formula, lb+1, rb-lb-1);
+    if (strip_string(all_names).empty()) 
+        return names;
     vector<string> nd = split_string(all_names, ',');
-    vector<string> names;
     for (vector<string>::const_iterator i = nd.begin(); i != nd.end(); ++i) {
         string::size_type eq = i->find('=');
         if (eq == string::npos) 
@@ -358,6 +360,16 @@ int Function::get_param_nr(string const& param) const
     return i - type_var_names.begin();
 }
 
+bool Function::get_param_value_safe(string const& param, fp &value) const
+{
+    vector<string>::const_iterator i = find(type_var_names.begin(), 
+                                            type_var_names.end(), param);
+    if (i == type_var_names.end())
+        return false;
+    value = get_var_value(i - type_var_names.begin());
+    return true;
+}
+
 fp Function::get_param_value(string const& param) const
 {
     if (param.empty())
@@ -543,7 +555,7 @@ bool is_compounded(string const& formula)
 vector<OpTree*> make_op_trees(string const& formula)
 {
     string rhs = Function::get_rhs_from_formula(formula);
-    tree_parse_info<> info = ast_parse(rhs.c_str(), FuncG, space_p);
+    tree_parse_info<> info = ast_parse(rhs.c_str(), FuncG >> end_p, space_p);
     assert(info.full);
     vector<string> vars = find_tokens_in_ptree(FuncGrammar::variableID, info);
     vector<string> lhs_vars = Function::get_varnames_from_formula(formula);
@@ -560,7 +572,7 @@ void check_fudf_rhs(string const& rhs, vector<string> const& lhs_vars)
 {
     if (rhs.empty())
         throw ExecuteError("No formula");
-    tree_parse_info<> info = ast_parse(rhs.c_str(), FuncG, space_p);
+    tree_parse_info<> info = ast_parse(rhs.c_str(), FuncG >> end_p, space_p);
     if (!info.full)
         throw ExecuteError("Syntax error in formula");
     vector<string> vars = find_tokens_in_ptree(FuncGrammar::variableID, info);
@@ -579,10 +591,13 @@ void check_rhs(string const& rhs, vector<string> const& lhs_vars)
 {
     string::size_type t = rhs.find_first_not_of(" \t\r\n");
     if (t != string::npos && isupper(rhs[t])) { //compound
-        parse_info<> info = parse(rhs.c_str(), 
-                               ((lexeme_d[(upper_p >> +alnum_p)] >> '('  
-                                 >> (no_actions_d[FuncG]  % ',') >> ')') % '+'),
-                               space_p);
+        parse_info<> info 
+            = parse(rhs.c_str(), 
+                    ((lexeme_d[(upper_p >> +alnum_p)] 
+                      >> '(' >> (no_actions_d[FuncG]  % ',') >> ')'
+                     ) % '+'
+                    ) >> end_p,
+                    space_p);
         if (!info.full)
             throw ExecuteError("Syntax error in compound formula.");
         vector<string> rf = get_cpd_rhs_components(rhs, false);
@@ -672,7 +687,7 @@ void check_cpd_rhs_function(std::string const& fun,
                                       + S(tvars.size()) + " parameters.");
     // ... and check if these parameters are ok 
     for (vector<string>::const_iterator j=gvars.begin(); j != gvars.end(); ++j){
-        tree_parse_info<> info = ast_parse(j->c_str(), FuncG, space_p);
+        tree_parse_info<> info = ast_parse(j->c_str(), FuncG >> end_p, space_p);
         assert(info.full);
         vector<string> vars=find_tokens_in_ptree(FuncGrammar::variableID, info);
         if (contains_element(vars, "x"))
