@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: mgr.cpp 294 2007-05-16 03:18:25Z wojdyr $
+// $Id: mgr.cpp 381 2008-01-23 15:22:19Z wojdyr $
 
 #include "mgr.h"
 #include "common.h"
@@ -144,7 +144,7 @@ string VariableManager::assign_variable(string const &name, string const &rhs)
         return put_into_variables(var);
     }
 
-    tree_parse_info<> info = ast_parse(rhs.c_str(), FuncG, space_p);
+    tree_parse_info<> info = ast_parse(rhs.c_str(), FuncG >> end_p, space_p);
     assert(info.full);
     const_tm_iter_t const &root = info.trees.begin();
     if (root->value.id() == FuncGrammar::variableID
