@@ -1,12 +1,13 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: ui.cpp 322 2007-07-24 00:17:11Z wojdyr $
+// $Id: ui.cpp 382 2008-01-23 21:37:02Z wojdyr $
 
-#include "common.h"
-#include "ui.h"
-#include "settings.h"
 #include <string>
 #include <iostream>
+#include <string.h>
+
+#include "ui.h"
+#include "settings.h"
 #include "logic.h"
 #include "cmd.h"
 
