@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: ui.h 322 2007-07-24 00:17:11Z wojdyr $
+// $Id: ui.h 407 2008-03-14 18:32:36Z wojdyr $
 
 
 #ifndef FITYK__UI__H__
@@ -9,6 +9,7 @@
 #include <vector>
 #include <utility>
 #include <fstream>
+#include <string>
 
 class wxString;
 struct NumberedLine;
@@ -66,8 +67,8 @@ public:
     bool keep_quiet;
 
     UserInterface(Ftk const* F_) 
-        : keep_quiet(false), F(F_), m_show_message(0), m_do_draw_plot(0), 
-          m_exec_command(0), m_refresh(0), m_wait(0) {}
+        : keep_quiet(false), F(F_), m_show_message(NULL), m_do_draw_plot(NULL), 
+          m_exec_command(NULL), m_refresh(NULL), m_wait(NULL) {}
     
     /// Update plot if pri<=auto_plot.   If !now, update can be delayed
     /// Different definition for GUI and CLI
