@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: var.cpp 264 2007-03-01 10:10:54Z wojdyr $
+// $Id: var.cpp 381 2008-01-23 15:22:19Z wojdyr $
 
 #include "var.h"
 #include "common.h"
@@ -198,6 +198,7 @@ FuncGrammar::definition<ScannerT>::definition(FuncGrammar const& /*self*/)
                 |  inner_node_d[ch_p('(') >> expression >> ')']
                 |  root_node_d[ as_lower_d[ str_p("sqrt") | "exp" 
                                           | "erfc" | "erf" | "log10" | "ln" 
+                                          | "sinh" | "cosh" | "tanh" 
                                           | "sin" | "cos" | "tan" 
                                           | "atan" | "asin" | "acos"
                                           | "lgamma"
@@ -238,7 +239,7 @@ template FuncGrammar::definition<scanner<char const*, scanner_policies<skipper_i
 /// uses calculate_deriv() to simplify formulae
 std::string simplify_formula(std::string const &formula)
 {
-    tree_parse_info<> info = ast_parse(formula.c_str(), FuncG, space_p);
+    tree_parse_info<> info = ast_parse(formula.c_str(), FuncG>>end_p, space_p);
     assert(info.full);
     const_tm_iter_t const &root = info.trees.begin();
     vector<string> vars(1, "x");
