@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: aplot.cpp 320 2007-07-22 01:44:23Z wojdyr $
+// $Id: aplot.cpp 401 2008-03-05 13:50:23Z wojdyr $
 
 /// In this file:
 ///  Auxiliary Plot, for displaying residuals, peak positions, etc. (AuxPlot)
@@ -15,7 +15,7 @@
 #include <wx/numdlg.h>
 
 #include "aplot.h"
-#include "gui.h"
+#include "frame.h"
 #include "../sum.h"
 #include "../data.h"
 #include "../logic.h"
@@ -117,23 +117,25 @@ double rdiff_y_perc_of_data_for_draw_data (vector<Point>::const_iterator i,
 
 void AuxPlot::draw(wxDC &dc, bool monochrome)
 {
-    int pos = ftk->get_active_ds_position();
+    int pos = frame->get_focused_ds_index();
     Data const* data = ftk->get_data(pos);
     Sum const* sum = ftk->get_sum(pos);
     if (auto_zoom_y || fit_y_once) {
         fit_y_zoom(data, sum);
         fit_y_once = false;
     }
-    set_scale();
+    const int pixel_width = get_pixel_width(dc);
+    const int pixel_height = get_pixel_height(dc);
+    set_scale(pixel_width, pixel_height);
     if (monochrome) {
         dc.SetPen(*wxBLACK_PEN);
         dc.SetBrush(*wxBLACK_BRUSH);
     }
     else
-        dc.SetPen(wxPen(xAxisCol));
+        dc.SetPen(wxPen(xAxisCol, pen_width));
 
     if (mark_peak_ctrs) {
-        int ymax = GetClientSize().GetHeight();
+        int ymax = pixel_height;
         std::vector<wxPoint> const& t = master->get_special_points();
         for (vector<wxPoint>::const_iterator i = t.begin(); i != t.end(); i++) 
             dc.DrawLine(i->x, 0, i->x, ymax);
@@ -144,16 +146,17 @@ void AuxPlot::draw(wxDC &dc, bool monochrome)
 
     if (x_axis_visible) {
         int Y0 = ys.px(0.);
-        dc.DrawLine (0, Y0, GetClientSize().GetWidth(), Y0);
+        dc.DrawLine (0, Y0, pixel_width, Y0);
         if (kind == apk_diff) 
             draw_zoom_text(dc, !monochrome);
     }
     if (y_axis_visible) {
         int X0 = xs.px(0.);
-        dc.DrawLine (X0, 0, X0, GetClientSize().GetHeight());
+        dc.DrawLine (X0, 0, X0, pixel_height);
     }
     if (ytics_visible) {
-        View v(0, 0, ys.val(GetClientSize().GetHeight()), ys.val(0));
+        View v(NULL);
+        v.set(0, 0, ys.val(pixel_height), ys.val(0));
         draw_ytics(dc, v, !monochrome);
     }
 
@@ -184,11 +187,11 @@ void AuxPlot::draw_zoom_text(wxDC& dc, bool set_pen)
         return;
     if (set_pen)
         dc.SetTextForeground(xAxisCol);
-    dc.SetFont(*wxNORMAL_FONT);  
+    set_font(dc, *wxNORMAL_FONT);  
     string s = "x" + S(y_zoom);  
     wxCoord w, h;
     dc.GetTextExtent (s2wx(s), &w, &h); 
-    dc.DrawText (s2wx(s), GetClientSize().GetWidth() - w - 2, 2);
+    dc.DrawText (s2wx(s), get_pixel_width(dc) - w - 2, 2);
 }
 
 void AuxPlot::OnMouseMove(wxMouseEvent &event)
@@ -196,18 +199,13 @@ void AuxPlot::OnMouseMove(wxMouseEvent &event)
     int X = event.GetX();
     vert_line_following_cursor(mat_move, X);
     frame->set_status_coord_info(xs.val(X), ys.val(event.GetY()), true);
-    int new_cursor;
     if (X < move_plot_margin_width)
-        new_cursor = wxCURSOR_POINT_LEFT;
+        set_cursor(wxCURSOR_POINT_LEFT);
     else if (X > GetClientSize().GetWidth() - move_plot_margin_width)
-        new_cursor = wxCURSOR_POINT_RIGHT;
+        set_cursor(wxCURSOR_POINT_RIGHT);
     else {
         frame->draw_crosshair(X, -1);
-        new_cursor = wxCURSOR_CROSS;
-    }
-    if (new_cursor != cursor_id) {
-        cursor_id = new_cursor;
-        SetCursor(wxCursor(new_cursor));
+        set_cursor(wxCURSOR_CROSS);
     }
 }
 
@@ -223,15 +221,17 @@ bool AuxPlot::is_zoomable()
            || kind == apk_diff_y_perc || kind == apk_cum_chi2;
 }
 
-void AuxPlot::set_scale()
+void AuxPlot::set_scale(int pixel_width, int pixel_height)
 {
-    master->set_scale();
+    // this functions depends on x scale in MainPlot 
+    // sometimes AuxPlot is redrawed before MainPlot, so we are updating 
+    // horizontal scale in MainPlot
+    master->set_scale(pixel_width, 0); 
     xs = master->get_x_scale();
 
-    int h = GetClientSize().GetHeight();
     if (kind == apk_cum_chi2) {
         ys.scale = -1. * y_zoom_base * y_zoom;
-        ys.origin = - h / ys.scale;
+        ys.origin = - pixel_height / ys.scale;
         return;
     }
     switch (kind) {
@@ -251,7 +251,7 @@ void AuxPlot::set_scale()
         default:
             assert(0);
     }
-    ys.origin = - h / 2. / ys.scale;
+    ys.origin = - pixel_height / 2. / ys.scale;
 }
  
 void AuxPlot::read_settings(wxConfigBase *cf)
@@ -317,7 +317,7 @@ void AuxPlot::OnLeftDown (wxMouseEvent &event)
     else {
         mouse_press_X = X;
         vert_line_following_cursor(mat_start, mouse_press_X+1, mouse_press_X);
-        SetCursor(wxCursor(wxCURSOR_SIZEWE));  
+        set_cursor(wxCURSOR_SIZEWE);  
         frame->set_status_text("Select x range and release button to zoom..."); 
         CaptureMouse();
     }
@@ -329,8 +329,7 @@ bool AuxPlot::cancel_mouse_left_press()
         vert_line_following_cursor(mat_stop);
         ReleaseMouse();
         mouse_press_X = INT_MIN;
-        cursor_id = wxCURSOR_CROSS;
-        SetCursor(wxCursor(wxCURSOR_CROSS));  
+        set_cursor(wxCURSOR_CROSS);  
         frame->set_status_text(""); 
         return true;
     }
@@ -486,12 +485,13 @@ void AuxPlot::fit_y_zoom(Data const* data, Sum const* sum)
                                   last = data->get_point_at(ftk->view.right);
     if (data->is_empty() || last==first)
         return;
+    int pixel_height = GetClientSize().GetHeight();
     switch (kind) { // setting y_zoom
         case apk_diff: 
             {
             y = get_max_abs_y(diff_of_data_for_draw_data, first, last, sum);
             Scale const& mys = master->get_y_scale();
-            y_zoom = fabs (GetClientSize().GetHeight() / (2 * y 
+            y_zoom = fabs (pixel_height / (2 * y 
                                            * (mys.logarithm ? 1 : mys.scale)));
             fp order = pow (10, floor (log10(y_zoom)));
             y_zoom = floor(y_zoom / order) * order;
@@ -500,20 +500,20 @@ void AuxPlot::fit_y_zoom(Data const* data, Sum const* sum)
         case apk_diff_stddev:
             y = get_max_abs_y(diff_stddev_of_data_for_draw_data, 
                               first, last, sum);
-            y_zoom_base = GetClientSize().GetHeight() / (2. * y);
+            y_zoom_base = pixel_height / (2. * y);
             y_zoom = 0.9;
             break;
         case apk_diff_y_perc:
             y = get_max_abs_y(diff_y_perc_of_data_for_draw_data, 
                               first, last, sum);
-            y_zoom_base = GetClientSize().GetHeight() / (2. * y);
+            y_zoom_base = pixel_height / (2. * y);
             y_zoom = 0.9;
             break;
         case apk_cum_chi2:
             y = 0.;
             for (vector<Point>::const_iterator i = first; i < last; i++) 
                 y += diff_chi2_of_data_for_draw_data(i, sum);
-            y_zoom_base = GetClientSize().GetHeight() / y;
+            y_zoom_base = pixel_height / y;
             y_zoom = 0.9;
             break;
         default:
