@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: aplot.h 288 2007-04-22 02:14:02Z wojdyr $
+// $Id: aplot.h 353 2007-11-05 15:14:45Z wojdyr $
 
 #ifndef FITYK__WX_APLOT__H__
 #define FITYK__WX_APLOT__H__
@@ -22,7 +22,7 @@ class AuxPlot : public FPlot
 public:
     AuxPlot(wxWindow *parent, FPlot *master_, wxString const& name_) 
         : FPlot(parent), master(master_), name(name_), 
-          y_zoom(1.), y_zoom_base(1.), fit_y_once(false) {}
+          y_zoom(1.), y_zoom_base(1.), fit_y_once(false), cursor_id(-1) {}
     ~AuxPlot() {}
     void OnPaint(wxPaintEvent &event);
     void draw(wxDC &dc, bool monochrome=false);
@@ -34,7 +34,7 @@ public:
     void OnMiddleDown (wxMouseEvent &event);
     void OnKeyDown (wxKeyEvent& event);
     bool cancel_mouse_left_press();
-    void set_scale();
+    void set_scale(int pixel_width, int pixel_height);
     bool is_zoomable(); //false if kind is eg. empty or peak-position
     void OnPopupPlot (wxCommandEvent& event);
     void OnPopupPlotCtr (wxCommandEvent& event);
@@ -46,6 +46,7 @@ public:
     void OnTicsFont (wxCommandEvent&) { change_tics_font(); }
     void save_settings(wxConfigBase *cf) const;
     void read_settings(wxConfigBase *cf);
+    void set_cursor(int id) { if (id != cursor_id) SetCursor((cursor_id=id)); } 
 
 private:
     FPlot* master;
