@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: app.h 317 2007-07-15 00:39:46Z wojdyr $
+// $Id: app.h 399 2008-03-01 14:15:02Z wojdyr $
 
 #ifndef FITYK__WX_APP__H__
 #define FITYK__WX_APP__H__
@@ -28,7 +28,7 @@ private:
     void process_argv(wxCmdLineParser &cmdLineParser);
 };
 
-std::string get_full_path_of_help_file (std::string const& name);
+wxString get_full_path_of_help_file (wxString const& name);
 
 DECLARE_APP(FApp)
 
