@@ -1,8 +1,7 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: cmn.cpp 317 2007-07-15 00:39:46Z wojdyr $
+// $Id: cmn.cpp 391 2008-02-15 02:59:57Z wojdyr $
 
-// wxwindows headers, see wxwindows samples for description
 #include <wx/wxprec.h>
 #ifdef __BORLANDC__
 #pragma hdrstop
@@ -254,3 +253,17 @@ bool should_focus_input(wxKeyEvent& event)
     return c < 128 && (c == ' ' || isalnum(c) || ispunct(c));
 }
 
+
+void updateControlWithItems(wxControlWithItems *cwi, vector<string> const& v)
+{
+    if (v.size() != (size_t) cwi->GetCount()) {
+        cwi->Clear();
+        for (size_t i = 0; i < v.size(); ++i)
+            cwi->Append(s2wx(v[i]));
+    }
+    else
+        for (size_t i = 0; i < v.size(); ++i)
+            if (cwi->GetString(i) != s2wx(v[i]))
+                cwi->SetString(i, s2wx(v[i]));
+}
+
