@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: cmn.h 317 2007-07-15 00:39:46Z wojdyr $
+// $Id: cmn.h 413 2008-03-31 20:06:42Z wojdyr $
 #ifndef FITYK__WX_CMN__H__
 #define FITYK__WX_CMN__H__
 
@@ -156,9 +156,10 @@ public:
 class KFTextCtrl : public wxTextCtrl
 {
 public:
-    KFTextCtrl(wxWindow* parent, wxWindowID id, wxString const& value) 
-        : wxTextCtrl(parent, id, value, wxDefaultPosition, wxDefaultSize,
-                     wxTE_PROCESS_ENTER) {}
+    KFTextCtrl(wxWindow* parent, wxWindowID id, wxString const& value,
+               int size=-1, long style=0) 
+        : wxTextCtrl(parent, id, value, wxDefaultPosition, wxSize(size, -1),
+                     style | wxTE_PROCESS_ENTER) {}
     void OnKillFocus(wxFocusEvent&);
     DECLARE_EVENT_TABLE()
 };
@@ -167,7 +168,7 @@ public:
 wxString get_user_conffile(std::string const& filename);
 
 
-/// introduced because OnOK and OnCancel were removed
+/// introduced because OnOK and OnCancel were removed;
 /// when wx-2.6 compatibility is dropped, it will be replaced with 
 /// wxSetEscapeId for Cancel and perhaps with wxSetAffirmativeId for OK.
 inline void close_it(wxDialog* dlg, int id=wxID_CANCEL)
@@ -180,4 +181,8 @@ inline void close_it(wxDialog* dlg, int id=wxID_CANCEL)
     }
 }
 
+// same as cwi->Clear(), cwi->Append(...), but optimized for some special cases
+void updateControlWithItems(wxControlWithItems *cwi, 
+                            std::vector<std::string> const& v);
+
 #endif 
