@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: dataedit.cpp 317 2007-07-15 00:39:46Z wojdyr $
+// $Id: dataedit.cpp 416 2008-04-09 13:33:49Z wojdyr $
 
 /// In this file:
 ///  Data Editor (DataEditorDlg) and helpers
@@ -15,11 +15,11 @@
 #include <wx/statline.h>
 
 #include "dataedit.h"
-#include "gui.h"
+#include "frame.h"
 #include "dialogs.h" //export_data_dlg()
 #include "../data.h" // Data, Point
 #include "../logic.h"  
-#include "../datatrans.h" //validate_transformation()
+#include "../datatrans.h" //compile_data_transformation()
 
 using namespace std;
 
@@ -81,7 +81,7 @@ public:
             case 3: t = "S";  break;
             default: assert(0);
         }
-        ftk->exec(t + "[" + S(row)+"]=" + S(value) + frame->get_in_dataset());
+        ftk->exec(t + "[" + S(row)+"]=" + S(value) + frame->get_in_datasets());
         if (col == 1) // order of items can be changed
             ded->grid->ForceRefresh();
         ded->rezoom_btn->Enable();
@@ -91,7 +91,7 @@ public:
     { 
         assert(col==0); 
         ftk->exec("A[" + S(row)+"]=" + (value?"true":"false") 
-                                                  + frame->get_in_dataset()); 
+                                                  + frame->get_in_datasets()); 
         ded->rezoom_btn->Enable();
     }
 
@@ -327,7 +327,7 @@ void DataEditorDlg::read_transforms(bool reset)
                                         "You can type eg. Y=log10(y).\n"
                                         "See Help for details.",
                                         "", false));
-    //TODO last transformation item
+    //TODO add last transformation item
     wxString transform_path = get_user_conffile("transform");
     string t_line;
     if (wxFileExists(transform_path) && !reset) {
@@ -382,8 +382,10 @@ void DataEditorDlg::update_data(ndnd_type const& dd)
     if (dd.size() == 1) {
         Data const* data = ndnd[0].second;
         filename = data->get_filename();
-        title = data->get_title();
         save_as_btn->Enable(true);
+        title = data->get_title();
+        title_label->Show(true);
+        title_label->SetLabel(s2wx(title));
         grid->SetTable(new DataTable(data, this), true, 
                        wxGrid::wxGridSelectRows);
         refresh_grid();
@@ -395,11 +397,19 @@ void DataEditorDlg::update_data(ndnd_type const& dd)
             title += nr + i->second->get_title() + "\n";
         }
         save_as_btn->Enable(false);
+        title_label->Show(false);
         grid->Show(false);
     }
+    revert_btn->Enable(is_revertable());
     filename_label->SetLabel(s2wx(filename));
-    revert_btn->Enable(!filename.empty());
-    title_label->SetLabel(s2wx(title));
+}
+
+bool DataEditorDlg::is_revertable() const
+{
+    for (ndnd_type::const_iterator i = ndnd.begin(); i != ndnd.end(); ++i) 
+        if (i->second->get_filename().empty())
+            return false;
+    return true;
 }
 
 void DataEditorDlg::refresh_grid()
@@ -418,14 +428,10 @@ void DataEditorDlg::OnRevert (wxCommandEvent&)
 {
     string cmd;
     for (ndnd_type::const_iterator i = ndnd.begin(); i != ndnd.end(); ++i) {
-        Data const* d = i->second;
-        if (!d->get_filename().empty())
-            cmd += "@" + S(i->first) + " <'" + d->get_filename() + "'" 
-                + d->get_given_type() + " " 
-                + join_vector(d->get_given_cols(), ",") + "; "; 
+        if (i != ndnd.begin())
+            cmd += "; ";
+        cmd += "@" + S(i->first) + ".revert";
     }
-    if (cmd.empty())
-        return;
     ftk->exec(cmd);
     refresh_grid();
 }
@@ -452,7 +458,7 @@ void DataEditorDlg::OnAdd (wxCommandEvent&)
         select_transform(pos);
 
         FFrame *fframe = static_cast<FFrame *>(GetParent());
-        fframe->update_menu_fast_tranforms();
+        fframe->update_menu_saved_tranforms();
     }
 }
 
@@ -531,16 +537,16 @@ void DataEditorDlg::execute_tranform(string code)
     string t;
     for (vector<string>::const_iterator i = cmds.begin(); i != cmds.end(); ++i){
         if (!strip_string(*i).empty())
-            t += *i + frame->get_in_one_or_all_datasets();
+            t += *i + frame->get_in_datasets();
         if (i+1 != cmds.end())
             t += ";";
     }
     ftk->exec(t);
 }
 
-void DataEditorDlg::OnHelp (wxCommandEvent&)
+void DataEditorDlg::OnHelp (wxCommandEvent& event)
 {
-    frame->display_help_section("Data transformations");
+    frame->OnShowHelp(event);
 }
 
 void DataEditorDlg::OnClose (wxCommandEvent&)
@@ -556,7 +562,7 @@ void DataEditorDlg::CodeText()
         string text = wx2s(code->GetValue());
         replace_all(text, "\n", " , ");
         // 'text' is not identical with the final command (, instead of ;)
-        apply_btn->Enable(validate_transformation(text));
+        apply_btn->Enable(compile_data_transformation(text));
     }
     else
         apply_btn->Enable(!code->GetValue().Trim().IsEmpty());
@@ -624,7 +630,8 @@ TransEditorDlg::TransEditorDlg(wxWindow* parent, wxWindowID id,
     code_tc = new wxTextCtrl(this, -1, s2wx(ex.code), 
                              wxDefaultPosition, wxSize(-1, 100),
                              wxTE_MULTILINE|wxHSCROLL|wxVSCROLL);
-    inmenu_cb = new wxCheckBox(this, -1,wxT("show item in Data->Fast_DT menu"));
+    inmenu_cb = new wxCheckBox(this, -1,
+                               wxT("show item in Data->Saved DT menu"));
     inmenu_cb->SetValue(ex.in_menu);
 
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
