@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: dialogs.cpp 326 2007-08-01 05:54:03Z wojdyr $
+// $Id: dialogs.cpp 408 2008-03-15 20:10:34Z wojdyr $
 
 /// In this file:
 ///  small dialogs:  SumHistoryDlg, FitRunDlg, DataExportDlg, AboutDlg,
@@ -16,16 +16,17 @@
 
 #include <wx/statline.h>
 #include <boost/spirit/version.hpp> //SPIRIT_VERSION for AboutDlg
+#include <xylib/xylib.h> //get_version() for AboutDlg
 
 #include "dialogs.h"
-#include "gui.h"
-#include "listptxt.h"
+#include "frame.h"
 #include "../fit.h"
 #include "../ui.h"
 #include "../settings.h"
 #include "../datatrans.h" 
 #include "../logic.h"
 #include "../data.h"
+#include "../sum.h" // get_ff_names()
 
 #include "img/up_arrow.xpm"
 #include "img/down_arrow.xpm"
@@ -135,8 +136,8 @@ void SumHistoryDlg::initialize_lc()
 void SumHistoryDlg::add_item_to_lc(int pos, vector<double> const& item)
 {
     lc->InsertItem(pos, wxString::Format(wxT("  %i  "), pos));
-    lc->SetItem (pos, 1, wxString::Format(wxT("%i"), item.size()));
-    lc->SetItem (pos, 2, wxT("      ?      "));
+    lc->SetItem(pos, 1, wxString::Format(wxT("%i"), (int) item.size()));
+    lc->SetItem(pos, 2, wxT("      ?      "));
     for (int j = 0; j < 4; j++) {
         int n = view[j];
         if (n < size(item))
@@ -178,9 +179,8 @@ void SumHistoryDlg::OnComputeWssrButton (wxCommandEvent&)
 {
     FitMethodsContainer const* fmc = ftk->get_fit_container();
     vector<double> const orig = ftk->get_parameters();
-    vector<DataWithSum*> dsds;
-    dsds.push_back(ftk->get_ds(ftk->get_active_ds_position()));
-        //TODO dsds = ftk->get_dsds() if ...
+    vector<DataWithSum*> dsds = frame->get_selected_ds();
+
     for (int i = 0; i != fmc->get_param_history_size(); ++i) {
         vector<double> const& item = fmc->get_item(i);
         if (item.size() == orig.size()) {
@@ -242,7 +242,13 @@ FitRunDlg::FitRunDlg(wxWindow* parent, wxWindowID id, bool initialize)
 {
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
     wxArrayString ds_choices;
-    ds_choices.Add(wxT("active dataset ") + s2wx(frame->get_active_data_str()));
+    sel = frame->get_selected_ds_indices();
+    string a = S(sel.size()) + " selected dataset";
+    if (sel.size() == 1)
+        a += ": @" + S(sel[0]);
+    else
+        a += "s";
+    ds_choices.Add(s2wx(a)); 
     ds_choices.Add(wxT("all datasets"));
     ds_rb = new wxRadioBox(this, -1, wxT("fit..."), 
                            wxDefaultPosition, wxDefaultSize,
@@ -307,8 +313,10 @@ void FitRunDlg::update_allow_continue()
     //use "::Fit", because Fit is also a method of wxDialog
     ::Fit const* f = ftk->get_fit_container()->get_method(m_sel);
     if (ds_rb->GetSelection() == 0) {
-        DataWithSum const* ds = ftk->get_ds(ftk->get_active_ds_position());
-        is_initialized = f->is_initialized(ds);
+        vector<DataWithSum*> dsds(sel.size());
+        for (size_t i = 0; i < sel.size(); ++i)
+            dsds[i] = ftk->get_ds(sel[i]);
+        is_initialized = f->is_initialized(dsds);
     }
     else {
         is_initialized = f->is_initialized(ftk->get_dsds());
@@ -351,7 +359,7 @@ void FitRunDlg::OnOK(wxCommandEvent&)
 
     if (ini) {
         if (ds_rb->GetSelection() == 0)
-            cmd += frame->get_in_dataset();
+            cmd += frame->get_in_datasets();
         else
             cmd += " in @*";
     }
@@ -367,23 +375,34 @@ void FitRunDlg::OnOK(wxCommandEvent&)
 bool export_data_dlg(wxWindow *parent, bool load_exported)
 {
     static wxString dir = wxConfig::Get()->Read(wxT("/exportDir"));
+
+    vector<int> sel = frame->get_selected_ds_indices();
+    int ds;
+    if (sel.size() == 1) 
+        ds = sel[0];
+    else if ((int) sel.size() == ftk->get_ds_count())
+        ds = -1;
+    else
+        return false;
+
     string columns = "";
-    string ds = frame->get_active_data_str();
     if (!load_exported) {
         DataExportDlg ded(parent, -1, ds);
         if (ded.ShowModal() != wxID_OK)
             return false;
         columns = " (" + ded.get_columns() + ")";
     }
+
     wxFileDialog fdlg (parent, wxT("Export data to file"), dir, wxT(""),
                        wxT("x y data (*.dat, *.xy)|*.dat;*.DAT;*.xy;*.XY"),
                        wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
     dir = fdlg.GetDirectory();
     if (fdlg.ShowModal() == wxID_OK) {
         string path = wx2s(fdlg.GetPath());
-        ftk->exec(ds + columns + " > '" + path + "'");
+        string ds_str = (ds == -1 ? string("@*") : "@" + S(ds));
+        ftk->exec("info " + ds_str + columns + " > '" + path + "'");
         if (load_exported)
-            ftk->exec(ds + " < '" + path + "'");
+            ftk->exec(ds_str + " < '" + path + "'");
         return true;
     }
     else
@@ -399,11 +418,11 @@ BEGIN_EVENT_TABLE(DataExportDlg, wxDialog)
     EVT_TEXT(ID_DED_TEXT, DataExportDlg::OnTextChanged)
 END_EVENT_TABLE()
 
-DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id, 
-                             std::string const& ds)
+DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id, int ds_)
     : wxDialog(parent, id, wxT("Export data/functions as points"), 
                wxDefaultPosition, wxSize(600, 500), 
-               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
+               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),
+      ds(ds_)
 {
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
     wxStaticText *st1 = new wxStaticText(this, -1, 
@@ -419,22 +438,37 @@ DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id,
     choices.Add(wxT("x, y, sigma"));
     cv.Add(wxT("x, y, s"));
     choices.Add(wxT("x, sum"));
-    cv.Add(wxT("x, ") + s2wx(ds) + wxT(".F(x)"));
+    cv.Add(wxT("x, F(x)"));
     choices.Add(wxT("x, sum, zero shift"));
-    cv.Add(wxT("x, ") + s2wx(ds) + wxT(".F(x), ") + s2wx(ds) + wxT(".Z(x)"));
+    cv.Add(wxT("x, F(x), Z(x)"));
     choices.Add(wxT("x, y, sigma, sum"));
-    cv.Add(wxT("x, y, s, ") + s2wx(ds) + wxT(".F(x)"));
+    wxString xysF = wxT("x, y, s, F(x)");
+    cv.Add(xysF);
+
     choices.Add(wxT("x, y, sigma, sum, all functions..."));
-    cv.Add(wxT("x, y, s, ") + s2wx(ds) + wxT(".F(x), *F(x)"));
+    wxString all_func;
+    if (ds >= 0) {
+        vector<string> const& ff_names = ftk->get_sum(ds)->get_ff_names();
+        for (vector<string>::const_iterator i = ff_names.begin(); 
+                                                      i != ff_names.end(); ++i)
+            all_func += wxT(", %") + s2wx(*i) + wxT("(x)");
+        cv.Add(xysF + all_func);
+    }
+    else
+        cv.Add(wxT(""));
+
     choices.Add(wxT("x, y, sigma, sum, residual"));
-    cv.Add(wxT("x, y, s, ") + s2wx(ds) + wxT(".F(x), y-") + s2wx(ds) 
-            + wxT(".F(x)"));
+    cv.Add(wxT("x, y, s, F(x), y-F(x)"));
     choices.Add(wxT("x, y, sigma, weighted residual"));
-    cv.Add(wxT("x, y, s, (y-") + s2wx(ds) + wxT(".F(x))/s"));
+    cv.Add(wxT("x, y, s, (y-F(x))/s"));
     choices.Add(wxT("custom"));
     rb = new wxRadioBox(this, ID_DED_RADIO, wxT("exported columns"),
                         wxDefaultPosition, wxDefaultSize, choices,
                         2, wxRA_SPECIFY_COLS);
+    for (size_t i = 0; i < cv.GetCount(); ++i) {
+        if (cv[i].IsEmpty())
+            rb->Enable(i, false);
+    }
     top_sizer->Add(rb, 0, wxALL|wxEXPAND, 5);
     inactive_cb = new wxCheckBox(this, ID_DED_INACT_CB, 
                                  wxT("export also inactive points"));
@@ -447,36 +481,32 @@ DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id,
     SetSizerAndFit(top_sizer);
 
     //read settings
-    wxString t = wxConfig::Get()->Read(wxT("/exportDataCols"), wxT("x, y, s"));
-    for (size_t i = 0; i < cv.GetCount(); ++i) {
-        if (t == cv[i]) {
-            rb->SetSelection(i);
-            inactive_cb->SetValue(false);
-            on_widget_change();
-            return;
-        }
-        else if (t == cv[i]+wxT(", a")) {
-            rb->SetSelection(i);
-            inactive_cb->SetValue(true);
-            on_widget_change();
-            return;
-        }
+    wxConfigBase *config = wxConfig::Get();
+    int n = config->Read(wxT("/exportDataSel"), 0L);
+    if (n >= 0 && n < (int) rb->GetCount())
+        rb->SetSelection(n);
+    if (is_custom()) {
+        text->SetValue(config->Read(wxT("/exportDataText"), wxT("")));
+        text->MarkDirty();
     }
-    rb->SetSelection(cv.GetCount());
-    text->SetValue(t);
-    text->MarkDirty();
+    else {
+        bool a = cfg_read_bool(config, wxT("/exportDataA"), false);
+        inactive_cb->SetValue(a);
+    }
+    on_widget_change();
 }
 
+
 void DataExportDlg::on_widget_change()
 {
-    int n = rb->GetSelection();
-    bool is_custom = (n == (int) cv.GetCount());
-    if (!is_custom) {
+    bool custom = is_custom();
+    if (!custom) {
+        int n = rb->GetSelection();
         text->SetValue(cv[n] + (inactive_cb->GetValue() ? wxT(", a"):wxT("")));
         FindWindow(wxID_OK)->Enable(true);
     }
-    text->Enable(is_custom);
-    inactive_cb->Enable(!is_custom);
+    text->Enable(custom);
+    inactive_cb->Enable(!custom);
 }
 
 void DataExportDlg::OnTextChanged(wxCommandEvent&) 
@@ -490,9 +520,8 @@ void DataExportDlg::OnTextChanged(wxCommandEvent&)
         string t = strip_string(*i);
         if (t == "a")
             has_a = true;
-        if (!(startswith(t, "*F(") && *(t.end()-1) == ')' 
-                    && validate_data_expression(string(t, 3, t.size()-4))
-              || validate_data_expression(t)))
+
+        if (!compile_data_expression(t))
             parsable = false;
     }
     FindWindow(wxID_OK)->Enable(parsable);
@@ -501,7 +530,14 @@ void DataExportDlg::OnTextChanged(wxCommandEvent&)
 
 void DataExportDlg::OnOk(wxCommandEvent& event) 
 {
-    wxConfig::Get()->Write(wxT("/exportDataCols"), text->GetValue());
+    // save current settings to config file
+    wxConfigBase *config = wxConfig::Get();
+    config->Write(wxT("/exportDataSel"), rb->GetSelection());
+    if (is_custom()) 
+        config->Write(wxT("/exportDataText"), text->GetValue());
+    else 
+        config->Write(wxT("/exportDataA"), inactive_cb->GetValue());
+
     event.Skip();
 }
 
@@ -541,9 +577,11 @@ AboutDlg::AboutDlg(wxWindow* parent)
                                        SPIRIT_VERSION / 0x1000,
                                        SPIRIT_VERSION % 0x1000 / 0x0100,
                                        SPIRIT_VERSION % 0x0100));
+    txt->AppendText(wxT("powered by xylib ") + s2wx(xylib::get_version())
+                    + wxT("\n"));
     txt->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, 
                                     *wxNORMAL_FONT));
-    txt->AppendText(wxT("\nCopyright (C) 2001 - 2007 Marcin Wojdyr\n\n"));
+    txt->AppendText(wxT("\nCopyright (C) 2001 - 2008 Marcin Wojdyr\n\n"));
     txt->SetDefaultStyle(wxTextAttr(*wxBLUE));
     txt->AppendText(wxT("http://www.unipress.waw.pl/fityk/\n\n"));
     txt->SetDefaultStyle(wxTextAttr(*wxBLACK));
@@ -577,8 +615,6 @@ void AboutDlg::OnTextURL(wxTextUrlEvent& event)
 //===============================================================
 
 BEGIN_EVENT_TABLE (MergePointsDlg, wxDialog)
-    EVT_LIST_ITEM_SELECTED(-1, MergePointsDlg::OnDataSelChanged)
-    EVT_LIST_ITEM_DESELECTED(-1, MergePointsDlg::OnDataSelChanged)
     EVT_CHECKBOX(-1, MergePointsDlg::OnCheckBox)
 END_EVENT_TABLE()
 
@@ -588,14 +624,10 @@ MergePointsDlg::MergePointsDlg(wxWindow* parent, wxWindowID id)
                wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER) 
 {
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
-    d = new DataListPlusText(this, -1, -1, 
-                         vector3(pair<string,int>("No", 50),
-                                 pair<string,int>("Name", 200),
-                                 pair<string,int>("File", 0)));
-    d->split(0.6);
-    d->update_data_list(true, false);
+    inf = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize,
+                         wxTE_RICH|wxTE_READONLY|wxTE_MULTILINE);
     update_info();
-    top_sizer->Add(d, 1, wxEXPAND|wxALL, 1);
+    top_sizer->Add(inf, 1, wxEXPAND|wxALL, 1);
     wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL); 
     dx_cb = new wxCheckBox(this, -1, wxT("merge points with |x1-x2|<"));
     dx_cb->SetValue(true);
@@ -608,8 +640,8 @@ MergePointsDlg::MergePointsDlg(wxWindow* parent, wxWindowID id)
                           make_wxArrStr(wxT("sum"), wxT("avg")),
                           1, wxRA_SPECIFY_ROWS);
     top_sizer->Add(y_rb, 0, wxEXPAND|wxALL, 5);
-    active_ds = ftk->get_active_ds_position();
-    wxString this_ds = wxString::Format(wxT("dataset @%d"), active_ds);
+    focused_data = frame->get_focused_ds_index();
+    wxString this_ds = wxString::Format(wxT("dataset @%d"), focused_data);
     output_rb = new wxRadioBox(this, -1, wxT("output to ..."), 
                                wxDefaultPosition, wxDefaultSize, 
                                make_wxArrStr(this_ds, wxT("new dataset")),
@@ -623,35 +655,28 @@ MergePointsDlg::MergePointsDlg(wxWindow* parent, wxWindowID id)
 
 void MergePointsDlg::update_info()
 {
-    int i = d->list->GetFirstSelected();
-    if (i == -1) {
-        d->inf->Clear();
-        //TODO disable OK button
-        return;
-    }
-    Data const* data = ftk->get_data(i);
+    vector<int> dd = frame->get_selected_ds_indices();
+    Data const* data = ftk->get_data(dd[0]);
     fp x_min = data->get_x_min();
     fp x_max = data->get_x_max();
     int n = data->points().size();
-    wxString dstr = wxString::Format(wxT("@%d"), i);
-    bool only_one = true;
-    while ((i = d->list->GetNextSelected(i)) != -1) {
-        only_one = false;
+    wxString dstr = wxString::Format(wxT("@%d"), dd[0]);
+    for (size_t i = 1; i < dd.size(); ++i) {
         data = ftk->get_data(i);
         if (data->get_x_min() < x_min)
             x_min = data->get_x_min();
         if (data->get_x_max() > x_max)
             x_max = data->get_x_max();
         n += data->points().size();
-        dstr += wxString::Format(wxT(" @%d"), i);
+        dstr += wxString::Format(wxT(" @%d"), (int) i);
     }
     wxString s = wxString::Format(wxT("%i data points from: "), n) + dstr;
     s += wxString::Format(wxT("\nx in range (%g, %g)"), x_min, x_max);
-    if (only_one && data->get_x_step() != 0.)
+    if (dd.size() != 1 && data->get_x_step() != 0.)
         s += wxString::Format(wxT("\nfixed step: %g"), data->get_x_step());
     else
         s += wxString::Format(wxT("\naverage step: %g"), (x_max-x_min) / (n-1));
-    d->inf->SetValue(s);
+    inf->SetValue(s);
 }
 
 string MergePointsDlg::get_command()
@@ -659,10 +684,11 @@ string MergePointsDlg::get_command()
     string s;
     if (dx_cb->GetValue())
         s += "with epsilon=" + wx2s(dx_val->GetValue()) + " ";
-    s += (output_rb->GetSelection() == 0 ? "@"+S(active_ds) : S("@+")) + " = ";
+    string dat = output_rb->GetSelection() == 0 ? S(focused_data) : S("+");
+    s += "@" + dat + " = ";
     if (dx_cb->GetValue())
         s += y_rb->GetSelection() == 0 ? "sum_same_x " : "avg_same_x ";
-    s += join_vector(d->get_selected_data(), " + ");
+    s += "@" + join_vector(frame->get_selected_ds_indices(), " + @");
     return s;
 }
 
