@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: dload.cpp 345 2007-08-21 01:09:15Z wojdyr $
+// $Id: dload.cpp 406 2008-03-10 22:16:45Z wojdyr $
 
 /// In this file:
 ///  Custom Data Load Dialog (DLoadDlg) and helpers
@@ -19,18 +19,21 @@
 #include <wx/file.h>
 #include <wx/filename.h>
 
+#include <xylib/xylib.h>
+
 #include "dload.h" 
-#include "gui.h"  // frame->add_recent_data_file(get_filename()
+#include "frame.h"  // frame->add_recent_data_file(get_filename())
 #include "plot.h" // scale_tics_step()
-#include "../data.h"
 #include "../logic.h" 
 #include "../settings.h"
+#include "../data.h" // get_file_basename()
 #include "../common.h" //iround
 
 using namespace std;
 
 enum {
     ID_DXLOAD_STDDEV_CB     =28000,
+    ID_DXLOAD_BLOCK               ,
     ID_DXLOAD_COLX                ,
     ID_DXLOAD_COLY                ,
     ID_DXLOAD_SDS                 ,
@@ -46,22 +49,29 @@ enum {
 class PreviewPlot : public BufferedPanel
 {
 public:
-    auto_ptr<Data> data;
+    int block_nr, idx_x, idx_y;
 
     PreviewPlot(wxWindow* parent)
-        : BufferedPanel(parent), data(new Data(ftk)) 
-                  { backgroundCol = *wxBLACK; }
+        : BufferedPanel(parent), block_nr(0), idx_x(1), idx_y(2),
+          data(NULL), data_updated(false)
+        { backgroundCol = *wxBLACK; }
+
+    ~PreviewPlot() { delete data; }
     void OnPaint(wxPaintEvent &event);
     void draw(wxDC &dc, bool);
+    void load_dataset(string const& filename, string const& filetype,
+                      vector<string> const& options);
+    xylib::DataSet const* get_data() { return data_updated ? data : NULL; }
+    void make_outdated() { data_updated = false; }
 
 private:
+    xylib::DataSet* data;
+    bool data_updated; // if false, draw() doesn't do anything (plot is clear)
+
     double xScale, yScale;
     double xOffset, yOffset;
-    int H;
     int getX(double x) { return iround(x * xScale + xOffset); }
     int getY(double y) { return iround(y * yScale + yOffset); }
-    void prepare_scaling(wxDC &dc);
-    void draw_scale(wxDC &dc);
 
     DECLARE_EVENT_TABLE()
 };
@@ -77,60 +87,106 @@ void PreviewPlot::OnPaint(wxPaintEvent&)
 
 void PreviewPlot::draw(wxDC &dc, bool)
 {
-    if (data->is_empty())
+    if (!data || !data_updated 
+        || block_nr < 0 || block_nr >= data->get_block_count())
         return;
-    prepare_scaling(dc);
-    draw_scale(dc);
-    vector<Point> const& pp = data->points();
-    dc.SetPen(*wxGREEN_PEN);
-    for (vector<Point>::const_iterator i = pp.begin(); i != pp.end(); ++i)
-        dc.DrawPoint(getX(i->x), getY(i->y));
-}
 
-void PreviewPlot::prepare_scaling(wxDC &dc)
-{
+    xylib::Block const *block = data->get_block(block_nr);
+
+    if (idx_x < 0 || idx_x > block->get_column_count() 
+            || idx_y < 0 || idx_y > block->get_column_count()) 
+        return;
+
+    xylib::Column const& xcol = block->get_column(idx_x);
+    xylib::Column const& ycol = block->get_column(idx_y);
+    const int np = block->get_point_count();
+
+    // prepare scaling
     double const margin = 0.1;
-    double dx = data->get_x_max() - data->get_x_min();
-    double dy = data->get_y_max() - data->get_y_min();
-    int W = GetClientSize().GetWidth();
-    H = GetClientSize().GetHeight();
+    double dx = xcol.get_max(np) - xcol.get_min();
+    double dy = ycol.get_max(np) - ycol.get_min();
+    int W = dc.GetSize().GetWidth();
+    int H = dc.GetSize().GetHeight();
     xScale = (1 - 1.2 * margin) *  W / dx;
     yScale = - (1 - 1.2 * margin) * H / dy;
-    xOffset = - data->get_x_min() * xScale + margin * W;
-    yOffset = H - data->get_y_min() * yScale - margin * H;
-}
-
-void PreviewPlot::draw_scale(wxDC &dc)
-{
+    xOffset = - xcol.get_min() * xScale + margin * W;
+    yOffset = H - ycol.get_min() * yScale - margin * H;
+    
+    // draw scale
     dc.SetPen(*wxWHITE_PEN);
     dc.SetTextForeground(*wxWHITE);
     dc.SetFont(*wxSMALL_FONT);  
+
+    // ... horizontal
     vector<double> minors;
     vector<double> tics 
-        = scale_tics_step(data->get_x_min(), data->get_x_max(), 4, minors);
+        = scale_tics_step(xcol.get_min(), xcol.get_max(np), 4, minors);
     for (vector<double>::const_iterator i = tics.begin(); i != tics.end(); ++i){
         int X = getX(*i);
         wxString label = s2wx(S(*i));
-        if (label == wxT("-0")) 
-            label = wxT("0");
         wxCoord tw, th;
         dc.GetTextExtent (label, &tw, &th);
         int Y = dc.DeviceToLogicalY(H - th - 2);
         dc.DrawText (label, X - tw/2, Y + 1);
         dc.DrawLine (X, Y, X, Y - 4);
     }
+    if (!xcol.name.empty()) {
+        wxCoord tw, th;
+        dc.GetTextExtent (s2wx(xcol.name), &tw, &th);
+        dc.DrawText (s2wx(xcol.name), (W - tw)/2, 2);
+    }
 
-    tics = scale_tics_step(data->get_y_min(), data->get_y_max(), 4, minors);
+    // ... vertical
+    tics = scale_tics_step(ycol.get_min(), ycol.get_max(np), 4, minors);
     for (vector<double>::const_iterator i = tics.begin(); i != tics.end(); ++i){
         int Y = getY(*i);
         wxString label = s2wx(S(*i));
-        if (label == wxT("-0")) 
-            label = wxT("0");
         wxCoord tw, th;
         dc.GetTextExtent (label, &tw, &th);
         dc.DrawText (label, dc.DeviceToLogicalX(5), Y - th/2);
         dc.DrawLine (dc.DeviceToLogicalX(0), Y, dc.DeviceToLogicalX(4), Y);
     }
+    if (!ycol.name.empty()) {
+        wxCoord tw, th;
+        dc.GetTextExtent (s2wx(ycol.name), &tw, &th);
+        dc.DrawRotatedText (s2wx(ycol.name), W - 2, (H - tw)/2, 270);
+    }
+
+    // draw data
+    dc.SetPen(*wxGREEN_PEN);
+    for (int i = 0; i < np; ++i)
+        dc.DrawPoint(getX(xcol.get_value(i)), getY(ycol.get_value(i)));
+}
+
+
+// wrapper around xylib::load_file()
+// with added caching (if the same filename and options are given)
+void PreviewPlot::load_dataset(string const& filename, 
+                               string const& filetype,
+                               vector<string> const& options)
+{
+    static string old_filename;
+    static string old_filetype;
+    static vector<string> old_options;
+    if (filename == old_filename && filetype == old_filetype 
+                                                && options == old_options) {
+        data_updated = true;
+        return;
+    }
+
+    try {
+        // if xylib::load_file() throws exception, we keep value of data
+        xylib::DataSet *new_ds = xylib::load_file(filename, filetype, options);
+        assert(new_ds);
+        delete data;
+        data = new_ds;
+        old_filename = filename;
+        old_filetype = filetype;
+        old_options = options;
+        data_updated = true;
+    } catch (runtime_error const& e) {
+        data_updated = false;
+    }
 }
 
 
@@ -141,6 +197,7 @@ BEGIN_EVENT_TABLE(DLoadDlg, wxDialog)
     EVT_CHECKBOX    (ID_DXLOAD_AUTO_PLOT, DLoadDlg::OnAutoPlotCheckBox)
     EVT_SPINCTRL    (ID_DXLOAD_COLX,      DLoadDlg::OnColumnChanged)
     EVT_SPINCTRL    (ID_DXLOAD_COLY,      DLoadDlg::OnColumnChanged)
+    EVT_CHOICE      (ID_DXLOAD_BLOCK,     DLoadDlg::OnBlockChanged)
     EVT_BUTTON      (wxID_CLOSE,          DLoadDlg::OnClose)
     EVT_BUTTON      (ID_DXLOAD_OPENHERE,  DLoadDlg::OnOpenHere)
     EVT_BUTTON      (ID_DXLOAD_OPENNEW,   DLoadDlg::OnOpenNew)
@@ -148,9 +205,11 @@ BEGIN_EVENT_TABLE(DLoadDlg, wxDialog)
     EVT_TEXT_ENTER(ID_DXLOAD_FN,          DLoadDlg::OnPathTextChanged)
 END_EVENT_TABLE()
 
+/// n - data slot to be used by "Replace ..." button, -1 means none
+/// data - used for default settings (path, columns, etc.), not NULL
 DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     : wxDialog(parent, id, wxT("Data load (custom)"), 
-               wxDefaultPosition, wxSize(600, 500), 
+               wxDefaultPosition, wxDefaultSize, 
                wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),
       data_nr(n), initialized(false)
 {
@@ -189,15 +248,8 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
 //#else
                        0,
 //#endif
-                       // multiple wildcards, eg. 
-                       // |*.dat;*.DAT;*.xy;*.XY;*.fio;*.FIO
-                       // are not supported by wxGenericDirCtrl  
-                       wxT("all files (*)|*")
-                       wxT("|ASCII x y files (*)|*" )
-                       wxT("|rit files (*.rit)|*.rit")
-                       wxT("|cpi files (*.cpi)|*.cpi")
-                       wxT("|mca files (*.mca)|*.mca")
-                       wxT("|Siemens/Bruker (*.raw)|*.raw"));
+                       wxT("All Files (*)|*|") 
+                           + s2wx(xylib::get_wildcards_string()));
     left_sizer->Add(dir_ctrl, 1, wxALL|wxEXPAND, 5);
     wxFileName path = s2wx(data->get_filename());
     path.Normalize();
@@ -206,6 +258,10 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     left_sizer->Add (filename_tc, 0, wxALL|wxEXPAND, 5);
                                      
 
+    // selecting block
+    block_ch = new wxChoice(left_panel, ID_DXLOAD_BLOCK);
+    left_sizer->Add(block_ch, 0, wxALL|wxEXPAND, 5);
+
     // selecting columns
     columns_panel = new wxPanel (left_panel, -1);
     wxStaticBoxSizer *h2a_sizer = new wxStaticBoxSizer(wxHORIZONTAL, 
@@ -226,13 +282,13 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     h2a_sizer->Add(s_column, 0, wxALL|wxALIGN_LEFT, 5);
     columns_panel->SetSizer(h2a_sizer);
     left_sizer->Add (columns_panel, 0, wxALL|wxEXPAND, 5);
-    if (data->get_given_cols().size() > 1) {
-        x_column->SetValue(data->get_given_cols()[0]);
-        y_column->SetValue(data->get_given_cols()[1]);
-        if (data->get_given_cols().size() > 2) {
-            std_dev_cb->SetValue(true);
-            s_column->SetValue(data->get_given_cols()[2]);
-        }
+    if (data->get_given_x() != INT_MAX) 
+        x_column->SetValue(data->get_given_x());
+    if (data->get_given_y() != INT_MAX) 
+        y_column->SetValue(data->get_given_y());
+    if (data->get_given_s() != INT_MAX) {
+        std_dev_cb->SetValue(true);
+        s_column->SetValue(data->get_given_s());
     }
 
     bool def_sqrt = (ftk->get_settings()->getp("data-default-sigma") == "sqrt");
@@ -242,11 +298,12 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     left_sizer->Add (sd_sqrt_cb, 0, wxALL|wxEXPAND, 5);
 
     wxStaticBoxSizer *dt_sizer = new wxStaticBoxSizer(wxVERTICAL, 
-                                    left_panel, wxT("Data title (optional):"));
-    htitle_cb = new wxCheckBox(left_panel, ID_DXLOAD_HTITLE, 
-                               wxT("get from 1st line"));
-    dt_sizer->Add(htitle_cb, 0, wxALL, 5);
+                                    left_panel, wxT("Data title:"));
+    title_cb = new wxCheckBox(left_panel, ID_DXLOAD_HTITLE, 
+                              wxT("custom title"));
+    dt_sizer->Add(title_cb, 0, wxALL, 5);
     title_tc = new wxTextCtrl(left_panel, -1, wxT(""));
+    title_tc->Enable(false);
     dt_sizer->Add(title_tc, 0, wxALL|wxEXPAND, 5);
     left_sizer->Add (dt_sizer, 0, wxALL|wxEXPAND, 5);
 
@@ -267,7 +324,7 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     rbottom_sizer->Add(plot_preview, 1, wxEXPAND|wxALL, 5);
     auto_plot_cb = new wxCheckBox(rbottom_panel, ID_DXLOAD_AUTO_PLOT, 
                                   wxT("plot"));
-    auto_plot_cb->SetValue(false);
+    auto_plot_cb->SetValue(true);
     rbottom_sizer->Add(auto_plot_cb, 0, wxALL, 5);
 
     // ------ finishing layout (+buttons) -----------
@@ -280,18 +337,22 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
 
     top_sizer->Add (new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 5);
     wxBoxSizer *button_sizer = new wxBoxSizer(wxHORIZONTAL);
+    string data_nr_str = (data_nr >= 0 ? S(data_nr) : S("?"));
     open_here = new wxButton(this, ID_DXLOAD_OPENHERE, 
-                             s2wx("&Replace dataset @"+S(data_nr)));
+                                        wxT("&Replace @") + s2wx(data_nr_str));
+    if (data_nr < 0)
+        open_here->Enable(false);
     open_new = new wxButton(this, ID_DXLOAD_OPENNEW, 
-                            wxT("&Open as new dataset"));
+                            wxT("&Open in new slot"));
     button_sizer->Add(open_here, 0, wxALL, 5);
     button_sizer->Add(open_new, 0, wxALL, 5);
     button_sizer->Add(new wxButton(this, wxID_CLOSE, wxT("&Close")), 
                       0, wxALL, 5);
     top_sizer->Add(button_sizer, 0, wxALL|wxALIGN_CENTER, 0);
     initialized = true;
-    SetSizer(top_sizer);
-    on_filter_change();
+    SetSizerAndFit(top_sizer);
+    SetSize(wxSize(700, 600));
+    update_block_list();
 }
 
 void DLoadDlg::StdDevCheckBoxChanged()
@@ -303,17 +364,41 @@ void DLoadDlg::StdDevCheckBoxChanged()
 
 void DLoadDlg::OnHTitleCheckBox (wxCommandEvent& event)
 {
-    if (event.IsChecked()) 
+    if (!event.IsChecked()) 
         update_title_from_file();
-    title_tc->Enable(!event.IsChecked());
+    title_tc->Enable(event.IsChecked());
+}
+
+void DLoadDlg::update_block_list()
+{
+    vector<string> bb;
+    if (plot_preview && plot_preview->get_data())
+        for (int i = 0; i < plot_preview->get_data()->get_block_count(); ++i) {
+            const string& name = plot_preview->get_data()->get_block(i)->name;
+            bb.push_back(name.empty() ? "Block #" + S(i+1) : name);
+        }
+    else {
+        bb.push_back("<default block>");
+    }
+    updateControlWithItems(block_ch, bb);
+    block_ch->SetSelection(0);
+    block_ch->Enable(block_ch->GetCount() > 1);
 }
 
 void DLoadDlg::update_title_from_file()
 {
-    assert (htitle_cb->GetValue());
-    ifstream f(dir_ctrl->GetFilePath().fn_str());
-    int col = columns_panel->IsEnabled() ? y_column->GetValue() : 0;
-    string title = Data::read_one_line_as_title(f, col);
+    if (title_cb->GetValue()) 
+        return;
+    string path = wx2s(dir_ctrl->GetFilePath());
+    if (path.empty()) {
+        title_tc->Clear();
+        return;
+    }
+    string title = get_file_basename(path);
+    int x = x_column->GetValue();
+    int y = y_column->GetValue();
+    if (x != 1 || y != 2 || std_dev_cb->GetValue()) 
+        title += ":" + S(x) + ":" + S(y);
     title_tc->SetValue(s2wx(title));
 }
 
@@ -325,7 +410,14 @@ void DLoadDlg::OnAutoTextCheckBox (wxCommandEvent& event)
         text_preview->Clear();
 }
 
-void DLoadDlg::OnAutoPlotCheckBox (wxCommandEvent&)
+void DLoadDlg::OnAutoPlotCheckBox (wxCommandEvent& event)
+{
+    update_plot_preview();
+    if (event.IsChecked()) 
+        update_title_from_file();
+}
+
+void DLoadDlg::OnBlockChanged (wxCommandEvent&)
 {
     update_plot_preview();
 }
@@ -334,8 +426,7 @@ void DLoadDlg::OnColumnChanged (wxSpinEvent&)
 {
     if (auto_plot_cb->GetValue()) 
         update_plot_preview();
-    if (htitle_cb->GetValue()) 
-        update_title_from_file();
+    update_title_from_file();
 }
 
 void DLoadDlg::OnClose (wxCommandEvent&)
@@ -374,64 +465,32 @@ void DLoadDlg::update_text_preview()
 void DLoadDlg::update_plot_preview()
 {
     if (auto_plot_cb->GetValue()) {
-        std::vector<int> cols;
-        if (columns_panel->IsEnabled()) {
-            cols.push_back(x_column->GetValue());
-            cols.push_back(y_column->GetValue());
-        }
-        ftk->get_ui()->keep_quiet = true;
-        try {
-            plot_preview->data->load_file(wx2s(dir_ctrl->GetFilePath()), 
-                                          "", cols, true);
-        } catch (ExecuteError&) {
-            plot_preview->data->clear();
-        }
-        ftk->get_ui()->keep_quiet = false;
-    }
-    else {
-        plot_preview->data->clear();
+        plot_preview->idx_x = x_column->GetValue();
+        plot_preview->idx_y = y_column->GetValue();
+        plot_preview->block_nr = block_ch->GetSelection();
+        plot_preview->load_dataset(wx2s(dir_ctrl->GetFilePath()), 
+                                   "", vector<string>());
     }
+    else 
+        plot_preview->make_outdated();
     plot_preview->refresh();
 }
 
-void DLoadDlg::on_filter_change()
-{
-    int idx = dir_ctrl->GetFilterIndex();
-    wxString path = dir_ctrl->GetFilePath();
-    bool is_text = (idx == 0 && !path.IsEmpty() 
-                             && Data::guess_file_type(wx2s(path)) == "text")
-                    || idx == 1; 
-    enable_text_options(is_text);
-}
-
-void DLoadDlg::enable_text_options(bool is_text)
-{
-    columns_panel->Enable(is_text);
-    htitle_cb->Enable(is_text);
-    if (!is_text && htitle_cb->IsChecked())
-        htitle_cb->SetValue(false);
-    sd_sqrt_cb->Enable(!(is_text  && std_dev_cb->GetValue()));
-}
-
 void DLoadDlg::on_path_change()
 {
     if (!initialized)
         return;
     wxString path = dir_ctrl->GetFilePath();
     filename_tc->SetValue(path);
-    if (dir_ctrl->GetFilterIndex() == 0) { // all files
-        bool is_text = !path.IsEmpty() 
-                             && Data::guess_file_type(wx2s(path)) == "text"; 
-        enable_text_options(is_text);
-    }
     open_here->Enable(!path.IsEmpty());
     open_new->Enable(!path.IsEmpty());
     if (auto_text_cb->GetValue())
         update_text_preview();
+    block_ch->SetSelection(0);
     if (auto_plot_cb->GetValue()) 
         update_plot_preview();
-    if (htitle_cb->GetValue()) 
-        update_title_from_file();
+    update_block_list();
+    update_title_from_file();
 }
 
 void DLoadDlg::OnPathTextChanged(wxCommandEvent&)
@@ -455,25 +514,27 @@ string DLoadDlg::get_command(string const& ds, int d_nr)
     string cmd;
 
     string cols;
-    if (columns_panel->IsEnabled()) { // a:b[:c]
-        int x = x_column->GetValue();
-        int y = y_column->GetValue();
-        bool has_s = std_dev_cb->GetValue();
-        // default parameter values are not passed explicitely
-        if (x != 1 || y != 2 || has_s) {
-            cols = " " + S(x) + "," + S(y);
-            if (has_s)
-                cols += S(",") + S(s_column->GetValue());
-        }
+    int x = x_column->GetValue();
+    int y = y_column->GetValue();
+    bool has_s = std_dev_cb->GetValue();
+    int b = block_ch->GetSelection();
+    // default parameter values are not passed explicitely
+    if (x != 1 || y != 2 || has_s || b != 0) {
+        cols = ":" + S(x) + ":" + S(y) + ":";
+        if (has_s)
+            cols += S(s_column->GetValue());
+        cols += ":";
+        if (b != 0)
+            cols += S(b);
     }
 
     string filetype;
-    if (htitle_cb->IsChecked())
-        filetype = " htext";
+    //if (title_cb->IsChecked())
+    //    filetype = " text, first-line-header";
 
     bool def_sqrt = (ftk->get_settings()->getp("data-default-sigma") == "sqrt");
     bool set_sqrt = sd_sqrt_cb->GetValue();
-    bool sigma_in_file = (columns_panel->IsEnabled() && std_dev_cb->GetValue());
+    bool sigma_in_file = std_dev_cb->GetValue();
     if (!sigma_in_file && set_sqrt != def_sqrt) {
         if (set_sqrt)
             cmd = "with data-default-sigma=sqrt ";
@@ -481,12 +542,12 @@ string DLoadDlg::get_command(string const& ds, int d_nr)
             cmd = "with data-default-sigma=one ";
     }
 
-    cmd += ds + " < '" + get_filename() + "'" + filetype + cols;
+    cmd += ds + " < '" + get_filename() + cols + "'" + filetype;
 
     if (title_tc->IsEnabled()) {
         wxString t = title_tc->GetValue().Trim();
         if (!t.IsEmpty())
-            cmd += "; @" + S(d_nr) + ".title = '" + wx2s(t) + "'";
+            cmd += "; set @" + S(d_nr) + ".title = '" + wx2s(t) + "'";
     }
 
     return cmd;
