@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: dload.h 276 2007-04-01 00:45:41Z wojdyr $
+// $Id: dload.h 399 2008-03-01 14:15:02Z wojdyr $
 
 #ifndef FITYK__WX_DLOAD__H__
 #define FITYK__WX_DLOAD__H__
@@ -28,8 +28,9 @@ protected:
     wxSpinCtrl *x_column, *y_column, *s_column;
     wxPanel *left_panel, *rupper_panel, *rbottom_panel, *columns_panel;
     PreviewPlot *plot_preview;
-    wxCheckBox *std_dev_cb, *sd_sqrt_cb, *htitle_cb, *auto_text_cb, 
+    wxCheckBox *std_dev_cb, *sd_sqrt_cb, *title_cb, *auto_text_cb, 
                *auto_plot_cb;
+    wxChoice *block_ch;
     wxButton *open_here, *open_new;
     bool initialized;
 
@@ -41,16 +42,16 @@ protected:
     void OnAutoTextCheckBox (wxCommandEvent& event);
     void OnAutoPlotCheckBox (wxCommandEvent& event);
     void OnColumnChanged (wxSpinEvent& event);
+    void OnBlockChanged (wxCommandEvent& event);
     void OnOpenHere (wxCommandEvent& event);
     void OnOpenNew (wxCommandEvent& event);
     void OnClose (wxCommandEvent& event);
     void on_path_change();
-    void on_filter_change();
-    void enable_text_options(bool is_text);
     void OnPathSelectionChanged(wxTreeEvent&) { on_path_change(); }
     void OnPathTextChanged(wxCommandEvent&);
     void update_text_preview();
     void update_plot_preview();
+    void update_block_list();
     void update_title_from_file();
     DECLARE_EVENT_TABLE()
 };
