@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: listptxt.cpp 296 2007-05-24 20:04:00Z wojdyr $
+// $Id: listptxt.cpp 403 2008-03-10 15:02:28Z wojdyr $
 
 /// In this file:
 ///  part of the sidebar (ListPlusText, which contains ListWithColors)
@@ -17,7 +17,7 @@
 #include "listptxt.h"
 #include "cmn.h" //SpinCtrl, ProportionalSplitter
 #include "sidebar.h" //SideBar::delete_selected_items()
-#include "gui.h" // frame->focus_input() and used in update_data_list()
+#include "frame.h" // frame->focus_input() and used in update_data_list()
 #include "mplot.h" // used in update_data_list()
 #include "../common.h" // s2wx
 #include "../logic.h" // used in update_data_list()
@@ -86,12 +86,12 @@ void ListWithColors::populate(vector<string> const& data,
                 ++c;
             }
         }
-        if (active != -2)
-            Select(i, i == active);
+        //if (active != -2)
+        //    Select(i, i == active);
     }
     list_data = data;
     
-    if (active >= 0)
+    if (active >= 0 && active < length)
         Focus(active);
     Thaw();
 }
@@ -189,8 +189,11 @@ void ListPlusText::OnSwitchInfo(wxCommandEvent&)
 
 //===============================================================
 
-void DataListPlusText::update_data_list(bool nondata_changed, bool with_stats)
+void DataListPlusText::update_data_list(bool nondata_changed)
 {
+    if (!frame)
+        return;
+
     MainPlot const* mplot = frame->get_main_plot();
     wxColour const& bg_col = mplot->get_bg_color();
 
@@ -198,8 +201,7 @@ void DataListPlusText::update_data_list(bool nondata_changed, bool with_stats)
     for (int i = 0; i < ftk->get_ds_count(); ++i) {
         DataWithSum const* ds = ftk->get_ds(i);
         data_data.push_back(S(i));
-        if (with_stats)
-            data_data.push_back(S(ds->get_sum()->get_ff_names().size()) 
+        data_data.push_back(S(ds->get_sum()->get_ff_names().size()) 
                             + "+" + S(ds->get_sum()->get_zz_names().size()));
         data_data.push_back(ds->get_data()->get_title());
         data_data.push_back(ds->get_data()->get_filename());
@@ -211,8 +213,12 @@ void DataListPlusText::update_data_list(bool nondata_changed, bool with_stats)
             data_images->Add(make_color_bitmap16(mplot->get_data_color(i), 
                                                  bg_col));
     }
-    int active_ds_pos = ftk->get_active_ds_position();
-    list->populate(data_data, data_images, active_ds_pos);
+    int focused = list->GetFocusedItem();
+    if (focused < 0) 
+        focused = 0;
+    else if (focused >= list->GetItemCount())
+        focused = list->GetItemCount() - 1;
+    list->populate(data_data, data_images, focused);
 }
 
 
@@ -221,10 +227,10 @@ vector<string> DataListPlusText::get_selected_data() const
     vector<string> dd;
     for (int i=list->GetFirstSelected(); i != -1; i = list->GetNextSelected(i))
         dd.push_back("@" + S(i));
-    if (dd.empty()) {
-        int n = list->GetFocusedItem();
-        dd.push_back("@" + S(n == -1 ? 0 : n));
-    }
+    //if (dd.empty()) {
+    //    int n = list->GetFocusedItem();
+    //    dd.push_back("@" + S(n == -1 ? 0 : n));
+    //}
     return dd;
 }
 
