@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: listptxt.h 312 2007-07-04 20:30:40Z wojdyr $
+// $Id: listptxt.h 356 2007-11-20 18:58:33Z wojdyr $
 
 #ifndef FITYK__WX_LISTPTXT__H__
 #define FITYK__WX_LISTPTXT__H__
@@ -58,7 +58,7 @@ public:
     DataListPlusText(wxWindow *parent, wxWindowID id, wxWindowID list_id,
                  std::vector<std::pair<std::string,int> > const& columns_)
         : ListPlusText(parent, id, list_id, columns_) {}
-    void update_data_list(bool nondata_changed, bool with_stats);
+    void update_data_list(bool nondata_changed);
     std::vector<std::string> get_selected_data() const;
 };
 
