@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: mplot.cpp 326 2007-08-01 05:54:03Z wojdyr $
+// $Id: mplot.cpp 424 2008-04-15 11:08:42Z wojdyr $
 
 #include <wx/wxprec.h>
 #ifdef __BORLANDC__
@@ -14,7 +14,7 @@
 #include <algorithm>
 
 #include "mplot.h"
-#include "gui.h"
+#include "frame.h"
 #include "sidebar.h"
 #include "../data.h" 
 #include "../logic.h"
@@ -233,7 +233,7 @@ BEGIN_EVENT_TABLE(MainPlot, FPlot)
 END_EVENT_TABLE()
 
 MainPlot::MainPlot (wxWindow *parent) 
-    : FPlot(parent), BgManager(xs),
+    : FPlot(parent), bgm(xs),
       basic_mode(mmd_zoom), mode(mmd_zoom), 
       pressed_mouse_button(0), ctrl_on_down(false), shift_on_down(false),
       over_peak(-1), limit1(INT_MIN), limit2(INT_MIN)
@@ -279,27 +279,24 @@ void MainPlot::draw_dataset(wxDC& dc, int n, bool set_pen)
 
 void MainPlot::draw(wxDC &dc, bool monochrome)
 {
-    int focused_data = ftk->get_active_ds_position();
+    int focused_data = frame->get_focused_ds_index();
     Sum const* sum = ftk->get_sum(focused_data);
 
-    set_scale();
+    set_scale(get_pixel_width(dc), get_pixel_height(dc));
 
     frame->draw_crosshair(-1, -1); //erase crosshair before redrawing plot
 
-    prepare_peaktops(sum);
+    int Ymax = get_pixel_height(dc);
+    prepare_peaktops(sum, Ymax);
 
     if (monochrome) {
         dc.SetPen(*wxBLACK_PEN);
         dc.SetBrush(*wxBLACK_BRUSH);
     }
-    //draw datasets
-    for (int i = 0; i < ftk->get_ds_count(); i++) {
-        if (i != focused_data) {
-            draw_dataset(dc, i, !monochrome);
-        }
-    }
-    // focused dataset is drawed at the end (to be at the top)
-    draw_dataset(dc, focused_data, !monochrome);
+    //draw datasets (selected and focused at the end)
+    vector<int> ord = frame->get_sidebar()->get_ordered_dataset_numbers();
+    for (vector<int>::const_iterator i = ord.begin(); i != ord.end(); ++i)
+        draw_dataset(dc, *i, !monochrome);
 
     if (xtics_visible)
         draw_xtics(dc, ftk->view, !monochrome);
@@ -338,24 +335,24 @@ bool MainPlot::visible_peaktops(MouseModeEnum mode)
 void MainPlot::draw_x_axis (wxDC& dc, bool set_pen)
 {
     if (set_pen)
-        dc.SetPen(wxPen(xAxisCol));
+        dc.SetPen(wxPen(xAxisCol, pen_width));
     int Y0 = ys.px(0.);
-    dc.DrawLine (0, Y0, GetClientSize().GetWidth(), Y0);
+    dc.DrawLine (0, Y0, get_pixel_width(dc), Y0);
 }
 
 void MainPlot::draw_y_axis (wxDC& dc, bool set_pen)
 {
     if (set_pen)
-        dc.SetPen(wxPen(xAxisCol));
+        dc.SetPen(wxPen(xAxisCol, pen_width));
     int X0 = xs.px(0.);
-    dc.DrawLine (X0, 0, X0, GetClientSize().GetHeight());
+    dc.DrawLine (X0, 0, X0, get_pixel_height(dc));
 }
 
 void MainPlot::draw_sum(wxDC& dc, Sum const* sum, bool set_pen)
 {
     if (set_pen)
-        dc.SetPen(wxPen(sumCol));
-    int n = GetClientSize().GetWidth();
+        dc.SetPen(wxPen(sumCol, pen_width));
+    int n = get_pixel_width(dc);
     vector<fp> xx(n), yy(n);
     vector<int> YY(n);
     for (int i = 0; i < n; ++i) 
@@ -365,6 +362,7 @@ void MainPlot::draw_sum(wxDC& dc, Sum const* sum, bool set_pen)
         YY[i] = ys.px(yy[i]);
     for (int i = 1; i < n; i++) 
         dc.DrawLine (i-1, YY[i-1], i, YY[i]); 
+    // perhaps wxDC::DrawLines() would be faster?
 }
 
 
@@ -377,7 +375,7 @@ void MainPlot::draw_peaks(wxDC& dc, Sum const* sum, bool set_pen)
 {
     fp level = 0;
     vector<int> const& idx = sum->get_ff_idx();
-    int n = GetClientSize().GetWidth();
+    int n = get_pixel_width(dc);
     vector<fp> xx(n), yy(n);
     vector<int> YY(n);
     for (int i = 0; i < n; ++i) 
@@ -392,7 +390,7 @@ void MainPlot::draw_peaks(wxDC& dc, Sum const* sum, bool set_pen)
             to = min(to, xs.px(right));
         }
         if (set_pen)
-            dc.SetPen(wxPen(peakCol[k % max_peak_cols]));
+            dc.SetPen(wxPen(peakCol[k % max_peak_cols], pen_width));
         f->calculate_value(xx, yy);
         for (int i = from; i <= to; ++i) 
             YY[i] = ys.px(yy[i]);
@@ -403,7 +401,7 @@ void MainPlot::draw_peaks(wxDC& dc, Sum const* sum, bool set_pen)
 
 void MainPlot::draw_peaktops (wxDC& dc, Sum const* sum)
 {
-    dc.SetPen(wxPen(xAxisCol));
+    dc.SetPen(wxPen(xAxisCol, pen_width));
     dc.SetBrush (*wxTRANSPARENT_BRUSH);
     for (vector<wxPoint>::const_iterator i = special_points.begin(); 
                                            i != special_points.end(); i++) {
@@ -430,7 +428,7 @@ void MainPlot::draw_peaktop_selection (wxDC& dc, Sum const* sum)
 void MainPlot::draw_plabels (wxDC& dc, Sum const* sum, bool set_pen)
 {
     prepare_peak_labels(sum); //TODO re-prepare only when peaks where changed
-    dc.SetFont(plabelFont);
+    set_font(dc, plabelFont);
     vector<wxRect> previous;
     vector<int> const& idx = sum->get_ff_idx();
     for (int k = 0; k < size(idx); k++) {
@@ -483,9 +481,8 @@ static bool operator< (const wxPoint& a, const wxPoint& b)
 }
 */
 
-void MainPlot::prepare_peaktops(Sum const* sum)
+void MainPlot::prepare_peaktops(Sum const* sum, int Ymax)
 {
-    int H =  GetClientSize().GetHeight();
     int Y0 = ys.px(0);
     vector<int> const& idx = sum->get_ff_idx();
     int n = idx.size();
@@ -505,7 +502,7 @@ void MainPlot::prepare_peaktops(Sum const* sum)
         }
         //FIXME: check if these zero_shift()'s above are needed
         Y = ys.px(f->calculate_value(x));
-        if (Y < 0 || Y > H) 
+        if (Y < 0 || Y > Ymax) 
             Y = Y0;
         special_points[k] = wxPoint(X, Y);
     }
@@ -550,19 +547,23 @@ void MainPlot::prepare_peak_labels(Sum const* sum)
 void MainPlot::draw_background(wxDC& dc, bool set_pen)
 {
     if (set_pen) 
-        dc.SetPen(wxPen(bg_pointsCol));
+        dc.SetPen(wxPen(bg_pointsCol, pen_width));
     dc.SetBrush (*wxTRANSPARENT_BRUSH);
+
     // bg line
     int X = -1, Y = -1;
-    for (vector<t_xy>::const_iterator i=bgline.begin(); i != bgline.end(); i++){
+    int width = get_pixel_width(dc);
+    vector<int> bgline = bgm.calculate_bgline(width, ys);
+    for (int i = 0; i < width; ++i) {
         int X_ = X, Y_ = Y;
-        X = xs.px(i->x);
-        Y = ys.px(i->y);
+        X = i;
+        Y = bgline[i];
         if (X_ >= 0 && (X != X_ || Y != Y_)) 
             dc.DrawLine (X_, Y_, X, Y); 
     }
+
     // bg points (circles)
-    for (bg_const_iterator i = bg.begin(); i != bg.end(); i++) {
+    for (BgManager::bg_const_iterator i = bgm.begin(); i != bgm.end(); i++) {
         dc.DrawCircle(xs.px(i->x), ys.px(i->y), 3);
         dc.DrawCircle(xs.px(i->x), ys.px(i->y), 4);
     }
@@ -573,8 +574,7 @@ void MainPlot::read_settings(wxConfigBase *cf)
     cf->SetPath(wxT("/MainPlot/Colors"));
     backgroundCol = cfg_read_color(cf, wxT("bg"), wxColour(wxT("BLACK")));
     for (int i = 0; i < max_data_cols; i++)
-        dataColour[i] = cfg_read_color(cf, 
-                                            wxString::Format(wxT("data/%i"), i),
+        dataColour[i] = cfg_read_color(cf, wxString::Format(wxT("data/%i"), i),
                                                wxColour(0, 255, 0));
     inactiveDataCol = cfg_read_color(cf,wxT("inactive_data"),
                                                       wxColour (128, 128, 128));
@@ -607,6 +607,7 @@ void MainPlot::read_settings(wxConfigBase *cf)
     ys.reversed = cfg_read_bool (cf, wxT("yReversed"), false); 
     xs.logarithm = cfg_read_bool (cf, wxT("xLogarithm"), false); 
     ys.logarithm = cfg_read_bool (cf, wxT("yLogarithm"), false); 
+    ftk->view.set_log_scale(xs.logarithm, ys.logarithm);
     FPlot::read_settings(cf);
     refresh();
 }
@@ -730,7 +731,8 @@ void MainPlot::OnPeakGuess(wxCommandEvent&)
             fp plusmin = max(fabs(p->fwhm()), p->iwidth());    
             ftk->exec(p->xname + " = guess " + p->type_name + " [" 
                              + S(ctr-plusmin) + ":" + S(ctr+plusmin) + "]"
-                             + frame->get_in_dataset());
+                             + frame->get_global_parameters()
+                             + frame->get_in_datasets());
         }
     }
 }
@@ -836,7 +838,7 @@ void MainPlot::OnMouseMove(wxMouseEvent &event)
 
 void MainPlot::look_for_peaktop (wxMouseEvent& event)
 {
-    int focused_data = ftk->get_active_ds_position();
+    int focused_data = frame->get_focused_ds_index();
     Sum const* sum = ftk->get_sum(focused_data);
     vector<int> const& idx = sum->get_ff_idx();
     if (special_points.size() != idx.size()) 
@@ -897,6 +899,7 @@ void MainPlot::cancel_mouse_press()
 void MainPlot::OnButtonDown (wxMouseEvent &event)
 {
     if (pressed_mouse_button) {
+        // if one button is already down, pressing other button cancels action
         cancel_mouse_press();
         return;
     }
@@ -910,7 +913,7 @@ void MainPlot::OnButtonDown (wxMouseEvent &event)
     mouse_press_Y = event.GetY();
     fp x = xs.val (event.GetX());
     fp y = ys.val (event.GetY());
-    if (button == 1 && (ctrl_on_down || mode == mmd_zoom) || button == 2) {
+    if ((button == 1 && (ctrl_on_down || mode == mmd_zoom)) || button == 2) {
         draw_temporary_rect(mat_start, event.GetX(), event.GetY());
         SetCursor(wxCURSOR_MAGNIFIER);  
         frame->set_status_text("Select second corner to zoom...");
@@ -930,11 +933,11 @@ void MainPlot::OnButtonDown (wxMouseEvent &event)
         cancel_mouse_press();
     }
     else if (button == 1 && mode == mmd_bg) {
-        add_background_point(x, y);
+        bgm.add_background_point(x, y);
         refresh(false);
     }
     else if (button == 3 && mode == mmd_bg) {
-        rm_background_point(x);
+        bgm.rm_background_point(x);
         refresh(false);
     }
     else if (button == 1 && mode == mmd_add) {
@@ -951,8 +954,7 @@ void MainPlot::OnButtonDown (wxMouseEvent &event)
     }
     else if (button != 2 && mode == mmd_range) {
         if (button == 1) {
-            Data const* data = ftk->get_data(ftk->get_active_ds_position());
-            if (!data->is_empty() && data->get_n() == size(data->points())) {
+            if (!can_disactivate()) {
                 cancel_mouse_press();
                 wxMessageBox(
                  wxT("You pressed the left mouse button in data-range mode,")
@@ -984,6 +986,18 @@ void MainPlot::OnButtonDown (wxMouseEvent &event)
     update_mouse_hints();
 }
 
+bool MainPlot::can_disactivate()
+{
+    vector<int> sel = frame->get_selected_ds_indices();
+    for (vector<int>::const_iterator i = sel.begin(); i != sel.end(); ++i) {
+        Data const* data = ftk->get_data(*i);
+        // if data->is_empty() we allow to try disactivate data to let user
+        // experiment with mouse right after launching the program
+        if (data->is_empty() || data->get_n() != size(data->points())) 
+            return true;
+    }
+    return false;
+}
 
 void MainPlot::OnButtonUp (wxMouseEvent &event)
 {
@@ -995,7 +1009,9 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
     int dist_X = abs(event.GetX() - mouse_press_X);  
     int dist_Y = abs(event.GetY() - mouse_press_Y);  
     // if Down and Up events are at the same position -> cancel
-    if (button == 1 && (ctrl_on_down || mode == mmd_zoom) || button == 2) {
+
+    // zoom
+    if ((button == 1 && (ctrl_on_down || mode == mmd_zoom)) || button == 2) {
         draw_temporary_rect(mat_stop);
         if (dist_X + dist_Y >= 10) {
             fp x1 = xs.val(mouse_press_X);
@@ -1009,6 +1025,7 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
         }
         frame->set_status_text("");
     }
+    // drag peak
     else if (mode == mmd_peak && button == 1) {
         if (dist_X + dist_Y >= 2) {
             string cmd = fmd.get_cmd();
@@ -1018,6 +1035,7 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
         draw_moving_func(mat_stop);
         frame->set_status_text("");
     }
+    // activate or disactivate data
     else if (mode == mmd_range && button != 2) {
         vert_line_following_cursor(mat_stop);
         draw_temporary_rect(mat_stop);
@@ -1026,7 +1044,7 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
             fp xmin = xs.val (min (event.GetX(), mouse_press_X));
             fp xmax = xs.val (max (event.GetX(), mouse_press_X));
             string cond = "(" + S(xmin) + "< x <" + S(xmax) + ")";
-            ftk->exec(c + cond + frame->get_in_one_or_all_datasets());
+            ftk->exec(c + cond + frame->get_in_datasets());
         }
         else if (shift_on_down && dist_X + dist_Y >= 10) {
             fp x1 = xs.val(mouse_press_X);
@@ -1035,33 +1053,18 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
             fp y2 = ys.val(event.GetY());
             string cond = "(" + S(min(x1,x2)) + " < x < " + S(max(x1,x2)) 
                    + " and " + S(min(y1,y2)) + " < y < " + S(max(y1,y2)) + ")";
-            ftk->exec(c + cond + frame->get_in_one_or_all_datasets());
+            ftk->exec(c + cond + frame->get_in_datasets());
         }
         frame->set_status_text("");
     }
+    // add peak (left button)
     else if (mode == mmd_add && button == 1) {
         frame->set_status_text("");
         peak_draft(mat_stop, event.GetX(), event.GetY());
-        string F = ftk->get_ds_count() > 1 ?  frame->get_active_data_str()+".F" 
-                                          : "F";
-        if (func_draft_kind == fk_linear) {
-            fp y = ys.val(event.GetY());
-            ftk->exec(F + " += " + frame->get_peak_type()  
-                         + "(slope=~" + S(0) + ", intercept=~" + S(y) 
-                         + ", avgy=~" + S(y) + ")");
-        }
-        else {
-            if (dist_X + dist_Y >= 5) {
-                fp height = ys.val(event.GetY());
-                fp center = xs.val(mouse_press_X);
-                fp fwhm = fabs(center - xs.val(event.GetX()));
-                fp area = height * fwhm;
-                ftk->exec(F + " += " + frame->get_peak_type()  
-                         + "(height=~" + S(height) + ", center=~" + S(center) 
-                         + ", fwhm=~" + S(fwhm) + ", area=~" + S(area) + ")");
-            }
-        }
+        if (func_draft_kind == fk_linear || dist_X + dist_Y >= 5) 
+            add_peak_from_draft(event.GetX(), event.GetY());
     }
+    // add peak (in range)
     else if (mode == mmd_add && button == 3) {
         frame->set_status_text("");
         if (dist_X >= 5) { 
@@ -1069,7 +1072,8 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
             fp x2 = xs.val(event.GetX());
             ftk->exec("guess " + frame->get_peak_type() 
                           + " [" + S(min(x1,x2)) + " : " + S(max(x1,x2)) + "]"
-                          + frame->get_in_dataset());
+                          + frame->get_global_parameters()
+                          + frame->get_in_datasets());
         }
         vert_line_following_cursor(mat_stop);
     }
@@ -1080,6 +1084,35 @@ void MainPlot::OnButtonUp (wxMouseEvent &event)
     update_mouse_hints();
 }
 
+void MainPlot::add_peak_from_draft(int X, int Y)
+{
+    string args;
+    if (func_draft_kind == fk_linear) {
+        fp y = ys.val(Y);
+        args = "slope=~" + S(0) + ", intercept=~" + S(y) + ", avgy=~" + S(y);
+    }
+    else {
+        fp height = ys.val(Y);
+        fp center = xs.val(mouse_press_X);
+        fp fwhm = fabs(center - xs.val(X));
+        fp area = height * fwhm;
+        args = "height=~" + S(height) + ", center=~" + S(center) 
+                 + ", fwhm=~" + S(fwhm) + ", area=~" + S(area);
+        string global = frame->get_global_parameters();
+        if (!global.empty())
+            args += "," + global;
+    }
+    string tail = "F += " + frame->get_peak_type() + "(" + args + ")";
+    string cmd;
+    if (ftk->get_ds_count() == 1)
+        cmd = tail;
+    else {
+        vector<int> sel = frame->get_selected_ds_indices();
+        cmd = "@" + join_vector(sel, "." + tail + "; @") + "." + tail;
+    }
+    ftk->exec(cmd); 
+}
+
 
 void MainPlot::OnKeyDown (wxKeyEvent& event)
 {
@@ -1148,7 +1181,7 @@ void MainPlot::draw_xor_peak(Function const* func, vector<fp> const& p_values)
     dc.SetLogicalFunction (wxINVERT);
     dc.SetPen(*wxBLACK_DASHED_PEN);
 
-    int n = GetClientSize().GetWidth();
+    int n = get_pixel_width(dc);
     if (n <= 0) 
         return;
     vector<fp> xx(n), yy(n, 0);
@@ -1195,7 +1228,7 @@ void MainPlot::draw_peak_draft(int Ctr, int Hwhm, int Y)
     dc.SetPen(*wxBLACK_DASHED_PEN);
     int Y0 = ys.px(0);
     if (func_draft_kind == fk_linear) {
-        dc.DrawLine (0, Y, GetClientSize().GetWidth(), Y); 
+        dc.DrawLine (0, Y, get_pixel_width(dc), Y); 
     }
     else {
         dc.DrawLine (Ctr, Y0, Ctr, Y); //vertical line
@@ -1465,6 +1498,7 @@ void ConfigureAxesDlg::OnApply (wxCommandEvent&)
     plot->y_tic_size = y_tics_size->GetValue();
     plot->ys.reversed = y_reversed_cb->GetValue();
     plot->ys.logarithm = y_logarithm_cb->GetValue();
+    ftk->view.set_log_scale(plot->xs.logarithm, plot->ys.logarithm);
     frame->refresh_plots(false, !scale_changed);
 }
 
@@ -1660,8 +1694,8 @@ void auto_background (int n, fp p1, bool is_perc1, fp p2, bool is_perc2)
 void BgManager::add_background_point(fp x, fp y)
 {
     if (!bg_backup.empty()) {
-        int r = wxMessageBox(wxT("The old background has been removed\n")
-                             wxT("and now you start to create a new one.\n")
+        int r = wxMessageBox(wxT("The old background will be removed\n")
+                             wxT("and the new one will be created.\n")
                              wxT("It will make impossible to undo\n")
                              wxT("removing the old background.\n\n")
                              wxT("Use GUI->Mode->Baseline Handling->Clear...\n")
@@ -1680,7 +1714,6 @@ void BgManager::add_background_point(fp x, fp y)
     PointQ t(x, y);
     bg_iterator l = lower_bound(bg.begin(), bg.end(), t);
     bg.insert (l, t);
-    recompute_bgline();
 }
 
 void BgManager::rm_background_point (fp x)
@@ -1695,7 +1728,6 @@ void BgManager::rm_background_point (fp x)
     if (u > l) {
         bg.erase(l, u);
         ftk->vmsg (S(u - l) + " background points removed.");
-        recompute_bgline();
     }
 }
 
@@ -1703,7 +1735,6 @@ void BgManager::clear_background()
 {
     int n = bg.size();
     bg.clear();
-    recompute_bgline();
     if (n != 0)
         ftk->vmsg (S(n) + " background points deleted.");
 }
@@ -1720,7 +1751,7 @@ void BgManager::strip_background()
     bg_backup = bg;
     cmd_tail = (spline_bg ? "spline[" : "interpolate[") 
                + join_vector(pars, ", ") + "](x)" 
-               + frame->get_in_one_or_all_datasets();
+               + frame->get_in_datasets();
     clear_background();
     ftk->exec("Y = y - " + cmd_tail);
     ftk->vmsg("Background stripped.");
@@ -1732,35 +1763,35 @@ void BgManager::undo_strip_background()
         return;
     bg = bg_backup;
     bg_backup.clear();
-    recompute_bgline();
     ftk->exec("Y = y + " + cmd_tail);
 }
 
-void BgManager::recompute_bgline()
+vector<int> BgManager::calculate_bgline(int window_width, Scale const& y_scale)
 {
-    int focused_data = ftk->get_active_ds_position();
-    const std::vector<Point>& p = ftk->get_data(focused_data)->points();
-    bgline.resize(p.size());
+    vector<int> bgline(window_width);
     if (spline_bg) 
         prepare_spline_interpolation(bg);
-    for (int i = 0; i < size(p); i++) {
-        bgline[i].x = p[i].x;
-        bgline[i].y = spline_bg ? get_spline_interpolation(bg, p[i].x)
-                                : get_linear_interpolation(bg, p[i].x);
+    for (int i = 0; i < window_width; i++) {
+        fp x = x_scale.val(i);
+        fp y = spline_bg ? get_spline_interpolation(bg, x)
+                         : get_linear_interpolation(bg, x);
+        bgline[i] = y_scale.px(y);
     }
+    return bgline;
 }
 
 void BgManager::set_as_convex_hull()
 {
     SimplePolylineConvex convex;
-    int focused_data = ftk->get_active_ds_position();
-    const Data* data = ftk->get_data(focused_data);
+    vector<int> sel = frame->get_selected_ds_indices();
+    if (sel.size() != 1)
+        return;
+    const Data* data = ftk->get_data(sel[0]);
     for (int i = 0; i < data->get_n(); ++i)
         convex.push_point(PointQ(data->get_x(i), data->get_y(i)));
 
     bg = convex.get_vertices();
     bg_backup.clear();
-    recompute_bgline();
 }
 
 
