@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: mplot.h 326 2007-08-01 05:54:03Z wojdyr $
+// $Id: mplot.h 414 2008-04-06 17:37:22Z wojdyr $
 
 #ifndef FITYK__WX_MPLOT__H__
 #define FITYK__WX_MPLOT__H__
@@ -14,12 +14,14 @@
 /// it cares about visualization of spline / polyline background 
 /// which can be set by selecting points on Plot
 
-struct t_xy { fp x, y; };
 class Function;
 
 class BgManager
 {
 public:
+    typedef std::vector<PointQ>::iterator bg_iterator;
+    typedef std::vector<PointQ>::const_iterator bg_const_iterator;
+
     BgManager(Scale const& x_scale_) : x_scale(x_scale_), min_dist(8), 
                                      spline_bg(true) {}
     void add_background_point(fp x, fp y);
@@ -30,19 +32,17 @@ public:
     void undo_strip_background();
     bool can_strip() const { return !bg.empty(); }
     bool can_undo() const { return !bg_backup.empty(); }
-    void set_spline_bg(bool s) { spline_bg=s; recompute_bgline(); }
+    void set_spline_bg(bool s) { spline_bg=s; }
     void set_as_convex_hull();
+    std::vector<int> calculate_bgline(int window_width, Scale const& y_scale);
+    bg_const_iterator begin() const { return bg.begin(); }
+    bg_const_iterator end() const { return bg.end(); }
 protected:
     Scale const& x_scale;
     int min_dist; //minimal distance in X between bg points
     bool spline_bg;
-    typedef std::vector<PointQ>::iterator bg_iterator;
-    typedef std::vector<PointQ>::const_iterator bg_const_iterator;
     std::vector<PointQ> bg, bg_backup;
-    std::vector<t_xy> bgline;
     std::string cmd_tail;
-
-    void recompute_bgline();
 };
 
 
@@ -107,11 +107,13 @@ private:
 };
 
 /// main plot, single in application, displays data, fitted peaks etc. 
-class MainPlot : public FPlot, public BgManager
+class MainPlot : public FPlot
 {
     friend class ConfigureAxesDlg;
     friend class ConfigurePLabelsDlg;
 public:
+    BgManager bgm;
+
     MainPlot(wxWindow *parent); 
     ~MainPlot() {}
     void OnPaint(wxPaintEvent &event);
@@ -191,7 +193,7 @@ private:
     void draw_peaktop_selection(wxDC& dc, Sum const* sum);
     void draw_plabels (wxDC& dc, Sum const* sum, bool set_pen=true);
     void draw_dataset(wxDC& dc, int n, bool set_pen=true);
-    void prepare_peaktops(Sum const* sum);
+    void prepare_peaktops(Sum const* sum, int Ymax);
     void prepare_peak_labels(Sum const* sum);
     void look_for_peaktop (wxMouseEvent& event);
     void show_peak_menu (wxMouseEvent &event);
@@ -201,6 +203,8 @@ private:
     void draw_temporary_rect(MouseActEnum ma, int X_=0, int Y_=0);
     void draw_rect (int X1, int Y1, int X2, int Y2);
     bool visible_peaktops(MouseModeEnum mode);
+    void add_peak_from_draft(int X, int Y);
+    bool can_disactivate();
 
     DECLARE_EVENT_TABLE()
 };
