@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: pane.cpp 296 2007-05-24 20:04:00Z wojdyr $
+// $Id: pane.cpp 420 2008-04-13 16:44:13Z wojdyr $
 
 #include <wx/wxprec.h>
 #ifdef __BORLANDC__
@@ -16,7 +16,7 @@
 #include <wx/image.h>
 
 #include "pane.h" 
-#include "gui.h" 
+#include "frame.h" 
 #include "plot.h" 
 #include "mplot.h" 
 #include "aplot.h" 
@@ -136,7 +136,7 @@ bool PlotPane::is_background_white()
     return true;
 }
 
-BgManager* PlotPane::get_bg_manager() { return plot; }
+BgManager* PlotPane::get_bg_manager() { return &plot->bgm; }
 
 const std::vector<FPlot*> PlotPane::get_visible_plots() const
 {
@@ -219,8 +219,11 @@ IOPane::IOPane(wxWindow *parent, wxWindowID id)
 {
     wxBoxSizer *io_sizer = new wxBoxSizer (wxVERTICAL);
 
+    // on Linux platform GUI and CLI history is stored in the same file
+    wxString hist_file = get_user_conffile("history");
     input_field = new InputLine(this, -1, 
-              make_callback<wxString const&>().V1(this, &IOPane::OnInputLine));
+              make_callback<wxString const&>().V1(this, &IOPane::OnInputLine),
+              hist_file);
     output_win = new OutputWin (this, -1);
     io_sizer->Add (output_win, 1, wxEXPAND);
     io_sizer->Add (input_field, 0, wxEXPAND);
@@ -305,9 +308,14 @@ void OutputWin::save_settings(wxConfigBase *cf) const
 
 void OutputWin::append_text (OutputStyle style, const wxString& str)
 {
+    const int max_len = 1048576;
+    const int delta = 262144;
+    if (GetLastPosition() > max_len)
+        Remove(0, delta);
+
     SetDefaultStyle (wxTextAttr (text_color[style]));
     AppendText (str);
-    ShowPosition(GetLastPosition());
+    //ShowPosition(GetLastPosition());
 }
 
 void OutputWin::OnPopupColor (wxCommandEvent& event)
