@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: plot.cpp 317 2007-07-15 00:39:46Z wojdyr $
+// $Id: plot.cpp 417 2008-04-10 12:54:28Z wojdyr $
 
 /// In this file:
 ///  FPlot, the base class for MainPlot and AuxPlot 
@@ -19,7 +19,7 @@
 
 #include "plot.h"
 #include "cmn.h" 
-#include "gui.h" //ftk
+#include "frame.h" //ftk
 #include "../data.h"
 #include "../logic.h" 
 
@@ -29,8 +29,8 @@ void Scale::set(fp m, fp M, int pixels)
 {
     fp h = 0;
     if (logarithm) {
-        M = log(max(M, 1e-1));
-        m = log(max(m, 1e-1));
+        M = log(max(M, epsilon));
+        m = log(max(m, epsilon));
     }
     h = M - m;
     origin = reversed ? M : m;
@@ -39,55 +39,24 @@ void Scale::set(fp m, fp M, int pixels)
     scale = pixels / (reversed ? -h : h);
 }
 
-//===============================================================
-//                       BufferedPanel
-//===============================================================
-
-void BufferedPanel::refresh(bool now)
-{
-    clear_and_draw();
-    Refresh(false);
-    if (now) 
-        Update();
-}
-
-bool BufferedPanel::resize_buffer(wxDC &dc)
-{
-    wxCoord w, h;
-    dc.GetSize(&w, &h);
-    if (!buffer.Ok()
-            || w != buffer.GetWidth() || h != buffer.GetHeight()) {
-        memory_dc.SelectObject(wxNullBitmap);
-        buffer = wxBitmap(w, h);
-        memory_dc.SelectObject(buffer);
-        return true;
-    }
-    return false;
-}
-
-void BufferedPanel::clear_and_draw()
-{
-    if (!buffer.Ok())
-        return;
-    memory_dc.SetLogicalFunction(wxCOPY);
-    memory_dc.SetBackground(wxBrush(backgroundCol));
-    memory_dc.Clear();
-    draw(memory_dc);
-}
-
-/// called from wxPaint event handler
-void BufferedPanel::buffered_draw()
-{
-    wxPaintDC dc(this);
-    if (resize_buffer(dc)) 
-        clear_and_draw();
-    dc.Blit(0, 0, buffer.GetWidth(), buffer.GetHeight(), &memory_dc, 0, 0);
-}
-
 //===============================================================
 //                FPlot (plot with data and fitted curves) 
 //===============================================================
 
+BEGIN_EVENT_TABLE(FPlot, wxPanel)
+END_EVENT_TABLE()
+
+void FPlot::set_font(wxDC &dc, wxFont const& font) 
+{ 
+    if (pen_width > 1) {
+        wxFont f = font;
+        f.SetPointSize(f.GetPointSize() * pen_width);
+        dc.SetFont(f);
+    }
+    else
+        dc.SetFont(font); 
+}
+
 void FPlot::draw_dashed_vert_line(int X, int style)
 {
     if (X != INT_MIN) {
@@ -100,7 +69,7 @@ void FPlot::draw_dashed_vert_line(int X, int style)
             pen.SetStyle(style);
             dc.SetPen(pen);
         }
-        int h = GetClientSize().GetHeight();
+        int h = get_pixel_height(dc);
         dc.DrawLine (X, 0, X, h);
     }
 }
@@ -154,10 +123,10 @@ void draw_line_with_style(wxDC& dc, int style,
 void FPlot::draw_xtics (wxDC& dc, View const &v, bool set_pen)
 {
     if (set_pen) {
-        dc.SetPen(wxPen(xAxisCol));
+        dc.SetPen(wxPen(xAxisCol, pen_width));
         dc.SetTextForeground(xAxisCol);
     }
-    dc.SetFont(ticsFont);
+    set_font(dc, ticsFont);
     // get tics text height
     wxCoord h;
     dc.GetTextExtent(wxT("1234567890"), 0, &h);
@@ -168,7 +137,8 @@ void FPlot::draw_xtics (wxDC& dc, View const &v, bool set_pen)
 
     //if x axis is visible tics are drawed at the axis, 
     //otherwise tics are drawed at the bottom edge of the plot
-    int Y = GetClientSize().GetHeight() - h;
+    const int pixel_height = get_pixel_height(dc);
+    int Y = pixel_height - h;
     if (x_axis_visible && !ys.logarithm && ys.px(0) >= 0 && ys.px(0) < Y)
         Y = ys.px(0);
     for (vector<double>::const_iterator i = x_tics.begin(); 
@@ -182,9 +152,8 @@ void FPlot::draw_xtics (wxDC& dc, View const &v, bool set_pen)
         dc.GetTextExtent (label, &w, 0);
         dc.DrawText (label, X - w/2, Y + 1);
         if (x_grid) {
-            draw_line_with_style(dc, wxDOT, X,0, X,Y);
-            draw_line_with_style(dc, wxDOT, X, Y+1+h, 
-                                            X, GetClientSize().GetHeight());
+            draw_line_with_style(dc, wxDOT, X, 0, X, Y);
+            draw_line_with_style(dc, wxDOT, X, Y+1+h, X, pixel_height);
         }
     }
     //draw minor tics
@@ -200,17 +169,18 @@ void FPlot::draw_xtics (wxDC& dc, View const &v, bool set_pen)
 void FPlot::draw_ytics (wxDC& dc, View const &v, bool set_pen)
 {
     if (set_pen) {
-        dc.SetPen(wxPen(xAxisCol));
+        dc.SetPen(wxPen(xAxisCol, pen_width));
         dc.SetTextForeground(xAxisCol);
     }
-    dc.SetFont(ticsFont);
+    set_font(dc, ticsFont);
+    const int pixel_width = get_pixel_width(dc);
 
 
     //if y axis is visible, tics are drawed at the axis, 
     //otherwise tics are drawed at the left hand edge of the plot
     int X = 0;
     if (y_axis_visible && xs.px(0) > 0 
-            && xs.px(0) < GetClientSize().GetWidth()-10)
+            && xs.px(0) < pixel_width - 10)
         X = xs.px(0);
     vector<double> minors;
     vector<double> y_tics = scale_tics_step(v.bottom, v.top, y_max_tics, 
@@ -230,7 +200,7 @@ void FPlot::draw_ytics (wxDC& dc, View const &v, bool set_pen)
         if (y_grid) {
             draw_line_with_style(dc, wxDOT, 0,Y, X,Y);
             draw_line_with_style(dc, wxDOT, X + y_tic_size + 1 + w + 1, Y,
-                                            GetClientSize().GetWidth(), Y);
+                                            pixel_width, Y);
         }
     }
     //draw minor tics
@@ -267,10 +237,11 @@ void FPlot::draw_data (wxDC& dc,
                        int Y_offset,
                        bool cumulative)
 {
-    Y_offset *= (GetClientSize().GetHeight() / 100);
-    wxPen const activePen(color.Ok() ? color : activeDataCol);
+    Y_offset *= (get_pixel_height(dc) / 100);
+    wxPen const activePen(color.Ok() ? color : activeDataCol, pen_width);
     wxPen const inactivePen(inactive_color.Ok() ? inactive_color 
-                                                : inactiveDataCol);
+                                                : inactiveDataCol, 
+                            pen_width);
     wxBrush const activeBrush(activePen.GetColour(), wxSOLID);
     wxBrush const inactiveBrush(inactivePen.GetColour(), wxSOLID);
     if (data->is_empty()) 
@@ -282,7 +253,7 @@ void FPlot::draw_data (wxDC& dc,
     dc.SetPen (active ? activePen : inactivePen);
     dc.SetBrush (active ? activeBrush : inactiveBrush);
     int X_ = INT_MIN, Y_ = INT_MIN;
-    // first line segment -- lines should be drawed towards points 
+    // first line segment -- lines should be drawn towards points 
     //                                                 that are outside of plot 
     if (line_between_points && first > data->points().begin() && !cumulative) {
         X_ = xs.px (ftk->view.left);
@@ -314,9 +285,11 @@ void FPlot::draw_data (wxDC& dc,
             //draw first half here and change X_, Y_; the rest will be drawed
             //as usually.
             if (line_between_points) {
-                int X_mid = (X_ + X) / 2, Y_mid = (Y_ + Y) / 2;
+                int X_mid = (X_ + X) / 2; 
+		int Y_mid = (Y_ + Y) / 2;
                 dc.DrawLine (X_, Y_, X_mid, Y_mid);
-                X_ = X_mid, Y_ = Y_mid;
+                X_ = X_mid; 
+		Y_ = Y_mid;
             }
             active = i->is_active;
             if (active) {
@@ -330,11 +303,12 @@ void FPlot::draw_data (wxDC& dc,
         }
 
         if (point_radius > 1) 
-            dc.DrawCircle (X, Y, point_radius - 1);
+            dc.DrawCircle (X, Y, (point_radius - 1) * pen_width);
         if (line_between_points) {
             if (X_ != INT_MIN)
                 dc.DrawLine (X_, Y_, X, Y);
-            X_ = X, Y_ = Y;
+            X_ = X; 
+	    Y_ = Y;
         }
         else {//no line_between_points
             if (point_radius == 1)
@@ -377,11 +351,13 @@ void FPlot::change_tics_font()
     }
 }
 
-void FPlot::set_scale()
+void FPlot::set_scale(int pixel_width, int pixel_height)
 {
     View const &v = ftk->view;
-    xs.set(v.left, v.right, GetClientSize().GetWidth());
-    ys.set(v.top, v.bottom, GetClientSize().GetHeight());
+    if (pixel_width > 0)
+	xs.set(v.left, v.right, pixel_width);
+    if (pixel_height > 0)
+	ys.set(v.top, v.bottom, pixel_height);
 }
 
 int FPlot::get_special_point_at_pointer(wxMouseEvent& event)
@@ -438,83 +414,3 @@ void FPlot::save_settings(wxConfigBase *cf) const
 }
 
 
-BEGIN_EVENT_TABLE(FPlot, wxPanel)
-END_EVENT_TABLE()
-
-
-//===============================================================
-//                             utilities
-//===============================================================
-
-/// returns major and minor tics positions
-vector<double> scale_tics_step (double beg, double end, int max_tics, 
-                                vector<double> &minors, bool log)
-{
-    vector<double> result;
-    minors.clear();
-    if (beg >= end || max_tics <= 0)
-        return result;
-    if (log) {
-        if (beg <= 0)
-            beg = 1e-1;
-        if (end <= beg)
-            end = 2*beg;
-        double min_logstep = (log10(end/beg)) / max_tics;
-        bool with_2_5 = (min_logstep < log10(2.));
-        double logstep = ceil(min_logstep);
-        double step0 = pow(10, logstep * ceil(log10(beg) / logstep));
-        for (int i = 2; i <= 9; ++i) {
-            double v = step0/10. * i;
-            if (v > beg && v < end) {
-                if (with_2_5 && (i == 2 || i == 5))
-                    result.push_back(v);
-                else
-                    minors.push_back(v);
-            }
-        }
-        for (double t = step0; t < end; t *= pow(10,logstep)) {
-            result.push_back(t);
-            for (int i = 2; i <= 9; ++i) {
-                double v = t * i;
-                if (v > beg && v < end)
-                    if (with_2_5 && (i == 2 || i == 5))
-                        result.push_back(v);
-                    else
-                        minors.push_back(v);
-            }
-        }
-    }
-    else { // !log
-        double min_step = (end - beg) / max_tics;
-        double s = pow(10, floor (log10 (min_step)));
-        int minor_div = 5; //ratio of major-step to minor-step
-        // now s <= min_step
-        if (s >= min_step)
-            ;
-        else if (s * 2 >= min_step) {
-            s *= 2;
-            minor_div = 2;
-        }
-        else if (s * 2.5 >= min_step) 
-            s *= 2.5;
-        else if (s * 5 >=  min_step) 
-            s *= 5;
-        else
-            s *= 10;
-        for (double t = s * ceil(beg / s); t < end; t += s) {
-            if (t > beg) {
-                // make sure that 0 is not displayed as e.g. -2.7893e-17 
-                if (is_zero(t))
-                    t = 0.;
-                result.push_back(t);
-            }
-            for (int i = 1; i < minor_div; ++i) {
-                double v = t + s * i / minor_div;
-                if (v > beg && v < end)
-                    minors.push_back(v);
-            }
-        }
-    }
-    return result;
-}
-
