@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: plot.h 288 2007-04-22 02:14:02Z wojdyr $
+// $Id: plot.h 417 2008-04-10 12:54:28Z wojdyr $
 
 #ifndef FITYK__WX_PLOT__H__
 #define FITYK__WX_PLOT__H__
@@ -9,6 +9,10 @@
 #include <vector>
 #include <wx/config.h>
 #include "../data.h" //Point
+#include "uplot.h" //BufferedPanel
+
+// convention: lowercase coordinates of point are real values,
+// and uppercase ones are coordinates of point on screen (integers).
 
 // INT_MIN, given as coordinate, is invalid value, means "cancel drawing"
 
@@ -21,31 +25,6 @@ enum MouseActEnum  { mat_start, mat_stop, mat_move, mat_redraw };
 void draw_line_with_style(wxDC& dc, int style, 
                           wxCoord X1, wxCoord Y1, wxCoord X2, wxCoord Y2);
 
-// convention here: lowercase coordinates of point are real values,
-// and uppercase ones are coordinates of point on screen (integers).
-
-
-class BufferedPanel : public wxPanel
-{
-public:
-    BufferedPanel(wxWindow *parent)
-       : wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, 
-                 wxNO_BORDER|wxFULL_REPAINT_ON_RESIZE) {}
-    void refresh(bool now=false);
-    void buffered_draw();
-    virtual void draw(wxDC &dc, bool monochrome=false) = 0;
-
-protected:
-    wxColour backgroundCol;
-
-private:
-    wxMemoryDC memory_dc;
-    wxBitmap buffer;
-
-    bool resize_buffer(wxDC &dc);
-    void clear_and_draw();
-};
-
 
 /// convertion between pixels and logical values
 class Scale
@@ -57,23 +36,9 @@ public:
     Scale() : scale(1.), origin(0.), logarithm(false), reversed(false) {}
 
     /// value -> pixel
-    int px(double val) const 
-    {
-        if (logarithm) {
-            if (val <= 0)
-                return inf_px(scale);
-            val = log(val);
-        }
-        double t = (val - origin) * scale;
-        return fabs(t) < SHRT_MAX ? static_cast<int>(t) : inf_px(t);
-    }
-
+    inline int px(double val) const;
     /// pixel -> value
-    double val(int px) const
-    { 
-        fp a = px / scale + origin; 
-        return logarithm ? exp(a) : a; 
-    }
+    inline double val(int px) const;
 
     // set scale using minimum and maximum logical values and width/height 
     // of the screen in pixels.
@@ -85,21 +50,46 @@ private:
 };
 
 
+int Scale::px(double val) const
+{
+    if (logarithm) {
+        if (val <= 0)
+            return inf_px(-scale);
+        val = log(val);
+    }
+    double t = (val - origin) * scale;
+    return fabs(t) < SHRT_MAX ? static_cast<int>(t) : inf_px(t);
+}
+
+double Scale::val(int px) const
+{ 
+    fp a = px / scale + origin; 
+    return logarithm ? exp(a) : a; 
+}
+
 
 /// This class has no instances, MainPlot and AuxPlot are derived from it
+/// It knows how to draw on wxDC. Note that wxDC:SetClippingRegion() should be
+/// used together with wxDC::SetDeviceOrigin(). Clipping box is used only in
+/// get_pixel_width() and get_pixel_height() functions.
+/// When plotting a curve, values in each x from 0 to get_pixel_width() is 
+/// calculated.
+
 class FPlot : public BufferedPanel 
 {
 public:
     FPlot (wxWindow *parent)
        : BufferedPanel(parent),
+         pen_width(1),
          draw_sigma(false), 
          mouse_press_X(INT_MIN), mouse_press_Y(INT_MIN),
          vlfc_prev_x(INT_MIN), vlfc_prev_x0(INT_MIN)   {}
          
     ~FPlot() {}
+    void set_font(wxDC &dc, wxFont const& font);
     wxColour const& get_bg_color() const { return backgroundCol; }
     void draw_crosshair(int X, int Y);
-    void set_scale();
+    void set_scale(int pixel_width, int pixel_height);
     int get_special_point_at_pointer(wxMouseEvent& event);
     std::vector<wxPoint> const& get_special_points() const
                                                 { return special_points; }
@@ -107,9 +97,11 @@ public:
     Scale const& get_y_scale() const { return ys; }
     virtual void save_settings(wxConfigBase *cf) const;
     virtual void read_settings(wxConfigBase *cf);
+    void set_magnification(int m) { pen_width = m > 0 ? m : 1; }
 
 protected:
     Scale xs, ys;
+    int pen_width;
     wxColour activeDataCol, inactiveDataCol, xAxisCol;
     wxFont ticsFont;
     int point_radius;
@@ -143,6 +135,21 @@ protected:
                     bool cumulative=false);
     void change_tics_font();
 
+    int get_pixel_width(wxDC const& dc) const 
+      //{ return dc.GetSize().GetWidth(); }
+    { 
+          int w;
+          dc.GetClippingBox(NULL, NULL, &w, NULL);
+          return w != 0 ? w : dc.GetSize().GetWidth(); 
+    }
+    int get_pixel_height(wxDC const& dc) const 
+    //  { return dc.GetSize().GetHeight(); }
+    { 
+          int h;
+          dc.GetClippingBox(NULL, NULL, NULL, &h);
+          return h != 0 ? h : dc.GetSize().GetHeight(); 
+    }
+
     DECLARE_EVENT_TABLE()
 };
 
@@ -151,8 +158,5 @@ protected:
 inline wxColour invert_colour(const wxColour& col)
 { return wxColour(255 - col.Red(), 255 - col.Green(), 255 - col.Blue()); }
 
-std::vector<double> scale_tics_step (double beg, double end, int max_tics, 
-                                 std::vector<double> &minors, bool log=false);
-
 #endif 
 
