@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: sdebug.cpp 296 2007-05-24 20:04:00Z wojdyr $
+// $Id: sdebug.cpp 354 2007-11-19 12:38:52Z wojdyr $
 
 /// In this file:
 ///  Script Editor and Debugger (ScriptDebugDlg)
@@ -15,7 +15,7 @@
 #include <fstream>
 
 #include "sdebug.h"
-#include "gui.h" //ftk
+#include "frame.h" //ftk
 #include "../cmd.h" //check_command_syntax()
 #include "../logic.h" 
 
