@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: setdlg.cpp 296 2007-05-24 20:04:00Z wojdyr $
+// $Id: setdlg.cpp 354 2007-11-19 12:38:52Z wojdyr $
 
 /// In this file:
 ///  Settings Dialog (SettingsDlg)
@@ -20,7 +20,7 @@
 #include "../settings.h" 
 #include "../ui.h"  //startup_commands_filename
 #include "../logic.h"  
-#include "gui.h"  //ftk
+#include "frame.h"  //ftk
 
 using namespace std;
 
@@ -244,8 +244,7 @@ SettingsDlg::SettingsDlg(wxWindow* parent, const wxWindowID id)
     add_persistence_note(page_fit_NM, sizer_fnm);
     page_fit_NM->SetSizerAndFit(sizer_fnm);
     
-    // sub-page GA
-    // TODO
+    // TODO: sub-page GA
 
     // page directories
     wxBoxSizer *sizer_dirs = new wxBoxSizer(wxVERTICAL);
@@ -297,7 +296,7 @@ void SettingsDlg::add_persistence_note(wxWindow *parent, wxSizer *sizer)
                                            parent, wxT("persistance note"));
     persistence->Add(new wxStaticText(parent, -1,
                        wxT("To have values above remained after restart, ")
-                       wxT("put proper\ncommands into init file:")
+                       wxT("put proper\ncommands into init file: ")
                        + get_user_conffile(startup_commands_filename)),
                      0, wxALL|wxALIGN_CENTER, 5);
     sizer->AddStretchSpacer();
