@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: sidebar.cpp 345 2007-08-21 01:09:15Z wojdyr $
+// $Id: sidebar.cpp 424 2008-04-15 11:08:42Z wojdyr $
 
 /// In this file:
 ///  SideBar class, the right hand sidebar in GUI 
@@ -13,13 +13,14 @@
 #include <wx/wx.h>
 #endif
 #include <wx/imaglist.h>
+#include <wx/tglbtn.h>
 
 #include "sidebar.h"
 #include "fancyrc.h"
 #include "listptxt.h"
 #include "gradient.h"
 #include "cmn.h" //SpinCtrl, ProportionalSplitter, change_color_dlg, ...
-#include "gui.h" //frame
+#include "frame.h" //frame
 #include "mplot.h" 
 #include "../common.h" //vector4, join_vector, S(), ...
 #include "../ui.h" 
@@ -37,7 +38,7 @@
 #include "img/filter.xpm"
 #include "img/shiftup.xpm"
 #include "img/dpsize.xpm"
-#include "img/convert.xpm"
+//#include "img/convert.xpm"
 #include "img/copyfunc.xpm"
 #include "img/unused.xpm"
 #include "img/zshift.xpm"
@@ -65,6 +66,8 @@ enum {
     ID_FP_EDIT                 ,
     ID_FP_CHTYPE               ,
     ID_FP_COL                  ,
+    ID_FP_HWHM                 ,
+    ID_FP_SHAPE                ,
     ID_VP_LIST                 ,
     ID_VP_NEW                  ,
     ID_VP_DEL                  ,
@@ -75,7 +78,7 @@ enum {
 //                           SideBar
 //===============================================================
 
-void add_bitmap_button(wxWindow* parent, wxWindowID id, char** xpm,
+void add_bitmap_button(wxWindow* parent, wxWindowID id, const char** xpm,
                        wxString const& tip, wxSizer* sizer)
 {
     wxBitmapButton *btn = new wxBitmapButton(parent, id, wxBitmap(xpm));
@@ -83,6 +86,19 @@ void add_bitmap_button(wxWindow* parent, wxWindowID id, char** xpm,
     sizer->Add(btn);
 }
 
+// wxToggleBitmapButton was added in 2.9. We use wxToggleButton instead
+void add_toggle_bitmap_button(wxWindow* parent, wxWindowID id, 
+                              wxString const& label,
+                              wxString const& tip, wxSizer* sizer)
+{
+    wxToggleButton *btn = new wxToggleButton(parent, id, label, 
+                                             wxDefaultPosition, wxDefaultSize,
+                                             wxBU_EXACTFIT);
+    btn->SetToolTip(tip);
+    sizer->Add(btn, wxSizerFlags().Expand());
+}
+
+
 BEGIN_EVENT_TABLE(SideBar, ProportionalSplitter)
     EVT_BUTTON (ID_DP_NEW, SideBar::OnDataButtonNew)
     EVT_BUTTON (ID_DP_DUP, SideBar::OnDataButtonDup)
@@ -95,8 +111,8 @@ BEGIN_EVENT_TABLE(SideBar, ProportionalSplitter)
     EVT_CHECKBOX (ID_DP_PLINE, SideBar::OnDataPLineChanged)
     EVT_CHECKBOX (ID_DP_PS,    SideBar::OnDataPSChanged)
     EVT_SPINCTRL (ID_DP_SHIFTUP, SideBar::OnDataShiftUpChanged)
-    EVT_LIST_ITEM_SELECTED(ID_DP_LIST, SideBar::OnDataFocusChanged)
-    EVT_LIST_ITEM_DESELECTED(ID_DP_LIST, SideBar::OnDataFocusChanged)
+    EVT_LIST_ITEM_SELECTED(ID_DP_LIST, SideBar::OnDataSelectionChanged)
+    EVT_LIST_ITEM_DESELECTED(ID_DP_LIST, SideBar::OnDataSelectionChanged)
     EVT_LIST_ITEM_FOCUSED(ID_DP_LIST, SideBar::OnDataFocusChanged)
     EVT_CHOICE (ID_FP_FILTER, SideBar::OnFuncFilterChanged)
     EVT_BUTTON (ID_FP_NEW, SideBar::OnFuncButtonNew)
@@ -104,11 +120,17 @@ BEGIN_EVENT_TABLE(SideBar, ProportionalSplitter)
     EVT_BUTTON (ID_FP_EDIT, SideBar::OnFuncButtonEdit)
     EVT_BUTTON (ID_FP_CHTYPE, SideBar::OnFuncButtonChType)
     EVT_BUTTON (ID_FP_COL, SideBar::OnFuncButtonCol)
+    EVT_TOGGLEBUTTON (ID_FP_HWHM, SideBar::OnFuncButtonHwhm)
+    EVT_TOGGLEBUTTON (ID_FP_SHAPE, SideBar::OnFuncButtonShape)
     EVT_LIST_ITEM_FOCUSED(ID_FP_LIST, SideBar::OnFuncFocusChanged)
+    EVT_LIST_ITEM_SELECTED(ID_FP_LIST, SideBar::OnFuncSelectionChanged)
+    EVT_LIST_ITEM_DESELECTED(ID_FP_LIST, SideBar::OnFuncSelectionChanged)
     EVT_BUTTON (ID_VP_NEW, SideBar::OnVarButtonNew)
     EVT_BUTTON (ID_VP_DEL, SideBar::OnVarButtonDel)
     EVT_BUTTON (ID_VP_EDIT, SideBar::OnVarButtonEdit)
     EVT_LIST_ITEM_FOCUSED(ID_VP_LIST, SideBar::OnVarFocusChanged)
+    EVT_LIST_ITEM_SELECTED(ID_VP_LIST, SideBar::OnVarSelectionChanged)
+    EVT_LIST_ITEM_DESELECTED(ID_VP_LIST, SideBar::OnVarSelectionChanged)
 END_EVENT_TABLE()
 
 SideBar::SideBar(wxWindow *parent, wxWindowID id)
@@ -173,8 +195,8 @@ SideBar::SideBar(wxWindow *parent, wxWindowID id)
     wxBoxSizer *data_buttons_sizer = new wxBoxSizer(wxHORIZONTAL);
     add_bitmap_button(data_page, ID_DP_COL, colorsel_xpm, 
                       wxT("change color"), data_buttons_sizer);
-    add_bitmap_button(data_page, ID_DP_NEW, add_xpm, 
-                      wxT("new data"), data_buttons_sizer);
+    //add_bitmap_button(data_page, ID_DP_NEW, add_xpm, 
+    //                  wxT("new data"), data_buttons_sizer);
     add_bitmap_button(data_page, ID_DP_DUP, sum_xpm, 
                       wxT("duplicate/sum"), data_buttons_sizer);
     add_bitmap_button(data_page, ID_DP_CPF, copyfunc_xpm, 
@@ -213,16 +235,22 @@ SideBar::SideBar(wxWindow *parent, wxWindowID id)
     f->list->set_side_bar(this);
     func_sizer->Add(f, 1, wxEXPAND|wxALL, 1);
     wxBoxSizer *func_buttons_sizer = new wxBoxSizer(wxHORIZONTAL);
+    add_toggle_bitmap_button(func_page, ID_FP_HWHM, wxT("=W"), 
+                             wxT("same HWHM for all functions"), 
+                             func_buttons_sizer);
+    add_toggle_bitmap_button(func_page, ID_FP_SHAPE, wxT("=S"), 
+                             wxT("same shape for all functions"), 
+                             func_buttons_sizer);
     add_bitmap_button(func_page, ID_FP_NEW, add_xpm, 
                       wxT("new function"), func_buttons_sizer);
-    add_bitmap_button(func_page, ID_FP_DEL, close_xpm, 
-                      wxT("delete"), func_buttons_sizer);
     add_bitmap_button(func_page, ID_FP_EDIT, editf_xpm, 
                       wxT("edit function"), func_buttons_sizer);
-    add_bitmap_button(func_page, ID_FP_CHTYPE, convert_xpm, 
-                      wxT("change type of function"), func_buttons_sizer);
+    //add_bitmap_button(func_page, ID_FP_CHTYPE, convert_xpm, 
+    //                  wxT("change type of function"), func_buttons_sizer);
     add_bitmap_button(func_page, ID_FP_COL, colorsel_xpm, 
                       wxT("change color"), func_buttons_sizer);
+    add_bitmap_button(func_page, ID_FP_DEL, close_xpm, 
+                      wxT("delete"), func_buttons_sizer);
     func_sizer->Add(func_buttons_sizer, 0, wxEXPAND);
     func_page->SetSizerAndFit(func_sizer);
     nb->AddPage(func_page, wxT("functions"));
@@ -264,7 +292,7 @@ SideBar::SideBar(wxWindow *parent, wxWindowID id)
 
 void SideBar::OnDataButtonNew (wxCommandEvent&)
 {
-    ftk->exec("@+");
+    ftk->exec("@+ = 0");
     frame->refresh_plots(false, true);
 }
 
@@ -285,7 +313,7 @@ void SideBar::OnDataButtonRen (wxCommandEvent&)
                          wxT("Rename dataset"),
                          old_title);
     if (!s.IsEmpty() && s != old_title)
-        ftk->exec("@" + S(n) + ".title = '" + wx2s(s) + "'");
+        ftk->exec("set @" + S(n) + ".title = '" + wx2s(s) + "'");
 }
 
 void SideBar::delete_selected_items()
@@ -314,11 +342,11 @@ void SideBar::OnDataButtonCopyF (wxCommandEvent&)
     d->list->Select(n, false);
     d->list->Select(n+1, true);
     d->list->Focus(n+1);
-    DataFocusChanged();
     string cmd = "@" + S(n+1) + ".F=copy(@" + S(n) + ".F)";
     if (!ftk->get_sum(n)->get_zz_names().empty() 
             || !ftk->get_sum(n+1)->get_zz_names().empty())
         cmd += "; @" + S(n+1) + ".Z=copy(@" + S(n) + ".Z)";
+    update_data_buttons();
     ftk->exec(cmd);
 }
 
@@ -337,7 +365,6 @@ void SideBar::OnDataButtonCol (wxCommandEvent&)
         }
     }
     else {//sel_size > 1
-        vector<string> sel = d->get_selected_data();
         int first_sel = d->list->GetFirstSelected();
         int last_sel = 0;
         for (int i = first_sel; i != -1; i = d->list->GetNextSelected(i))
@@ -432,7 +459,7 @@ void SideBar::OnFuncButtonChType (wxCommandEvent&)
 void SideBar::OnFuncButtonCol (wxCommandEvent&)
 {
     vector<int> const& ffi 
-        = ftk->get_sum(ftk->get_active_ds_position())->get_ff_idx();
+        = ftk->get_sum(frame->get_focused_ds_index())->get_ff_idx();
     vector<int>::const_iterator in_ff = find(ffi.begin(), ffi.end(), 
                                                          active_function);
     if (in_ff == ffi.end())
@@ -482,21 +509,14 @@ void SideBar::save_settings(wxConfigBase *cf) const
 void SideBar::update_lists(bool nondata_changed)
 {
     Freeze();
-    d->update_data_list(nondata_changed, true);
+    d->update_data_list(nondata_changed);
     update_func_list(nondata_changed);
     update_var_list();
     
     //-- enable/disable buttons
-    bool not_the_last = get_focused_data()+1 < ftk->get_ds_count();
-    data_page->FindWindow(ID_DP_CPF)->Enable(not_the_last);
-    bool has_any_funcs = (f->list->GetItemCount() > 0);
-    func_page->FindWindow(ID_FP_DEL)->Enable(has_any_funcs);
-    func_page->FindWindow(ID_FP_EDIT)->Enable(has_any_funcs);
-    func_page->FindWindow(ID_FP_CHTYPE)->Enable(has_any_funcs);
-    func_page->FindWindow(ID_FP_COL)->Enable(has_any_funcs);
-    bool has_any_vars = (v->list->GetItemCount() > 0);
-    var_page->FindWindow(ID_VP_DEL)->Enable(has_any_vars);
-    var_page->FindWindow(ID_VP_EDIT)->Enable(has_any_vars);
+    update_data_buttons();
+    update_func_buttons();
+    update_var_buttons();
 
     int n = get_focused_data();
     dpline_cb->SetValue(frame->get_main_plot()->get_data_with_line(n));
@@ -526,8 +546,7 @@ void SideBar::update_func_list(bool nondata_changed)
     static int old_func_size;
     vector<string> func_data;
     vector<int> new_func_col_id;
-    int active_ds_pos = ftk->get_active_ds_position();
-    Sum const* sum = ftk->get_sum(active_ds_pos);
+    Sum const* sum = ftk->get_sum(frame->get_focused_ds_index());
     Sum const* filter_sum = 0;
     if (filter_ch->GetSelection() > 0)
         filter_sum = ftk->get_sum(filter_ch->GetSelection()-1);
@@ -620,8 +639,18 @@ void SideBar::update_var_list()
 
 int SideBar::get_focused_data() const
 { 
-    int n = d->list->GetFocusedItem(); 
-    return n == -1 ? 0 : n; 
+    wxListView *lv = d->list;
+    int focused = lv->GetFocusedItem();
+    if (focused >= ftk->get_ds_count()) {
+        d->update_data_list(false);
+        focused = ftk->get_ds_count() - 1;
+        lv->Focus(focused);
+    }
+    else if (focused < 0) {
+        focused = 0;
+        lv->Focus(focused);
+    }
+    return focused;
 }
 
 int SideBar::get_focused_var() const 
@@ -634,6 +663,71 @@ int SideBar::get_focused_var() const
     }
 }
 
+vector<int> SideBar::get_selected_ds_indices()
+{
+    vector<int> sel;
+    if (!frame) // app not fully initialized yet
+        return sel;
+    wxListView *lv = d->list;
+    if (ftk->get_ds_count() != lv->GetItemCount())
+        d->update_data_list(false);
+    for (int i = lv->GetFirstSelected(); i != -1; i = lv->GetNextSelected(i))
+        sel.push_back(i);
+    if (sel.empty()) {
+        int n = get_focused_data();
+        lv->Select(n, true);
+        sel.push_back(n);
+    }
+    return sel;
+}
+
+// in plotting order 
+vector<int> SideBar::get_ordered_dataset_numbers()
+{
+    wxListView *lv = d->list;
+    if (ftk->get_ds_count() != lv->GetItemCount())
+        d->update_data_list(false);
+    vector<int> ordered;
+    int count = lv->GetItemCount();
+    if (count == 0)
+        return ordered;
+    vector<int> selected;
+    int focused = lv->GetFocusedItem();
+    // focused but not selected is not important
+    if (focused >= 0 && !lv->IsSelected(focused)) 
+        focused = -1;
+    for (int i = 0; i < count; ++i) {
+        if (lv->IsSelected(i)) {
+            if (i != focused) 
+                selected.push_back(i);
+        }
+        else
+            ordered.push_back(i);
+    }
+    ordered.insert(ordered.end(), selected.begin(), selected.end());
+    if (focused >= 0)
+        ordered.push_back(focused);
+    assert (size(ordered) == count);
+    return ordered;
+}
+
+string SideBar::get_plot_in_datasets()
+{
+    if (ftk->get_ds_count() == 1)
+        return "";
+    int focused = get_focused_data();
+    string s = " in @" + S(focused);
+    if (data_look->GetSelection() == 0) // all datasets
+        s += ", @*";
+    else {
+        for (int i = d->list->GetFirstSelected(); i != -1; 
+                                             i = d->list->GetNextSelected(i))
+            if (i != focused)
+                s += ", @" + S(i);
+    }
+    return s;
+}
+
 //bool SideBar::is_func_selected(int n) const 
 //{ 
 //    return f->list->IsSelected(n) || f->list->GetFocusedItem() == n; 
@@ -663,29 +757,84 @@ void SideBar::do_activate_function()
     update_bottom_panel();
 }
 
-void SideBar::DataFocusChanged()
-{
+// Focus is _not_ used for data-related operations, only for function-related
+// ones. Sum and functions are shown only for focused dataset.
+void SideBar::OnDataFocusChanged(wxListEvent &) 
+{ 
     int n = d->list->GetFocusedItem();
+    //ftk->msg("[F] id:" + S(event.GetIndex()) + " focused:" 
+    //         + S(d->list->GetFocusedItem()));
     if (n < 0)
         return;
     int length = ftk->get_ds_count();
-    if (length > 1 && ftk->get_active_ds_position() != n) 
-        ftk->exec("plot .. in @" + S(n));
-    data_page->FindWindow(ID_DP_CPF)->Enable(n+1<length);
+    if (length > 1)
+        frame->refresh_plots();
+    update_data_inf();
 }
 
-string SideBar::get_datasets_str()
+void SideBar::OnDataSelectionChanged(wxListEvent &event) 
+{ 
+    if (data_look->GetSelection() != 0) // ! all datasets
+        frame->refresh_plots();
+    bool selected = (event.GetEventType() == wxEVT_COMMAND_LIST_ITEM_SELECTED);
+    assert (selected 
+            || event.GetEventType() == wxEVT_COMMAND_LIST_ITEM_DESELECTED);
+    //long index = event.GetIndex();
+    //ftk->msg("id:" + S(index) + " e.sel:" + S(selected)
+    //         + " issel:" + S(d->list->IsSelected(index))
+    //         + " foc:" + S(d->list->GetFocusedItem()));
+    update_data_buttons();
+}
+
+void SideBar::update_data_buttons()
 {
+    bool not_the_last = get_focused_data()+1 < ftk->get_ds_count();
+    int sel_d = d->list->GetSelectedItemCount();
+    data_page->FindWindow(ID_DP_REN)->Enable(sel_d == 1);
+    //data_page->FindWindow(ID_DP_DEL)->Enable(sel_d > 0);
+    //data_page->FindWindow(ID_DP_COL)->Enable(sel_d > 0);
+    data_page->FindWindow(ID_DP_CPF)->Enable(sel_d == 1 && not_the_last);
+}
+
+void SideBar::update_func_buttons()
+{
+    int sel_f = f->list->GetSelectedItemCount();
+    func_page->FindWindow(ID_FP_DEL)->Enable(sel_f > 0);
+    func_page->FindWindow(ID_FP_EDIT)->Enable(sel_f == 1);
+    //func_page->FindWindow(ID_FP_CHTYPE)->Enable(sel_f > 0);
+    func_page->FindWindow(ID_FP_COL)->Enable(sel_f > 0);
+
+    bool has_hwhm = false, has_shape = false;
+    for (vector<Function*>::const_iterator i = ftk->get_functions().begin();
+                                         i != ftk->get_functions().end(); ++i) {
+        vector<string> const& t = (*i)->type_var_names;
+        if (find(t.begin(), t.end(), "hwhm") != t.end())
+            has_hwhm = true;
+        if (find(t.begin(), t.end(), "shape") != t.end())
+            has_shape = true;
+    }
+    func_page->FindWindow(ID_FP_HWHM)->Enable(has_hwhm);
+    func_page->FindWindow(ID_FP_SHAPE)->Enable(has_shape);
+}
+
+void SideBar::update_var_buttons()
+{
+    int sel_v = v->list->GetSelectedItemCount();
+    var_page->FindWindow(ID_VP_DEL)->Enable(sel_v > 0);
+    var_page->FindWindow(ID_VP_EDIT)->Enable(sel_v == 1);
+}
+
+string SideBar::get_datasets_for_plot()
+{
+    int first = frame->get_focused_ds_index();
+    string s = "@" + S(first);
     if (data_look->GetSelection() == 0) // all datasets
-        return "@*";
+        return s + ", @*";
     else { // only selected datasets
-        int first = d->list->GetFirstSelected();
-        if (first == -1)
-            return frame->get_active_data_str();
-        string s = "@" + S(first);
-        for (int i = d->list->GetNextSelected(first); i != -1; 
+        for (int i = d->list->GetFirstSelected(); i != -1; 
                                               i = d->list->GetNextSelected(i))
-            s += ", @" + S(i);
+            if (i != first)
+                s += ", @" + S(i);
         return s;
     }
 }
@@ -708,8 +857,10 @@ void SideBar::update_data_inf()
     inf->SetDefaultStyle(defattr);
     inf->AppendText(s2wx(ftk->get_data(n)->get_info()));
     wxFileName fn(s2wx(ftk->get_data(n)->get_filename()));
-    if (fn.IsOk() && !fn.IsAbsolute())
-        inf->AppendText(wxT("\nPath: ") + fn.MakeAbsolute());
+    if (fn.IsOk() && !fn.IsAbsolute()) {
+        fn.MakeAbsolute();
+        inf->AppendText(wxT("\nPath: ") + fn.GetFullPath());
+    }
     inf->ShowPosition(0);
 }
 
@@ -914,8 +1065,6 @@ void SideBar::update_bottom_panel()
 
 bool SideBar::howto_plot_dataset(int n, bool& shadowed, int& offset) const
 {
-    if (ftk->get_ds_count() != d->list->GetItemCount())
-        d->update_data_list(true, true);
     // choice_idx: 0: "show all datasets" 
     //             1: "show only selected" 
     //             2: "shadow unselected" 
@@ -935,10 +1084,10 @@ vector<string> SideBar::get_selected_func() const
     for (int i = f->list->GetFirstSelected(); i != -1; 
                                             i = f->list->GetNextSelected(i))
         dd.push_back(ftk->get_function(i)->xname);
-    if (dd.empty() && f->list->GetItemCount() > 0) {
-        int n = f->list->GetFocusedItem();
-        dd.push_back(ftk->get_function(n == -1 ? 0 : n)->xname);
-    }
+    //if (dd.empty() && f->list->GetItemCount() > 0) {
+    //    int n = f->list->GetFocusedItem();
+    //    dd.push_back(ftk->get_function(n == -1 ? 0 : n)->xname);
+    //}
     return dd;
 }
 
@@ -948,10 +1097,10 @@ vector<string> SideBar::get_selected_vars() const
     for (int i = v->list->GetFirstSelected(); i != -1; 
                                              i = v->list->GetNextSelected(i))
         dd.push_back(ftk->get_variable(i)->xname);
-    if (dd.empty() && v->list->GetItemCount() > 0) {
-        int n = v->list->GetFocusedItem();
-        dd.push_back(ftk->get_function(n == -1 ? 0 : n)->xname);
-    }
+    //if (dd.empty() && v->list->GetItemCount() > 0) {
+    //    int n = v->list->GetFocusedItem();
+    //    dd.push_back(ftk->get_function(n == -1 ? 0 : n)->xname);
+    //}
     return dd;
 }
 
@@ -973,4 +1122,43 @@ void SideBar::OnVarFocusChanged(wxListEvent&)
     update_var_inf();
 }
 
+void SideBar::make_same_func_par(string const& p, bool checked)
+{
+    string varname = "_" + p;
+    string cmd;
+    if (checked) {
+
+        // find value
+        fp value = 0;
+        bool found = false;
+        if (active_function != -1) {
+            Function const* f = ftk->get_function(active_function);
+            found = f->get_param_value_safe(p, value);
+        }
+        vector<Function*> const& ff = ftk->get_functions();
+        for (vector<Function*>::const_iterator i = ff.begin(); 
+                                                 i != ff.end() && !found; ++i)
+            found = (*i)->get_param_value_safe(p, value);
+        if (!found)
+            return;
+
+        cmd += "$" + varname + " = ~" + S(value);
+        for (int i = 0; i < ftk->get_ds_count(); ++i)
+            if (ftk->get_sum(i)->get_ff_names().size() > 0)
+                cmd += "; @" + S(i) + ".F." + p + " = $" + varname;
+    }
+    else {
+        int nr = ftk->find_variable_nr(varname);
+        if (nr == -1)
+            return;
+        fp value = ftk->get_variable(nr)->get_value();
+        for (int i = 0; i < ftk->get_ds_count(); ++i)
+            if (ftk->get_sum(i)->get_ff_names().size() > 0)
+                cmd += "@" + S(i) + ".F." + p + " = " + S(value) + "; ";
+        // the variable was auto-deleted.
+        //cmd += "delete " + varname;
+    }
+    ftk->exec(cmd);
+}
+
 
