@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License version 2
-// $Id: sidebar.h 326 2007-08-01 05:54:03Z wojdyr $
+// $Id: sidebar.h 424 2008-04-15 11:08:42Z wojdyr $
 
 #ifndef FITYK__WX_SIDEBAR__H__
 #define FITYK__WX_SIDEBAR__H__
@@ -43,14 +43,23 @@ public:
     void OnFuncButtonEdit (wxCommandEvent& event);
     void OnFuncButtonChType (wxCommandEvent& event);
     void OnFuncButtonCol (wxCommandEvent& event);
+    void OnFuncButtonHwhm (wxCommandEvent& event) 
+                            { make_same_func_par("hwhm", event.IsChecked()); }
+    void OnFuncButtonShape (wxCommandEvent& event)
+                            { make_same_func_par("shape", event.IsChecked()); }
     void OnVarButtonNew (wxCommandEvent& event);
     void OnVarButtonDel (wxCommandEvent&) { delete_selected_items(); }
     void OnVarButtonEdit (wxCommandEvent& event);
     void OnFuncFilterChanged (wxCommandEvent& event);
-    void OnDataFocusChanged(wxListEvent &) { DataFocusChanged(); }
-    void DataFocusChanged();
+    void OnDataFocusChanged(wxListEvent &);
+    void OnDataSelectionChanged(wxListEvent &);
+    void update_data_buttons();
+    void update_func_buttons();
+    void update_var_buttons();
     void OnFuncFocusChanged(wxListEvent &event);
+    void OnFuncSelectionChanged(wxListEvent&) { update_func_buttons(); }
     void OnVarFocusChanged(wxListEvent &event);
+    void OnVarSelectionChanged(wxListEvent&) { update_var_buttons(); }
     void read_settings(wxConfigBase *cf);
     void save_settings(wxConfigBase *cf) const;
     void update_lists(bool nondata_changed=true);
@@ -58,6 +67,9 @@ public:
     int get_focused_data() const;
     int get_active_function() const { return active_function; }
     int get_focused_var() const;
+    std::vector<int> get_ordered_dataset_numbers();
+    std::string get_plot_in_datasets();
+    std::vector<int> get_selected_ds_indices();
     //bool is_func_selected(int n) const;
     int set_selection(int page) { return nb->SetSelection(page); }
     void activate_function(int n);
@@ -72,7 +84,7 @@ public:
     void delete_selected_items();
     void draw_function_draft(FancyRealCtrl const* frc) const;
     void change_bp_parameter_value(int idx, double value);
-    std::string get_datasets_str();
+    std::string get_datasets_for_plot();
 private:
     wxNotebook *nb;
     wxPanel *data_page, *func_page, *var_page, *bottom_panel;
@@ -100,6 +112,7 @@ private:
     void on_changing_frc_value(FancyRealCtrl const* frc);
     void on_changed_frc_value(FancyRealCtrl const* frc);
     void on_toggled_frc_lock(FancyRealCtrl const* frc);
+    void make_same_func_par(std::string const& p, bool checked);
 
     DECLARE_EVENT_TABLE()
 };
